package com.persagy.apm.energy.report.centermiddleware.client;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.centermiddleware.client.fallback.CenterMiddlewareWebClientFallbackFactory;
import com.persagy.apm.energy.report.centermiddleware.model.dto.BatchQueryPeriodDataDTO;
import com.persagy.apm.energy.report.centermiddleware.model.dto.QueryEnergyDataDTO;
import com.persagy.apm.energy.report.centermiddleware.model.dto.QueryItemInfoDTO;
import com.persagy.apm.energy.report.centermiddleware.model.dto.QueryObjectInfoDTO;
import com.persagy.apm.energy.report.centermiddleware.model.vo.EnergyDataInfoVO;
import com.persagy.apm.energy.report.centermiddleware.model.vo.EnergyItemInfoVO;
import com.persagy.apm.energy.report.centermiddleware.model.vo.ObjectInfoVO;
import com.persagy.apm.energy.report.centermiddleware.model.vo.PeriodDataInfoVO;
import com.persagy.apm.energy.report.common.FeignClientNameConstants;
import com.persagy.apm.energy.report.common.FeignConfig;
import feign.Headers;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;

/**
 * 中间件feign客户端
 **/
@FeignClient(name = FeignClientNameConstants.CENTER_MIDDLEWARE, fallbackFactory = CenterMiddlewareWebClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface CenterMiddlewareWebClient {

    /**
     * 获取分项的能耗数据
     *
     * @param queryEnergyDataDTO 查询条件
     * @return 项目下的建筑信息
     * @author wch
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/ems/queryEnergyData")
    @Headers({"Content-Type: application/json"})
    PoemsFeignResponse<EnergyDataInfoVO> getItemEnergyData(QueryEnergyDataDTO queryEnergyDataDTO);

    /**
     * 获取分项名称
     *
     * @param queryItemInfoDTO 查询条件
     * @return 项目下的建筑信息
     * @author wch
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/ems/queryItemName")
    @Headers({"Content-Type: application/json"})
    PoemsFeignResponse<EnergyItemInfoVO> getItemNameInfo(QueryItemInfoDTO queryItemInfoDTO);

    /**
     * 查询中台对应对象信息
     *
     * @param queryObjectInfoDTO 查询条件
     * @return
     */
    @PostMapping("/object/query")
    @Headers({"Content-Type: application/json"})
    PoemsFeignResponse<ObjectInfoVO> getObjectInfo(QueryObjectInfoDTO queryObjectInfoDTO);

    /**
     * 批量查询分精度数据
     *
     * @param batchQueryPeriodDataDTO 查询条件
     * @return
     */
    @PostMapping("/hisdata/batchQueryPeriodData")
    @Headers({"Content-Type: application/json"})
    PoemsFeignResponse<PoemsFeignResponse<PeriodDataInfoVO>> getPeriodDataInfo(BatchQueryPeriodDataDTO batchQueryPeriodDataDTO);

}
