package com.persagy.apm.energy.report.common.service;

import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.*;
import io.swagger.annotations.Api;

import java.util.Date;
import java.util.List;

@Api(value = "公共service接口")
public interface CommonService {

    /**
     * @param pjIdList 项目id列表
     * @return
     */
    List<String> getPjNameByPjId(List<String> pjIdList);

    /**
     * 根据配置条目信息获取能耗管理平台运维数据
     *
     * @param statisticItems        条目信息
     * @param queryPlatformParamDTO 查询条件对象
     * @return 能耗管理平台运维数据
     */
    Platform getPlatform(StatisticItems statisticItems, QueryPlatformParamDTO queryPlatformParamDTO);

    /**
     * 获取酒店项目报警处理信息
     *
     * @param projectId  项目id
     * @param reportDate 报告月
     * @return 酒店项目报警处理信息
     */
    HotelProjectAlarmDealInfo getHotelProjectAlarmDealInfo(String projectId, Date reportDate);

    /**
     * 获取酒店项目环境指标条目信息
     *
     * @param projectId     项目id
     * @param reportDate    报告月
     * @param projectIdList 当前报告类型下的所有项目id列表
     * @param function      function信息
     * @return 酒店项目环境指标条目信息
     */
    HotelProjectKpiItem getHotelProjectKpiItem(String projectId, Date reportDate, List<String> projectIdList, Function function);

    /**
     * 获取酒店平台数据质量信息
     *
     * @param userId     用户id
     * @param projectId  项目id
     * @param reportDate 报告月
     * @param function   function信息
     * @return 酒店项目环境指标条目信息
     */
    HotelProjectMaintenanceGraph getHotelProjectMaintenanceGraph(String userId, String projectId, Date reportDate, Function function);

    /**
     * 获取酒店项目维护信息
     *
     * @param userId        用户id
     * @param projectId     项目id
     * @param reportDate    报告月
     * @param functionList  function信息列表
     * @param projectIdList 当前报告类型下的所有项目id列表
     * @return 酒店项目环境指标条目信息
     */
    HotelProjectMaintenanceInfo getHotelProjectMaintenanceInfo(String userId, String projectId, Date reportDate, List<Function> functionList, List<String> projectIdList);

    /**
     * 获取酒店项目节能工单信息
     *
     * @param projectId  项目id
     * @param reportDate 报告月
     * @return
     */
    HotelProjectEnergyWorkerOrderInfo getHotelProjectEnergyWorkerOrderInfo(String projectId, Date reportDate);

    /**
     * 获取项目变损和总支不平衡率信息
     *
     * @param projectId  项目id
     * @param reportDate 报告月
     * @param userId     用户id
     * @return
     */
    HotelProjectMaintenanceBaseInfo getHotelProjectMaintenanceBaseInfo(String projectId, Date reportDate, String userId);

    /**
     * 获取酒店项目冷站效率
     *
     * @param projectId  项目id
     * @param reportDate 报告月
     * @param pjIdList   项目id列表
     * @return
     */
    HotelProjectChillerPlantEfficiency getHotelProjectChillerPlantEfficiency(String projectId, Date reportDate, List<String> pjIdList);

}
