package com.persagy.apm.energy.report.common.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.service.PlatformService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.common.vo.PlatformAreaInfoVO;
import com.persagy.apm.energy.report.environment.model.dto.QueryEnvListByParamDTO;
import com.persagy.apm.energy.report.environment.model.vo.EnvAvgRateVO;
import com.persagy.apm.energy.report.environment.service.impl.EnvironmentWebClientWrapper;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TreeMap;
import java.util.stream.Collectors;

@Service("TemperatureRateEnvironmentWebService")
@Slf4j
public class EnvironmentPlatformServiceImpl implements PlatformService {

    @Autowired
    private EnvironmentWebClientWrapper environmentWebClientWrapper;
    @Resource
    private CommonService commonService;

    @Override
    public List<Double> getPjPlatformRate(QueryPlatformParamDTO queryPlatformParamDTO) {
        if (null == queryPlatformParamDTO || CollectionUtils.isEmpty(queryPlatformParamDTO.getProjectIdList()) || null == queryPlatformParamDTO.getReportDate()) {
            return new ArrayList<>();
        }
        String pjId = queryPlatformParamDTO.getProjectIdList().get(0);
        TreeMap<Date, Double> date_rateMap = new TreeMap<>();
        Date reportDate = queryPlatformParamDTO.getReportDate();
        Date startDate = DateUtils.getFirstDayOfYear(reportDate);
        while (startDate.before(DateUtils.getMonthOff(reportDate, 1))) {
            date_rateMap.put(startDate, getEnvAvgRate(pjId, startDate));
            startDate = DateUtils.getMonthOff(startDate, 1);
        }
        List<Double> monthlySummaries = date_rateMap.values().stream().collect(Collectors.toList());
        return monthlySummaries;
    }

    private Double getEnvAvgRate(String pjId, Date reportDate) {
        Double avgRate = null;
        try {
            QueryEnvListByParamDTO paramDTO = new QueryEnvListByParamDTO();
            paramDTO.setProjectId(pjId);
            paramDTO.setEnvTypeId(DataConstants.ENV_TYPE_ID);
            paramDTO.setEnvParentTypeId(DataConstants.ENV_PARENT_TYPE_ID);
            paramDTO.setStartTime(DateUtils.date2Str(reportDate, DateUtils.SDF_SECOND));
            paramDTO.setEndTime(DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDF_SECOND));
            PoemsFeignResponse<EnvAvgRateVO> response = environmentWebClientWrapper.listEnvTypeAvgRateByParam(paramDTO);
            List<EnvAvgRateVO> envAvgRateVOList = response.getContent();
            if (!CollectionUtils.isEmpty(envAvgRateVOList)) {
                EnvAvgRateVO envAvgRateVO = envAvgRateVOList.get(0);
                if (null != envAvgRateVO) {
                    return envAvgRateVO.getAvgNum();
                }
            }
        } catch (Exception e) {
            log.error("获取环境健康服务温度达标率数据异常", e);
        }
        return avgRate;
    }

    @Override
    public PlatformAreaInfoVO getAreaPlatformRate(StatisticItems statisticItems, QueryPlatformParamDTO queryPlatformParamDTO) {
        if (null == queryPlatformParamDTO || CollectionUtils.isEmpty(queryPlatformParamDTO.getProjectIdList()) || null == queryPlatformParamDTO.getReportDate()) {
            return new PlatformAreaInfoVO();
        }
        // 合格标准
        String standardValueStr = statisticItems.getStandardValue();
        Double standardValue = DataUtils.parseDouble(standardValueStr);
        // 校验规则
        String qualifyFormula = statisticItems.getQualifyFormula();
        // 达标数量
        Integer satisfiedCount = 0;
        // 未达标数量
        Integer unsatisfiedCount = 0;
        // 未达标项目id列表
        List<String> unsatisfiedPjIdList = new ArrayList<>();
        for (String pjId : queryPlatformParamDTO.getProjectIdList()) {
            Double completeRate = getEnvAvgRate(pjId, queryPlatformParamDTO.getReportDate());
            if (null != completeRate && null != standardValue) {
                if (DataUtils.getQualifyResult(standardValue, qualifyFormula, completeRate) == 0.0) {
                    unsatisfiedCount++;
                    unsatisfiedPjIdList.add(pjId);
                } else {
                    satisfiedCount++;
                }
            }
        }
        // 根据未达标项目id列表获取不达标项目名称
        List<String> pjNameList = commonService.getPjNameByPjId(unsatisfiedPjIdList);
        PlatformAreaInfoVO platformAreaInfoVO = new PlatformAreaInfoVO();
        platformAreaInfoVO.setSatisfiedCount(satisfiedCount);
        platformAreaInfoVO.setUnsatisfiedCount(unsatisfiedCount);
        platformAreaInfoVO.setUnsatisfiedProjects(pjNameList);
        return platformAreaInfoVO;
    }
}
