package com.persagy.apm.energy.report.common.utils;

import org.apache.http.HttpEntity;
import org.apache.http.HttpVersion;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.util.EntityUtils;

import java.nio.charset.Charset;

public class HttpUtils {
    private final static PoolingHttpClientConnectionManager CONNECTION_MANAGER = new PoolingHttpClientConnectionManager();
    private final static CloseableHttpClient HTTP_CLIENT;
    private static final String UTF8 = "utf-8";

    static {
        // 设置整个连接池最大连接数 根据自己的场景决定
        CONNECTION_MANAGER.setMaxTotal(200);
        // 每个路由(网站)的最大连接数
        CONNECTION_MANAGER.setDefaultMaxPerRoute(100);
        RequestConfig requestConfig = RequestConfig.custom()
                // 连接建立时间，三次握手完成时间
                .setConnectTimeout(5000)
                // 数据传输过程中数据包之间间隔的最大时间
                .setSocketTimeout(5000)
                // httpClient使用连接池来管理连接，这个时间就是从连接池获取连接的超时时间
                .setConnectionRequestTimeout(6000).build();
        HTTP_CLIENT = HttpClients.custom().setConnectionManager(CONNECTION_MANAGER)
                .setDefaultRequestConfig(requestConfig).build();
    }

    public static String postJson(String url, String json) throws Exception {
        HttpPost httpPost = new HttpPost(url);
        //使用http1.0协议
        httpPost.setProtocolVersion(HttpVersion.HTTP_1_0);
        httpPost.addHeader("Content-type", "application/json");
        httpPost.setEntity(new StringEntity(json, Charset.forName("UTF-8")));
        CloseableHttpResponse response = HTTP_CLIENT.execute(httpPost);
        int statusCode = response.getStatusLine().getStatusCode();
        if (statusCode != 200 && statusCode != 201) {
            httpPost.abort();
            response.close();
            throw new RuntimeException("HttpClient,error status code :" + statusCode);
        }
        HttpEntity entity = response.getEntity();
        String result = null;
        if (entity != null) {
            result = EntityUtils.toString(entity, "utf-8");
        }
        EntityUtils.consume(entity);
        response.close();
        return result;
    }

}
