package com.persagy.apm.energy.report.environment.client;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.FeignClientNameConstants;
import com.persagy.apm.energy.report.common.FeignConfig;
import com.persagy.apm.energy.report.environment.client.fallback.EnvironmentWebClientFallbackFactory;
import com.persagy.apm.energy.report.environment.model.vo.EnvAvgRateVO;
import com.persagy.apm.energy.report.environment.model.vo.EnvRateInfoVO;
import feign.Headers;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 环境健康feign客户端
 **/
@FeignClient(name = FeignClientNameConstants.ENVIRONMENT, fallbackFactory = EnvironmentWebClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface EnvironmentWebClient {
    /**
     * 根据条件查询环境温度达标率
     *
     * @param queryEnvListByParamDTO 查询条件
     * @return 平均达标率
     * @author wch
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/GetEnvTypeRate")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<EnvAvgRateVO> listEnvTypeAvgRateByParam(@RequestParam("jsonString") String queryEnvListByParamDTO);

    /**
     * 根据条件查询环境对应指标的达标率信息
     *
     * @param queryEnvListByParamDTO 查询条件
     * @return 环境对应指标的达标率信息
     * @author wch
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/BatchQueryEnvTypeRate")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<EnvRateInfoVO> listEnvTypeRateInfoByParam(@RequestParam("jsonString") String queryEnvListByParamDTO);

}
