package com.persagy.apm.energy.report.environment.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.common.dto.QueryAreaPlatformParamDTO;
import com.persagy.apm.energy.report.common.dto.QueryPjPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.environment.model.dto.BatchQueryEnvListByParamDTO;
import com.persagy.apm.energy.report.environment.model.dto.QueryEnvListByParamDTO;
import com.persagy.apm.energy.report.environment.model.vo.EnvAvgRateVO;
import com.persagy.apm.energy.report.environment.model.vo.EnvRateInfoVO;
import com.persagy.apm.energy.report.environment.service.IEnvironmentWebService;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.DateGraphItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectKpiItem;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class IEnvironmentWebServiceImpl implements IEnvironmentWebService {

    @Autowired
    private EnvironmentWebClientWrapper environmentWebClientWrapper;
    @Resource
    private CommonService commonService;

    @Override
    public void getPjTempRate(QueryPjPlatformParamDTO queryPjPlatformParamDTO, Platform platform) {
        TreeMap<Date, Double> date_rateMap = new TreeMap<>();
        Date reportDate = queryPjPlatformParamDTO.getReportDate();
        Date startDate = DateUtils.getFirstDayOfYear(reportDate);
        while (startDate.before(DateUtils.getMonthOff(reportDate, 1))) {
            date_rateMap.put(startDate, getEnvAvgRate(queryPjPlatformParamDTO.getProjectId(), startDate));
            startDate = DateUtils.getMonthOff(startDate, 1);
        }
        List<Double> monthlySummaries = date_rateMap.values().stream().collect(Collectors.toList());
        platform.setMonthlySummaries(monthlySummaries);
    }

    private Double getEnvAvgRate(String pjId, Date reportDate) {
        Double avgRate = null;
        try {
            QueryEnvListByParamDTO paramDTO = new QueryEnvListByParamDTO();
            paramDTO.setProjectId(pjId);
            paramDTO.setEnvTypeId(DataConstants.ENV_TYPE_ID);
            paramDTO.setEnvParentTypeId(DataConstants.ENV_PARENT_TYPE_ID);
            paramDTO.setStartTime(DateUtils.date2Str(reportDate, DateUtils.SDF_SECOND));
            paramDTO.setEndTime(DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDF_SECOND));
            PoemsFeignResponse<EnvAvgRateVO> response = environmentWebClientWrapper.listEnvTypeAvgRateByParam(paramDTO);
            List<EnvAvgRateVO> envAvgRateVOList = response.getContent();
            if (!CollectionUtils.isEmpty(envAvgRateVOList)) {
                EnvAvgRateVO envAvgRateVO = envAvgRateVOList.get(0);
                if (null != envAvgRateVO) {
                    return envAvgRateVO.getAvgNum();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return avgRate;
    }

    @Override
    public void getAreaTempRate(QueryAreaPlatformParamDTO queryAreaPlatformParamDTO, Platform platform) {
        // 合格标准
        Double standardValue = platform.getStandardValue();
        // 校验规则
        String qualifyFormula = platform.getQualifyFormula();
        // 达标数量
        Integer satisfiedCount = 0;
        // 未达标数量
        Integer unsatisfiedCount = 0;
        // 未达标项目id列表
        List<String> unsatisfiedPjIdList = new ArrayList<>();
        for (String pjId : queryAreaPlatformParamDTO.getProjectIdList()) {
            Double completeRate = getEnvAvgRate(pjId, queryAreaPlatformParamDTO.getReportDate());
            if (null != completeRate && null != standardValue) {
                if (DataUtils.getQualifyResult(standardValue, qualifyFormula, completeRate) == 0.0) {
                    unsatisfiedCount++;
                    unsatisfiedPjIdList.add(pjId);
                } else {
                    satisfiedCount++;
                }
            }
        }
        // 根据未达标项目id列表获取不达标项目名称
        List<String> pjNameList = commonService.getPjNameByPjId(unsatisfiedPjIdList);
        platform.setSatisfiedCount(satisfiedCount);
        platform.setUnsatisfiedCount(unsatisfiedCount);
        platform.setUnsatisfiedProjects(pjNameList);
    }

    @Override
    public List<HotelProjectKpiItem> getPjEnvRateInfoMap(BatchQueryEnvListByParamDTO batchQueryEnvListByParamDTO) {
        List<HotelProjectKpiItem> resultList = new ArrayList<>();
        try {
            PoemsFeignResponse<EnvRateInfoVO> response = environmentWebClientWrapper.listEnvTypeRateInfoByParam(batchQueryEnvListByParamDTO);
            List<EnvRateInfoVO> envRateInfoVOList = response.getContent();
            if (!CollectionUtils.isEmpty(envRateInfoVOList)) {
                for (EnvRateInfoVO vo : envRateInfoVOList) {
                    HotelProjectKpiItem hotelProjectKpiItem = new HotelProjectKpiItem();
                    hotelProjectKpiItem.setControlRates(vo.getAvgNum());
                    List<DateGraphItemVO> detailInfo = new ArrayList<>();
                    hotelProjectKpiItem.setDetailInfo(detailInfo);
                    hotelProjectKpiItem.setProjectId(vo.getProjectId());
                    hotelProjectKpiItem.setProjectNumWithinClimateZone(batchQueryEnvListByParamDTO.getProjectIdList().size());
                    Map dataInfoMap = vo.getDayDataList();
                    if(null == dataInfoMap){
                        dataInfoMap = new HashMap();
                    }
                    TreeMap<String, Double> dataMap =  new TreeMap<>(dataInfoMap);
                    if (!CollectionUtils.isEmpty(dataMap)) {
                        for (Map.Entry<String, Double> entry : dataMap.entrySet()) {
                            DateGraphItemVO dateGraphItemVO = new DateGraphItemVO();
                            dateGraphItemVO.setDate(DateUtils.str2Date(entry.getKey(),DateUtils.SDF_SECOND));
                            dateGraphItemVO.setValue(entry.getValue());
                            detailInfo.add(dateGraphItemVO);
                        }
                    }
                    resultList.add(hotelProjectKpiItem);
                }
            }
        } catch (Exception e) {
            log.error("获取环境健康服务" + batchQueryEnvListByParamDTO.getEnvTypeId() + "类型达标率数据异常", e);
        }
        return resultList;
    }
}
