package com.persagy.apm.energy.report.equiprunmanager.service.impl;

import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.equiprunmanager.model.dto.QueryEnergySaveWorkerParamDTO;
import com.persagy.apm.energy.report.equiprunmanager.model.vo.EnergySaveWorkInfoVO;
import com.persagy.apm.energy.report.equiprunmanager.service.IEquipRunManagerService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class IEquipRunManagerServiceImpl implements IEquipRunManagerService {

    @Autowired
    EquipRunManagerWebClientWrapper equipRunManagerWebClientWrapper;

    @Override
    public Map<String, Double> getEnergySaveWorderMap(List<String> workOrderIdList) {
        if (CollectionUtils.isEmpty(workOrderIdList)) {
            log.info("获取节能工单信息入参为空，故不进行查询");
            return new HashMap<>();
        }
        Map<String, Double> resultMap = new HashMap<>();
        try {
            QueryEnergySaveWorkerParamDTO queryEnergySaveWorkerParamDTO = new QueryEnergySaveWorkerParamDTO();
            queryEnergySaveWorkerParamDTO.setWorkIds(workOrderIdList);
            PoemsFeignResponse<EnergySaveWorkInfoVO> response = equipRunManagerWebClientWrapper.listEnergySaveWorkByParam(queryEnergySaveWorkerParamDTO);
            List<EnergySaveWorkInfoVO> energySaveWorkInfoVOList = response.getContent();
            if (!CollectionUtils.isEmpty(energySaveWorkInfoVOList)) {
                for (EnergySaveWorkInfoVO vo : energySaveWorkInfoVOList) {
                    resultMap.put(vo.getWorkId(), vo.getEnergyData());
                }
            }
        } catch (Exception e) {
            log.error("获取高效运行服务节能工单节能量异常", e);
        }
        return resultMap;
    }
}
