package com.persagy.apm.energy.report.manualfilling.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.manualfilling.model.dto.QueryEnergyItemDataDTO;
import com.persagy.apm.energy.report.manualfilling.model.dto.SaveEnergyItemDataDTO;
import com.persagy.apm.energy.report.manualfilling.model.vo.EnergyItemInfoVo;
import com.persagy.apm.energy.report.manualfilling.service.IEnergyItemDataService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * 与数据录入对接控制层
 */
@Api(tags = "数据录入对接")
@Validated
@RestController
@RequestMapping("manualFilling")
public class ManualFillingController {
    @Autowired
    private IEnergyItemDataService iEnergyItemDataService;

    @ApiOperation(value = "查询报告能耗分项列表")
    @PostMapping("/query")
    public CommonResult<PageList<EnergyItemInfoVo>> queryEnergyItemDataList(
            @Valid @RequestBody QueryEnergyItemDataDTO queryEnergyItemDataDTO) {
        List<EnergyItemInfoVo> energyItemInfoVoList = iEnergyItemDataService.queryEnergyItemDataList(queryEnergyItemDataDTO);
        return ResultHelper.multi(energyItemInfoVoList);
    }

    @ApiOperation(value = "保存能耗分项数值")
    @PostMapping("/save")
    public CommonResult<ApmResponseUpsertVO> updateReportBusinessDetail(
            @Valid @RequestBody SaveEnergyItemDataDTO saveEnergyItemDataDTO) {
        iEnergyItemDataService.saveEnergyItemData(saveEnergyItemDataDTO);
        return ResultHelper.single(new ApmResponseUpsertVO());
    }

}

