package com.persagy.apm.energy.report.manualfilling.service.impl;

import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.manualfilling.model.dto.QueryEnergyItemDataDTO;
import com.persagy.apm.energy.report.manualfilling.model.dto.SaveEnergyItemDataDTO;
import com.persagy.apm.energy.report.manualfilling.model.vo.EnergyItemDataVo;
import com.persagy.apm.energy.report.manualfilling.model.vo.EnergyItemInfoVo;
import com.persagy.apm.energy.report.manualfilling.service.IEnergyItemDataService;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.function.model.dto.QueryFunctionDTO;
import com.persagy.apm.energy.report.monthly.config.function.service.IFunctionService;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BuildingTypeEnum;
import com.persagy.apm.energy.report.monthly.functionvalue.model.FunctionValue;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.BatchDeleteFunctionValueDTO;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.QueryFunctionValueDTO;
import com.persagy.apm.energy.report.monthly.functionvalue.service.IFunctionValueService;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
@Slf4j
public class IEnergyItemDataServiceImpl implements IEnergyItemDataService {

    @Autowired
    private IFunctionService functionService;

    @Autowired
    private ISaasWebService iSaasWebService;

    @Autowired
    private IFunctionValueService functionValueService;

    @Value("${group.code}")
    private String groupCode;

    @Override
    public List<EnergyItemInfoVo> queryEnergyItemDataList(QueryEnergyItemDataDTO queryEnergyItemDataDTO) {
        try {
            PoemsContext.setContext(DataConstants.USER_ID, DataConstants.LOGIN_DEVICE, DataConstants.PD);
            ReportProjectVO projectVO = iSaasWebService.getReportProjectInfo(queryEnergyItemDataDTO.getProjectId());
            if (null == projectVO || StringUtils.isEmpty(projectVO.getFunctionType())) {
                return new ArrayList<>();
            }
            String buildingType = getParentFuncType(projectVO.getFunctionType());
            if (null == buildingType) {
                return new ArrayList<>();
            }
            QueryFunctionDTO queryFunctionDTO = new QueryFunctionDTO();
            queryFunctionDTO.setGroupCode(groupCode);
            if (!StringUtils.isEmpty(queryEnergyItemDataDTO.getGroupCode())) {
                queryFunctionDTO.setGroupCode(queryEnergyItemDataDTO.getGroupCode());
            }
            queryFunctionDTO.setEnergyType(queryEnergyItemDataDTO.getEnergyType());
            queryFunctionDTO.setFunctionType(queryEnergyItemDataDTO.getFunctionType());
            queryFunctionDTO.setBuildingType(buildingType);
            // 查询配置的分项信息列表
            List<Function> functionList = functionService.queryFunctionList(queryFunctionDTO);
            if (CollectionUtils.isEmpty(functionList)) {
                return new ArrayList<>();
            }
            // 配置信息点id列表
            List<String> functionIdList = new ArrayList<>();
            for (Function function : functionList) {
                functionIdList.add(function.getId());
            }
            QueryFunctionValueDTO queryFunctionValueDTO = new QueryFunctionValueDTO();
            queryFunctionValueDTO.setProjectId(queryEnergyItemDataDTO.getProjectId());
            queryFunctionValueDTO.setFunctionIdList(functionIdList);
            queryFunctionValueDTO.setStartTime(DateUtils.str2Date(queryEnergyItemDataDTO.getStartDate(), DateUtils.SDF_SECOND));
            queryFunctionValueDTO.setEndTime(DateUtils.str2Date(queryEnergyItemDataDTO.getEndDate(), DateUtils.SDF_SECOND));
            // 查询配置分项某时间段内的数据
            List<FunctionValue> functionValueList = functionValueService.queryFunctionValueList(queryFunctionValueDTO);
            // {信息点id:{数据时间：数据}}
            Map<String, List<EnergyItemDataVo>> funcId_dateAndDataMap = new HashMap<>();
            if (CollectionUtils.isNotEmpty(functionValueList)) {
                for (FunctionValue functionValue : functionValueList) {
                    List<EnergyItemDataVo> energyItemDataVoList = funcId_dateAndDataMap.get(functionValue.getFunctionId());
                    if (null == energyItemDataVoList) {
                        energyItemDataVoList = new ArrayList<>();
                    }
                    EnergyItemDataVo energyItemDataVo = new EnergyItemDataVo();
                    energyItemDataVo.setId(functionValue.getId());
                    energyItemDataVo.setTime(DateUtils.date2Str(functionValue.getDataTime(), DateUtils.SDF_SECOND));
                    energyItemDataVo.setValue(functionValue.getValue());
                    energyItemDataVoList.add(energyItemDataVo);
                    funcId_dateAndDataMap.put(functionValue.getFunctionId(), energyItemDataVoList);
                }
            }
            List<EnergyItemInfoVo> resultList = new ArrayList<>();
            for (Function function : functionList) {
                EnergyItemInfoVo energyItemInfoVo = new EnergyItemInfoVo();
                energyItemInfoVo.setItemId(function.getItemId());
                energyItemInfoVo.setFunctionId(function.getId());
                energyItemInfoVo.setModelCode(function.getModelCode());
                energyItemInfoVo.setDataList(funcId_dateAndDataMap.get(function.getId()) == null ? new ArrayList<>() : funcId_dateAndDataMap.get(function.getId()));
                resultList.add(energyItemInfoVo);
            }
            return resultList;
        } catch (Exception e) {
            log.error("查询分项数据异常", e);
        }
        return Lists.newArrayList();
    }

    @Override
    public void saveEnergyItemData(SaveEnergyItemDataDTO saveEnergyItemDataDTO) {
        try {
            PoemsContext.setContext(DataConstants.USER_ID, DataConstants.LOGIN_DEVICE, DataConstants.PD);
            ReportProjectVO projectVO = iSaasWebService.getReportProjectInfo(saveEnergyItemDataDTO.getProjectId());
            if (null == projectVO || StringUtils.isEmpty(projectVO.getFunctionType())) {
                throw new Exception("获取不到项目信息");
            }
            String buildingType = getParentFuncType(projectVO.getFunctionType());
            if (null == buildingType) {
                throw new Exception("获取不到项目功能类型信息");
            }
            saveEnergyItemDataDTO.setBuildingType(buildingType);
            List<EnergyItemInfoVo> energyItemInfoVoList = saveEnergyItemDataDTO.getDataList();
            Map<String, String> itemId_functionIdMap = getFunctionIdMap(saveEnergyItemDataDTO);
            for (EnergyItemInfoVo energyItemInfoVo : energyItemInfoVoList) {
                // 配置信息点id
                String functionId = itemId_functionIdMap.get(energyItemInfoVo.getItemId());
                List<EnergyItemDataVo> energyItemDataVos = energyItemInfoVo.getDataList();
                List<Date> deleteDates = new ArrayList<>();
                List<FunctionValue> functionValues = new ArrayList<>();
                for (EnergyItemDataVo energyItemDataVo : energyItemDataVos) {
                    FunctionValue functionValue = new FunctionValue();
                    functionValue.setProjectId(saveEnergyItemDataDTO.getProjectId());
                    functionValue.setFunctionId(functionId);
                    functionValue.setDataTime(DateUtils.str2Date(energyItemDataVo.getTime(), DateUtils.SDF_SECOND));
                    functionValue.setValue(energyItemDataVo.getValue());
                    deleteDates.add(functionValue.getDataTime());
                    functionValues.add(functionValue);
                }
                BatchDeleteFunctionValueDTO batchDeleteFunctionValueDTO = new BatchDeleteFunctionValueDTO();
                batchDeleteFunctionValueDTO.setProjectId(saveEnergyItemDataDTO.getProjectId());
                batchDeleteFunctionValueDTO.setFunctionId(functionId);
                batchDeleteFunctionValueDTO.setDataTimeList(deleteDates);
                functionValueService.batchDeleteFunctionValue(batchDeleteFunctionValueDTO);
                functionValueService.batchCreateFunctionValue(functionValues);
            }
        } catch (Exception e) {
            log.error("保存分项录入数据异常", e);
        }
    }

    private Map<String, String> getFunctionIdMap(SaveEnergyItemDataDTO saveEnergyItemDataDTO) {
        QueryFunctionDTO queryFunctionDTO = new QueryFunctionDTO();
        queryFunctionDTO.setEnergyType(saveEnergyItemDataDTO.getEnergyType());
        queryFunctionDTO.setFunctionType(saveEnergyItemDataDTO.getFunctionType());
        queryFunctionDTO.setBuildingType(saveEnergyItemDataDTO.getBuildingType());
        List<Function> functionList = functionService.queryFunctionList(queryFunctionDTO);
        if (CollectionUtils.isEmpty(functionList)) {
            return new HashMap<>();
        }
        Map<String, String> resultMap = new HashMap<>();
        for (Function function : functionList) {
            resultMap.put(function.getItemId(), function.getId());
        }
        return resultMap;
    }

    private String getParentFuncType(String buildingType) {
        for (BuildingTypeEnum typeEnum : BuildingTypeEnum.values()) {
            List<String> buildingTypes = iSaasWebService.getRelatedBuildingTypes(typeEnum.getType());
            if (buildingTypes.contains(buildingType)) {
                return typeEnum.getType();
            }
        }
        return null;
    }

}
