package com.persagy.apm.energy.report.monthly.config.function.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.monthly.config.function.model.ConvertFunctionTool;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.function.model.dto.*;
import com.persagy.apm.energy.report.monthly.config.function.model.vo.ResponseFunctionItemVO;
import com.persagy.apm.energy.report.monthly.config.function.model.vo.ResponseFunctionListItemVO;
import com.persagy.apm.energy.report.monthly.config.function.service.IFunctionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 能耗报告信息点控制层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:05:33
 */
@ApiIgnore
@Api(tags = "能耗报告信息点", hidden = true)
@Validated
@RestController
@RequestMapping("functions")
public class FunctionController {
    @Autowired
    private IFunctionService functionService;

    @ApiOperation(value = "创建能耗报告信息点")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createFunction(
            @Valid @RequestBody AddFunctionDTO addFunctionDTO) {
        String id = functionService.createFunction(addFunctionDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "能耗报告信息点详情")
    @PostMapping("/get")
    public CommonResult<ResponseFunctionItemVO> queryFunctionDetail(
            @Valid @RequestBody QueryFunctionDetailDTO queryFunctionDetailDTO) {
        Function function = functionService.queryFunctionDetail(queryFunctionDetailDTO.getId());
        ResponseFunctionItemVO responseFunctionItemVO = ConvertFunctionTool.INSTANCE.convert2ResponseItemDTO(function);
        return ResultHelper.single(responseFunctionItemVO);
    }


    @ApiOperation(value = "更新能耗报告信息点")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateFunction(
            @Valid @RequestBody UpdateFunctionDTO updateFunctionDTO) {
        functionService.updateFunction(updateFunctionDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateFunctionDTO.getId()));
    }


    @ApiOperation(value = "删除能耗报告信息点")
    @PostMapping("/delete")
    public CommonResult deleteFunction(
            @RequestBody @Valid DeleteFunctionDTO deleteFunctionDTO) {
        functionService.deleteFunction(deleteFunctionDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "能耗报告信息点列表")
    @PostMapping("/query")
    public CommonResult<PageList<ResponseFunctionListItemVO>> queryFunction(
            @Valid @RequestBody QueryFunctionDTO queryFunctionDTO) {
        List<Function> functionList = functionService.queryFunctionList(queryFunctionDTO);
        List<ResponseFunctionListItemVO> resultList = ConvertFunctionTool.INSTANCE.convert2List(functionList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "能耗报告信息点列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ResponseFunctionListItemVO>> queryFunction(
    //        @Valid @RequestBody PageQueryFunctionDTO pageQueryFunctionDTO) {
    //    IPage<Function> pagedList = functionService.pageQueryFunction(pageQueryFunctionDTO);
    //    IPage<ResponseFunctionListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertFunctionTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，Function实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

