package com.persagy.apm.energy.report.monthly.config.function.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.config.function.model.vo.*;
import com.persagy.apm.energy.report.monthly.config.function.model.dto.*;

import java.util.List;

/**
 * 能耗报告信息点(Function) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:05:36 2021-05-20 19:05:36
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertFunctionTool {

    ConvertFunctionTool INSTANCE = Mappers.getMapper(ConvertFunctionTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param function do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-05-20 19:05:36
     */
    ResponseFunctionItemVO convert2ResponseItemDTO(Function function);

    /**
     * do转换为ResponseListItemVO
     *
     * @param function do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-05-20 19:05:36
     */
    ResponseFunctionListItemVO convert2ResponseListItemDTO(Function function);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param functionList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-05-20 19:05:36
     */
    List<ResponseFunctionListItemVO> convert2List(List<Function> functionList);

    /**
     * addDTO转换为do
     *
     * @param addFunctionDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-05-20 19:05:36
     */
    Function convertAddDto2Entity(AddFunctionDTO addFunctionDTO);

    /**
     * updateDTO转换为实体
     *
     * @param function          更新前的do对象
     * @param updateFunctionDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-05-20 19:05:36
     */
    Function convertUpdateDto2Entity(@MappingTarget Function function, UpdateFunctionDTO updateFunctionDTO);
}

