package com.persagy.apm.energy.report.monthly.config.functiongroup.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.ConvertFunctionGroupTool;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.FunctionGroup;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.dto.*;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.vo.ResponseFunctionGroupItemVO;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.vo.ResponseFunctionGroupListItemVO;
import com.persagy.apm.energy.report.monthly.config.functiongroup.service.IFunctionGroupService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 信息点分组控制层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:06:28
 */
@ApiIgnore
@Api(tags = "信息点分组", hidden = true)
@Validated
@RestController
@RequestMapping("functionGroups")
public class FunctionGroupController {
    @Autowired
    private IFunctionGroupService functionGroupService;

    @ApiOperation(value = "创建信息点分组")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createFunctionGroup(
            @Valid @RequestBody AddFunctionGroupDTO addFunctionGroupDTO) {
        String id = functionGroupService.createFunctionGroup(addFunctionGroupDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "信息点分组详情")
    @PostMapping("/get")
    public CommonResult<ResponseFunctionGroupItemVO> queryFunctionGroupDetail(
            @Valid @RequestBody QueryFunctionGroupDetailDTO queryFunctionGroupDetailDTO) {
        FunctionGroup functionGroup = functionGroupService.queryFunctionGroupDetail(queryFunctionGroupDetailDTO.getId());
        ResponseFunctionGroupItemVO responseFunctionGroupItemVO = ConvertFunctionGroupTool.INSTANCE.convert2ResponseItemDTO(functionGroup);
        return ResultHelper.single(responseFunctionGroupItemVO);
    }


    @ApiOperation(value = "更新信息点分组")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateFunctionGroup(
            @Valid @RequestBody UpdateFunctionGroupDTO updateFunctionGroupDTO) {
        functionGroupService.updateFunctionGroup(updateFunctionGroupDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateFunctionGroupDTO.getId()));
    }


    @ApiOperation(value = "删除信息点分组")
    @PostMapping("/delete")
    public CommonResult deleteFunctionGroup(
            @RequestBody @Valid DeleteFunctionGroupDTO deleteFunctionGroupDTO) {
        functionGroupService.deleteFunctionGroup(deleteFunctionGroupDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "信息点分组列表")
    @PostMapping("/query")
    public CommonResult<PageList<ResponseFunctionGroupListItemVO>> queryFunctionGroup(
            @Valid @RequestBody QueryFunctionGroupDTO queryFunctionGroupDTO) {
        List<FunctionGroup> functionGroupList = functionGroupService.queryFunctionGroupList(queryFunctionGroupDTO);
        List<ResponseFunctionGroupListItemVO> resultList = ConvertFunctionGroupTool.INSTANCE.convert2List(functionGroupList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "信息点分组列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ResponseFunctionGroupListItemVO>> queryFunctionGroup(
    //        @Valid @RequestBody PageQueryFunctionGroupDTO pageQueryFunctionGroupDTO) {
    //    IPage<FunctionGroup> pagedList = functionGroupService.pageQueryFunctionGroup(pageQueryFunctionGroupDTO);
    //    IPage<ResponseFunctionGroupListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertFunctionGroupTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，FunctionGroup实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

