package com.persagy.apm.energy.report.monthly.config.functiongroup.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.vo.*;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.dto.*;

import java.util.List;

/**
 * 信息点分组(FunctionGroup) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:06:31 2021-05-20 19:06:31
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertFunctionGroupTool {

    ConvertFunctionGroupTool INSTANCE = Mappers.getMapper(ConvertFunctionGroupTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param functionGroup do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-05-20 19:06:31
     */
    ResponseFunctionGroupItemVO convert2ResponseItemDTO(FunctionGroup functionGroup);

    /**
     * do转换为ResponseListItemVO
     *
     * @param functionGroup do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-05-20 19:06:31
     */
    ResponseFunctionGroupListItemVO convert2ResponseListItemDTO(FunctionGroup functionGroup);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param functionGroupList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-05-20 19:06:31
     */
    List<ResponseFunctionGroupListItemVO> convert2List(List<FunctionGroup> functionGroupList);

    /**
     * addDTO转换为do
     *
     * @param addFunctionGroupDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-05-20 19:06:31
     */
    FunctionGroup convertAddDto2Entity(AddFunctionGroupDTO addFunctionGroupDTO);

    /**
     * updateDTO转换为实体
     *
     * @param functionGroup          更新前的do对象
     * @param updateFunctionGroupDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-05-20 19:06:31
     */
    FunctionGroup convertUpdateDto2Entity(@MappingTarget FunctionGroup functionGroup, UpdateFunctionGroupDTO updateFunctionGroupDTO);
}

