package com.persagy.apm.energy.report.monthly.config.functiongroup.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.config.functiongroup.dao.FunctionGroupMapper;
import com.persagy.apm.energy.report.monthly.config.functiongroup.service.IFunctionGroupService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.*;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 信息点分组(FunctionGroup) service层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:06:29
 */
@Service
public class FunctionGroupServiceImpl extends ServiceImpl<FunctionGroupMapper, FunctionGroup>
        implements IFunctionGroupService {

    /**
     * 创建信息点分组
     *
     * @return 信息点分组主键
     * @author lixing
     * @version V1.0 2021-05-20 19:06:29
     */
    @Override
    public String createFunctionGroup(AddFunctionGroupDTO addFunctionGroupDTO) {
        FunctionGroup functionGroup = ConvertFunctionGroupTool.INSTANCE.convertAddDto2Entity(addFunctionGroupDTO);
        // 设置默认值
        setDefaultValue(functionGroup);
        save(functionGroup);
        return functionGroup.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param functionGroup 信息点分组实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(FunctionGroup functionGroup) {
        functionGroup.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 信息点分组详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-20 19:06:29
     */
    @Override
    public FunctionGroup queryFunctionGroupDetail(String id) {
        FunctionGroup functionGroup = getById(id);
        if (functionGroup == null) {
            throw new IllegalArgumentException("查看FunctionGroup详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return functionGroup;
    }

    /**
     * 更新信息点分组
     *
     * @author lixing
     * @version V1.0 2021-05-20 19:06:29
     */
    @Override
    public void updateFunctionGroup(UpdateFunctionGroupDTO updateFunctionGroupDTO) {
        FunctionGroup functionGroup = getById(updateFunctionGroupDTO.getId());
        functionGroup = ConvertFunctionGroupTool.INSTANCE.convertUpdateDto2Entity(functionGroup, updateFunctionGroupDTO);
        functionGroup.setModifier(AppContext.getContext().getAccountId());
        updateById(functionGroup);
    }

    /**
     * 校验信息点分组是否可删除
     *
     * @param id 信息点分组主键
     * @return 信息点分组do类
     * @author lixing
     * @version V1.0 2021-05-20 19:06:29
     */
    public FunctionGroup checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除FunctionGroup时发生异常，主键为空");
        }

        FunctionGroup functionGroup = getById(id);

        if (functionGroup == null) {
            throw new IllegalArgumentException("删除FunctionGroup时发生异常，找不到要删除的数据，id:" + id);
        }

        return functionGroup;
    }

    /**
     * 删除信息点分组
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-20 19:06:29
     */
    @Override
    public void deleteFunctionGroup(String id) {
        // 校验是否可删除
        FunctionGroup functionGroup = checkDeletable(id);

        functionGroup.setValid(ValidEnum.FALSE.getType());
        updateById(functionGroup);
    }

    /**
     * 查询信息点分组
     *
     * @return List<FunctionGroup>
     * @author lixing
     * @version V1.0 2021-05-20 19:06:29
     */
    @Override
    public List<FunctionGroup> queryFunctionGroupList(QueryFunctionGroupDTO queryFunctionGroupDTO) {
        QueryWrapper<FunctionGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(FunctionGroup.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, FunctionGroup.PROP_CREATIONTIME);

        if (queryFunctionGroupDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryFunctionGroupDTO.getName())) {
                queryWrapper.like(FunctionGroup.PROP_NAME, queryFunctionGroupDTO.getName());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryFunctionGroupDTO.getGroupIds())) {
                queryWrapper.like(FunctionGroup.PROP_GROUP_IDS, queryFunctionGroupDTO.getGroupIds());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryFunctionGroupDTO.getFunctionIds())) {
                queryWrapper.like(FunctionGroup.PROP_FUNCTION_IDS, queryFunctionGroupDTO.getFunctionIds());
            }


        }

        return list(queryWrapper);
    }

    /**
     * 分页查询信息点分组
     *
     * @return IPage<FunctionGroup>
     * @author lixing
     * @version V1.0 2021-05-20 19:06:29
     */
    @Override
    public IPage<FunctionGroup> pageQueryFunctionGroup(PageQueryFunctionGroupDTO pageQueryFunctionGroupDTO) {
        QueryWrapper<FunctionGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(FunctionGroup.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<FunctionGroup> pageParam = new Page<>(pageQueryFunctionGroupDTO.getPage(), pageQueryFunctionGroupDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryFunctionGroupDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, FunctionGroup.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryFunctionGroupDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionGroupDTO.getName())) {
            queryWrapper.like(FunctionGroup.PROP_NAME, pageQueryFunctionGroupDTO.getName());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionGroupDTO.getGroupIds())) {
            queryWrapper.like(FunctionGroup.PROP_GROUP_IDS, pageQueryFunctionGroupDTO.getGroupIds());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionGroupDTO.getFunctionIds())) {
            queryWrapper.like(FunctionGroup.PROP_FUNCTION_IDS, pageQueryFunctionGroupDTO.getFunctionIds());
        }


        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
