package com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.ConvertReportDetailStatisticsItemRelTool;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.ReportDetailStatisticsItemRel;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.dto.*;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.vo.ResponseReportDetailStatisticsItemRelItemVO;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.vo.ResponseReportDetailStatisticsItemRelListItemVO;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.service.IReportDetailStatisticsItemRelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 报告详情与统计条目的关联控制层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:19:38
 */
@ApiIgnore
@Api(tags = "报告详情与统计条目的关联", hidden = true)
@Validated
@RestController
@RequestMapping("reportDetailStatisticsItemRels")
public class ReportDetailStatisticsItemRelController {
    @Autowired
    private IReportDetailStatisticsItemRelService reportDetailStatisticsItemRelService;

    @ApiOperation(value = "创建报告详情与统计条目的关联")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createReportDetailStatisticsItemRel(
            @Valid @RequestBody AddReportDetailStatisticsItemRelDTO addReportDetailStatisticsItemRelDTO) {
        String id = reportDetailStatisticsItemRelService.createReportDetailStatisticsItemRel(addReportDetailStatisticsItemRelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "报告详情与统计条目的关联详情")
    @PostMapping("/get")
    public CommonResult<ResponseReportDetailStatisticsItemRelItemVO> queryReportDetailStatisticsItemRelDetail(
            @Valid @RequestBody QueryReportDetailStatisticsItemRelDetailDTO queryReportDetailStatisticsItemRelDetailDTO) {
        ReportDetailStatisticsItemRel reportDetailStatisticsItemRel = reportDetailStatisticsItemRelService.queryReportDetailStatisticsItemRelDetail(queryReportDetailStatisticsItemRelDetailDTO.getId());
        ResponseReportDetailStatisticsItemRelItemVO responseReportDetailStatisticsItemRelItemVO = ConvertReportDetailStatisticsItemRelTool.INSTANCE.convert2ResponseItemDTO(reportDetailStatisticsItemRel);
        return ResultHelper.single(responseReportDetailStatisticsItemRelItemVO);
    }


    @ApiOperation(value = "更新报告详情与统计条目的关联")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateReportDetailStatisticsItemRel(
            @Valid @RequestBody UpdateReportDetailStatisticsItemRelDTO updateReportDetailStatisticsItemRelDTO) {
        reportDetailStatisticsItemRelService.updateReportDetailStatisticsItemRel(updateReportDetailStatisticsItemRelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateReportDetailStatisticsItemRelDTO.getId()));
    }


    @ApiOperation(value = "删除报告详情与统计条目的关联")
    @PostMapping("/delete")
    public CommonResult deleteReportDetailStatisticsItemRel(
            @RequestBody @Valid DeleteReportDetailStatisticsItemRelDTO deleteReportDetailStatisticsItemRelDTO) {
        reportDetailStatisticsItemRelService.deleteReportDetailStatisticsItemRel(deleteReportDetailStatisticsItemRelDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "报告详情与统计条目的关联列表")
    @PostMapping("/query")
    public CommonResult<PageList<ResponseReportDetailStatisticsItemRelListItemVO>> queryReportDetailStatisticsItemRel(
            @Valid @RequestBody QueryReportDetailStatisticsItemRelDTO queryReportDetailStatisticsItemRelDTO) {
        List<ReportDetailStatisticsItemRel> reportDetailStatisticsItemRelList = reportDetailStatisticsItemRelService.queryReportDetailStatisticsItemRelList(queryReportDetailStatisticsItemRelDTO);
        List<ResponseReportDetailStatisticsItemRelListItemVO> resultList = ConvertReportDetailStatisticsItemRelTool.INSTANCE.convert2List(reportDetailStatisticsItemRelList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "报告详情与统计条目的关联列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ResponseReportDetailStatisticsItemRelListItemVO>> queryReportDetailStatisticsItemRel(
    //        @Valid @RequestBody PageQueryReportDetailStatisticsItemRelDTO pageQueryReportDetailStatisticsItemRelDTO) {
    //    IPage<ReportDetailStatisticsItemRel> pagedList = reportDetailStatisticsItemRelService.pageQueryReportDetailStatisticsItemRel(pageQueryReportDetailStatisticsItemRelDTO);
    //    IPage<ResponseReportDetailStatisticsItemRelListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertReportDetailStatisticsItemRelTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ReportDetailStatisticsItemRel实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

