package com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.dao.ReportDetailStatisticsItemRelMapper;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.service.IReportDetailStatisticsItemRelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.*;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告详情与统计条目的关联(ReportDetailStatisticsItemRel) service层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:19:39
 */
@Service
public class ReportDetailStatisticsItemRelServiceImpl extends ServiceImpl<ReportDetailStatisticsItemRelMapper, ReportDetailStatisticsItemRel>
        implements IReportDetailStatisticsItemRelService {

    /**
     * 创建报告详情与统计条目的关联
     *
     * @return 报告详情与统计条目的关联主键
     * @author lixing
     * @version V1.0 2021-05-20 19:19:39
     */
    @Override
    public String createReportDetailStatisticsItemRel(AddReportDetailStatisticsItemRelDTO addReportDetailStatisticsItemRelDTO) {
        ReportDetailStatisticsItemRel reportDetailStatisticsItemRel = ConvertReportDetailStatisticsItemRelTool.INSTANCE.convertAddDto2Entity(addReportDetailStatisticsItemRelDTO);
        // 设置默认值
        setDefaultValue(reportDetailStatisticsItemRel);
        save(reportDetailStatisticsItemRel);
        return reportDetailStatisticsItemRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportDetailStatisticsItemRel 报告详情与统计条目的关联实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportDetailStatisticsItemRel reportDetailStatisticsItemRel) {
        reportDetailStatisticsItemRel.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 报告详情与统计条目的关联详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-20 19:19:39
     */
    @Override
    public ReportDetailStatisticsItemRel queryReportDetailStatisticsItemRelDetail(String id) {
        ReportDetailStatisticsItemRel reportDetailStatisticsItemRel = getById(id);
        if (reportDetailStatisticsItemRel == null) {
            throw new IllegalArgumentException("查看ReportDetailStatisticsItemRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportDetailStatisticsItemRel;
    }

    /**
     * 更新报告详情与统计条目的关联
     *
     * @author lixing
     * @version V1.0 2021-05-20 19:19:39
     */
    @Override
    public void updateReportDetailStatisticsItemRel(UpdateReportDetailStatisticsItemRelDTO updateReportDetailStatisticsItemRelDTO) {
        ReportDetailStatisticsItemRel reportDetailStatisticsItemRel = getById(updateReportDetailStatisticsItemRelDTO.getId());
        reportDetailStatisticsItemRel = ConvertReportDetailStatisticsItemRelTool.INSTANCE.convertUpdateDto2Entity(reportDetailStatisticsItemRel, updateReportDetailStatisticsItemRelDTO);
        reportDetailStatisticsItemRel.setModifier(AppContext.getContext().getAccountId());
        updateById(reportDetailStatisticsItemRel);
    }

    /**
     * 校验报告详情与统计条目的关联是否可删除
     *
     * @param id 报告详情与统计条目的关联主键
     * @return 报告详情与统计条目的关联do类
     * @author lixing
     * @version V1.0 2021-05-20 19:19:39
     */
    public ReportDetailStatisticsItemRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportDetailStatisticsItemRel时发生异常，主键为空");
        }

        ReportDetailStatisticsItemRel reportDetailStatisticsItemRel = getById(id);

        if (reportDetailStatisticsItemRel == null) {
            throw new IllegalArgumentException("删除ReportDetailStatisticsItemRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportDetailStatisticsItemRel;
    }

    /**
     * 删除报告详情与统计条目的关联
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-20 19:19:39
     */
    @Override
    public void deleteReportDetailStatisticsItemRel(String id) {
        // 校验是否可删除
        ReportDetailStatisticsItemRel reportDetailStatisticsItemRel = checkDeletable(id);

        reportDetailStatisticsItemRel.setValid(ValidEnum.FALSE.getType());
        updateById(reportDetailStatisticsItemRel);
    }

    /**
     * 查询报告详情与统计条目的关联
     *
     * @return List<ReportDetailStatisticsItemRel>
     * @author lixing
     * @version V1.0 2021-05-20 19:19:39
     */
    @Override
    public List<ReportDetailStatisticsItemRel> queryReportDetailStatisticsItemRelList(QueryReportDetailStatisticsItemRelDTO queryReportDetailStatisticsItemRelDTO) {
        QueryWrapper<ReportDetailStatisticsItemRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportDetailStatisticsItemRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportDetailStatisticsItemRel.PROP_CREATIONTIME);

        if (queryReportDetailStatisticsItemRelDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailStatisticsItemRelDTO.getReportDetailId())) {
                queryWrapper.like(ReportDetailStatisticsItemRel.PROP_REPORT_DETAIL_ID, queryReportDetailStatisticsItemRelDTO.getReportDetailId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailStatisticsItemRelDTO.getReportTypeId())) {
                queryWrapper.like(ReportDetailStatisticsItemRel.PROP_REPORT_TYPE_ID, queryReportDetailStatisticsItemRelDTO.getReportTypeId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailStatisticsItemRelDTO.getStatisticsItemId())) {
                queryWrapper.like(ReportDetailStatisticsItemRel.PROP_STATISTICS_ITEM_ID, queryReportDetailStatisticsItemRelDTO.getStatisticsItemId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailStatisticsItemRelDTO.getUnqualifiedReason())) {
                queryWrapper.like(ReportDetailStatisticsItemRel.PROP_UNQUALIFIED_REASON, queryReportDetailStatisticsItemRelDTO.getUnqualifiedReason());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportDetailStatisticsItemRelDTO.getRemark())) {
                queryWrapper.like(ReportDetailStatisticsItemRel.PROP_REMARK, queryReportDetailStatisticsItemRelDTO.getRemark());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告详情与统计条目的关联
     *
     * @return IPage<ReportDetailStatisticsItemRel>
     * @author lixing
     * @version V1.0 2021-05-20 19:19:39
     */
    @Override
    public IPage<ReportDetailStatisticsItemRel> pageQueryReportDetailStatisticsItemRel(PageQueryReportDetailStatisticsItemRelDTO pageQueryReportDetailStatisticsItemRelDTO) {
        QueryWrapper<ReportDetailStatisticsItemRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportDetailStatisticsItemRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportDetailStatisticsItemRel> pageParam = new Page<>(pageQueryReportDetailStatisticsItemRelDTO.getPage(), pageQueryReportDetailStatisticsItemRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportDetailStatisticsItemRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportDetailStatisticsItemRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportDetailStatisticsItemRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailStatisticsItemRelDTO.getReportDetailId())) {
            queryWrapper.like(ReportDetailStatisticsItemRel.PROP_REPORT_DETAIL_ID, pageQueryReportDetailStatisticsItemRelDTO.getReportDetailId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailStatisticsItemRelDTO.getReportTypeId())) {
            queryWrapper.like(ReportDetailStatisticsItemRel.PROP_REPORT_TYPE_ID, pageQueryReportDetailStatisticsItemRelDTO.getReportTypeId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailStatisticsItemRelDTO.getStatisticsItemId())) {
            queryWrapper.like(ReportDetailStatisticsItemRel.PROP_STATISTICS_ITEM_ID, pageQueryReportDetailStatisticsItemRelDTO.getStatisticsItemId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailStatisticsItemRelDTO.getUnqualifiedReason())) {
            queryWrapper.like(ReportDetailStatisticsItemRel.PROP_UNQUALIFIED_REASON, pageQueryReportDetailStatisticsItemRelDTO.getUnqualifiedReason());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportDetailStatisticsItemRelDTO.getRemark())) {
            queryWrapper.like(ReportDetailStatisticsItemRel.PROP_REMARK, pageQueryReportDetailStatisticsItemRelDTO.getRemark());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
