package com.persagy.apm.energy.report.monthly.config.rel.typefunction.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.config.rel.typefunction.dao.ReportTypeFunctionRelMapper;
import com.persagy.apm.energy.report.monthly.config.rel.typefunction.service.IReportTypeFunctionRelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.config.rel.typefunction.model.*;
import com.persagy.apm.energy.report.monthly.config.rel.typefunction.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告类型(ReportTypeFunctionRel) service层
 *
 * @author lixing
 * @version V1.0 2021-05-24 16:02:41
 */
@Service
public class ReportTypeFunctionRelServiceImpl extends ServiceImpl<ReportTypeFunctionRelMapper, ReportTypeFunctionRel>
        implements IReportTypeFunctionRelService {

    /**
     * 创建报告类型
     *
     * @return 报告类型主键
     * @author lixing
     * @version V1.0 2021-05-24 16:02:41
     */
    @Override
    public String createReportTypeFunctionRel(AddReportTypeFunctionRelDTO addReportTypeFunctionRelDTO) {
        ReportTypeFunctionRel reportTypeFunctionRel = ConvertReportTypeFunctionRelTool.INSTANCE.convertAddDto2Entity(addReportTypeFunctionRelDTO);
        // 设置默认值
        setDefaultValue(reportTypeFunctionRel);
        save(reportTypeFunctionRel);
        return reportTypeFunctionRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportTypeFunctionRel 报告类型实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportTypeFunctionRel reportTypeFunctionRel) {
        reportTypeFunctionRel.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 报告类型详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-24 16:02:41
     */
    @Override
    public ReportTypeFunctionRel queryReportTypeFunctionRelDetail(String id) {
        ReportTypeFunctionRel reportTypeFunctionRel = getById(id);
        if (reportTypeFunctionRel == null) {
            throw new IllegalArgumentException("查看ReportTypeFunctionRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportTypeFunctionRel;
    }

    /**
     * 更新报告类型
     *
     * @author lixing
     * @version V1.0 2021-05-24 16:02:41
     */
    @Override
    public void updateReportTypeFunctionRel(UpdateReportTypeFunctionRelDTO updateReportTypeFunctionRelDTO) {
        ReportTypeFunctionRel reportTypeFunctionRel = getById(updateReportTypeFunctionRelDTO.getId());
        reportTypeFunctionRel = ConvertReportTypeFunctionRelTool.INSTANCE.convertUpdateDto2Entity(reportTypeFunctionRel, updateReportTypeFunctionRelDTO);
        reportTypeFunctionRel.setModifier(AppContext.getContext().getAccountId());
        updateById(reportTypeFunctionRel);
    }

    /**
     * 校验报告类型是否可删除
     *
     * @param id 报告类型主键
     * @return 报告类型do类
     * @author lixing
     * @version V1.0 2021-05-24 16:02:41
     */
    public ReportTypeFunctionRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportTypeFunctionRel时发生异常，主键为空");
        }

        ReportTypeFunctionRel reportTypeFunctionRel = getById(id);

        if (reportTypeFunctionRel == null) {
            throw new IllegalArgumentException("删除ReportTypeFunctionRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportTypeFunctionRel;
    }

    /**
     * 删除报告类型
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-24 16:02:41
     */
    @Override
    public void deleteReportTypeFunctionRel(String id) {
        // 校验是否可删除
        ReportTypeFunctionRel reportTypeFunctionRel = checkDeletable(id);

        reportTypeFunctionRel.setValid(ValidEnum.FALSE.getType());
        updateById(reportTypeFunctionRel);
    }

    /**
     * 查询报告类型
     *
     * @return List<ReportTypeFunctionRel>
     * @author lixing
     * @version V1.0 2021-05-24 16:02:41
     */
    @Override
    public List<ReportTypeFunctionRel> queryReportTypeFunctionRelList(QueryReportTypeFunctionRelDTO queryReportTypeFunctionRelDTO) {
        QueryWrapper<ReportTypeFunctionRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeFunctionRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportTypeFunctionRel.PROP_CREATIONTIME);

        if (queryReportTypeFunctionRelDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportTypeFunctionRelDTO.getReportTypeId())) {
                queryWrapper.like(ReportTypeFunctionRel.PROP_REPORT_TYPE_ID, queryReportTypeFunctionRelDTO.getReportTypeId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportTypeFunctionRelDTO.getFunctionId())) {
                queryWrapper.like(ReportTypeFunctionRel.PROP_FUNCTION_ID, queryReportTypeFunctionRelDTO.getFunctionId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportTypeFunctionRelDTO.getBelongParagraph())) {
                queryWrapper.like(ReportTypeFunctionRel.PROP_BELONG_PARAGRAPH, queryReportTypeFunctionRelDTO.getBelongParagraph());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告类型
     *
     * @return IPage<ReportTypeFunctionRel>
     * @author lixing
     * @version V1.0 2021-05-24 16:02:41
     */
    @Override
    public IPage<ReportTypeFunctionRel> pageQueryReportTypeFunctionRel(PageQueryReportTypeFunctionRelDTO pageQueryReportTypeFunctionRelDTO) {
        QueryWrapper<ReportTypeFunctionRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeFunctionRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportTypeFunctionRel> pageParam = new Page<>(pageQueryReportTypeFunctionRelDTO.getPage(), pageQueryReportTypeFunctionRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportTypeFunctionRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportTypeFunctionRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportTypeFunctionRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeFunctionRelDTO.getReportTypeId())) {
            queryWrapper.like(ReportTypeFunctionRel.PROP_REPORT_TYPE_ID, pageQueryReportTypeFunctionRelDTO.getReportTypeId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeFunctionRelDTO.getFunctionId())) {
            queryWrapper.like(ReportTypeFunctionRel.PROP_FUNCTION_ID, pageQueryReportTypeFunctionRelDTO.getFunctionId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeFunctionRelDTO.getBelongParagraph())) {
            queryWrapper.like(ReportTypeFunctionRel.PROP_BELONG_PARAGRAPH, pageQueryReportTypeFunctionRelDTO.getBelongParagraph());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
