package com.persagy.apm.energy.report.monthly.config.rel.typefunctiongroup.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.config.rel.typefunctiongroup.dao.ReportTypeFunctionGroupRelMapper;
import com.persagy.apm.energy.report.monthly.config.rel.typefunctiongroup.service.IReportTypeFunctionGroupRelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.config.rel.typefunctiongroup.model.*;
import com.persagy.apm.energy.report.monthly.config.rel.typefunctiongroup.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告类型(ReportTypeFunctionGroupRel) service层
 *
 * @author lixing
 * @version V1.0 2021-05-24 16:02:14
 */
@Service
public class ReportTypeFunctionGroupRelServiceImpl extends ServiceImpl<ReportTypeFunctionGroupRelMapper, ReportTypeFunctionGroupRel>
        implements IReportTypeFunctionGroupRelService {

    /**
     * 创建报告类型
     *
     * @return 报告类型主键
     * @author lixing
     * @version V1.0 2021-05-24 16:02:14
     */
    @Override
    public String createReportTypeFunctionGroupRel(AddReportTypeFunctionGroupRelDTO addReportTypeFunctionGroupRelDTO) {
        ReportTypeFunctionGroupRel reportTypeFunctionGroupRel = ConvertReportTypeFunctionGroupRelTool.INSTANCE.convertAddDto2Entity(addReportTypeFunctionGroupRelDTO);
        // 设置默认值
        setDefaultValue(reportTypeFunctionGroupRel);
        save(reportTypeFunctionGroupRel);
        return reportTypeFunctionGroupRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportTypeFunctionGroupRel 报告类型实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportTypeFunctionGroupRel reportTypeFunctionGroupRel) {
        reportTypeFunctionGroupRel.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 报告类型详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-24 16:02:14
     */
    @Override
    public ReportTypeFunctionGroupRel queryReportTypeFunctionGroupRelDetail(String id) {
        ReportTypeFunctionGroupRel reportTypeFunctionGroupRel = getById(id);
        if (reportTypeFunctionGroupRel == null) {
            throw new IllegalArgumentException("查看ReportTypeFunctionGroupRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportTypeFunctionGroupRel;
    }

    /**
     * 更新报告类型
     *
     * @author lixing
     * @version V1.0 2021-05-24 16:02:14
     */
    @Override
    public void updateReportTypeFunctionGroupRel(UpdateReportTypeFunctionGroupRelDTO updateReportTypeFunctionGroupRelDTO) {
        ReportTypeFunctionGroupRel reportTypeFunctionGroupRel = getById(updateReportTypeFunctionGroupRelDTO.getId());
        reportTypeFunctionGroupRel = ConvertReportTypeFunctionGroupRelTool.INSTANCE.convertUpdateDto2Entity(reportTypeFunctionGroupRel, updateReportTypeFunctionGroupRelDTO);
        reportTypeFunctionGroupRel.setModifier(AppContext.getContext().getAccountId());
        updateById(reportTypeFunctionGroupRel);
    }

    /**
     * 校验报告类型是否可删除
     *
     * @param id 报告类型主键
     * @return 报告类型do类
     * @author lixing
     * @version V1.0 2021-05-24 16:02:14
     */
    public ReportTypeFunctionGroupRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportTypeFunctionGroupRel时发生异常，主键为空");
        }

        ReportTypeFunctionGroupRel reportTypeFunctionGroupRel = getById(id);

        if (reportTypeFunctionGroupRel == null) {
            throw new IllegalArgumentException("删除ReportTypeFunctionGroupRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportTypeFunctionGroupRel;
    }

    /**
     * 删除报告类型
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-24 16:02:14
     */
    @Override
    public void deleteReportTypeFunctionGroupRel(String id) {
        // 校验是否可删除
        ReportTypeFunctionGroupRel reportTypeFunctionGroupRel = checkDeletable(id);

        reportTypeFunctionGroupRel.setValid(ValidEnum.FALSE.getType());
        updateById(reportTypeFunctionGroupRel);
    }

    /**
     * 查询报告类型
     *
     * @return List<ReportTypeFunctionGroupRel>
     * @author lixing
     * @version V1.0 2021-05-24 16:02:14
     */
    @Override
    public List<ReportTypeFunctionGroupRel> queryReportTypeFunctionGroupRelList(QueryReportTypeFunctionGroupRelDTO queryReportTypeFunctionGroupRelDTO) {
        QueryWrapper<ReportTypeFunctionGroupRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeFunctionGroupRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, true, ReportTypeFunctionGroupRel.PROP_CREATIONTIME);

        if (queryReportTypeFunctionGroupRelDTO != null) {

            if (StringUtils.isNotEmpty(queryReportTypeFunctionGroupRelDTO.getReportTypeId())) {
                queryWrapper.eq(ReportTypeFunctionGroupRel.PROP_REPORT_TYPE_ID, queryReportTypeFunctionGroupRelDTO.getReportTypeId());
            }

            if (StringUtils.isNotEmpty(queryReportTypeFunctionGroupRelDTO.getFunctionGroupId())) {
                queryWrapper.eq(ReportTypeFunctionGroupRel.PROP_FUNCTION_GROUP_ID, queryReportTypeFunctionGroupRelDTO.getFunctionGroupId());
            }

            if (StringUtils.isNotEmpty(queryReportTypeFunctionGroupRelDTO.getBelongParagraph())) {
                queryWrapper.eq(ReportTypeFunctionGroupRel.PROP_BELONG_PARAGRAPH, queryReportTypeFunctionGroupRelDTO.getBelongParagraph());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告类型
     *
     * @return IPage<ReportTypeFunctionGroupRel>
     * @author lixing
     * @version V1.0 2021-05-24 16:02:14
     */
    @Override
    public IPage<ReportTypeFunctionGroupRel> pageQueryReportTypeFunctionGroupRel(PageQueryReportTypeFunctionGroupRelDTO pageQueryReportTypeFunctionGroupRelDTO) {
        QueryWrapper<ReportTypeFunctionGroupRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeFunctionGroupRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportTypeFunctionGroupRel> pageParam = new Page<>(pageQueryReportTypeFunctionGroupRelDTO.getPage(), pageQueryReportTypeFunctionGroupRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportTypeFunctionGroupRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportTypeFunctionGroupRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportTypeFunctionGroupRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeFunctionGroupRelDTO.getReportTypeId())) {
            queryWrapper.like(ReportTypeFunctionGroupRel.PROP_REPORT_TYPE_ID, pageQueryReportTypeFunctionGroupRelDTO.getReportTypeId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeFunctionGroupRelDTO.getFunctionGroupId())) {
            queryWrapper.like(ReportTypeFunctionGroupRel.PROP_FUNCTION_GROUP_ID, pageQueryReportTypeFunctionGroupRelDTO.getFunctionGroupId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeFunctionGroupRelDTO.getBelongParagraph())) {
            queryWrapper.like(ReportTypeFunctionGroupRel.PROP_BELONG_PARAGRAPH, pageQueryReportTypeFunctionGroupRelDTO.getBelongParagraph());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
