package com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.ConvertReportTypeStatisticsItemRelTool;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.ReportTypeStatisticsItemRel;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.dto.*;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.vo.ResponseReportTypeStatisticsItemRelItemVO;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.vo.ResponseReportTypeStatisticsItemRelListItemVO;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.service.IReportTypeStatisticsItemRelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 报告类型和统计条目关联关系控制层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:20:19
 */
@ApiIgnore
@Api(tags = "报告类型和统计条目关联关系", hidden = true)
@Validated
@RestController
@RequestMapping("reportTypeStatisticsItemRels")
public class ReportTypeStatisticsItemRelController {
    @Autowired
    private IReportTypeStatisticsItemRelService reportTypeStatisticsItemRelService;

    @ApiOperation(value = "创建报告类型和统计条目关联关系")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createReportTypeStatisticsItemRel(
            @Valid @RequestBody AddReportTypeStatisticsItemRelDTO addReportTypeStatisticsItemRelDTO) {
        String id = reportTypeStatisticsItemRelService.createReportTypeStatisticsItemRel(addReportTypeStatisticsItemRelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "报告类型和统计条目关联关系详情")
    @PostMapping("/get")
    public CommonResult<ResponseReportTypeStatisticsItemRelItemVO> queryReportTypeStatisticsItemRelDetail(
            @Valid @RequestBody QueryReportTypeStatisticsItemRelDetailDTO queryReportTypeStatisticsItemRelDetailDTO) {
        ReportTypeStatisticsItemRel reportTypeStatisticsItemRel = reportTypeStatisticsItemRelService.queryReportTypeStatisticsItemRelDetail(queryReportTypeStatisticsItemRelDetailDTO.getId());
        ResponseReportTypeStatisticsItemRelItemVO responseReportTypeStatisticsItemRelItemVO = ConvertReportTypeStatisticsItemRelTool.INSTANCE.convert2ResponseItemDTO(reportTypeStatisticsItemRel);
        return ResultHelper.single(responseReportTypeStatisticsItemRelItemVO);
    }


    @ApiOperation(value = "更新报告类型和统计条目关联关系")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateReportTypeStatisticsItemRel(
            @Valid @RequestBody UpdateReportTypeStatisticsItemRelDTO updateReportTypeStatisticsItemRelDTO) {
        reportTypeStatisticsItemRelService.updateReportTypeStatisticsItemRel(updateReportTypeStatisticsItemRelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateReportTypeStatisticsItemRelDTO.getId()));
    }


    @ApiOperation(value = "删除报告类型和统计条目关联关系")
    @PostMapping("/delete")
    public CommonResult deleteReportTypeStatisticsItemRel(
            @RequestBody @Valid DeleteReportTypeStatisticsItemRelDTO deleteReportTypeStatisticsItemRelDTO) {
        reportTypeStatisticsItemRelService.deleteReportTypeStatisticsItemRel(deleteReportTypeStatisticsItemRelDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "报告类型和统计条目关联关系列表")
    @PostMapping("/query")
    public CommonResult<PageList<ResponseReportTypeStatisticsItemRelListItemVO>> queryReportTypeStatisticsItemRel(
            @Valid @RequestBody QueryReportTypeStatisticsItemRelDTO queryReportTypeStatisticsItemRelDTO) {
        List<ReportTypeStatisticsItemRel> reportTypeStatisticsItemRelList = reportTypeStatisticsItemRelService.queryReportTypeStatisticsItemRelList(queryReportTypeStatisticsItemRelDTO);
        List<ResponseReportTypeStatisticsItemRelListItemVO> resultList = ConvertReportTypeStatisticsItemRelTool.INSTANCE.convert2List(reportTypeStatisticsItemRelList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "报告类型和统计条目关联关系列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ResponseReportTypeStatisticsItemRelListItemVO>> queryReportTypeStatisticsItemRel(
    //        @Valid @RequestBody PageQueryReportTypeStatisticsItemRelDTO pageQueryReportTypeStatisticsItemRelDTO) {
    //    IPage<ReportTypeStatisticsItemRel> pagedList = reportTypeStatisticsItemRelService.pageQueryReportTypeStatisticsItemRel(pageQueryReportTypeStatisticsItemRelDTO);
    //    IPage<ResponseReportTypeStatisticsItemRelListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertReportTypeStatisticsItemRelTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ReportTypeStatisticsItemRel实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

