package com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.dao.ReportTypeStatisticsItemRelMapper;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.service.IReportTypeStatisticsItemRelService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.*;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报告类型和统计条目关联关系(ReportTypeStatisticsItemRel) service层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:20:20
 */
@Service
public class ReportTypeStatisticsItemRelServiceImpl extends ServiceImpl<ReportTypeStatisticsItemRelMapper, ReportTypeStatisticsItemRel>
        implements IReportTypeStatisticsItemRelService {

    /**
     * 创建报告类型和统计条目关联关系
     *
     * @return 报告类型和统计条目关联关系主键
     * @author lixing
     * @version V1.0 2021-05-20 19:20:20
     */
    @Override
    public String createReportTypeStatisticsItemRel(AddReportTypeStatisticsItemRelDTO addReportTypeStatisticsItemRelDTO) {
        ReportTypeStatisticsItemRel reportTypeStatisticsItemRel = ConvertReportTypeStatisticsItemRelTool.INSTANCE.convertAddDto2Entity(addReportTypeStatisticsItemRelDTO);
        // 设置默认值
        setDefaultValue(reportTypeStatisticsItemRel);
        save(reportTypeStatisticsItemRel);
        return reportTypeStatisticsItemRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportTypeStatisticsItemRel 报告类型和统计条目关联关系实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportTypeStatisticsItemRel reportTypeStatisticsItemRel) {
        reportTypeStatisticsItemRel.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 报告类型和统计条目关联关系详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-20 19:20:20
     */
    @Override
    public ReportTypeStatisticsItemRel queryReportTypeStatisticsItemRelDetail(String id) {
        ReportTypeStatisticsItemRel reportTypeStatisticsItemRel = getById(id);
        if (reportTypeStatisticsItemRel == null) {
            throw new IllegalArgumentException("查看ReportTypeStatisticsItemRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportTypeStatisticsItemRel;
    }

    /**
     * 更新报告类型和统计条目关联关系
     *
     * @author lixing
     * @version V1.0 2021-05-20 19:20:20
     */
    @Override
    public void updateReportTypeStatisticsItemRel(UpdateReportTypeStatisticsItemRelDTO updateReportTypeStatisticsItemRelDTO) {
        ReportTypeStatisticsItemRel reportTypeStatisticsItemRel = getById(updateReportTypeStatisticsItemRelDTO.getId());
        reportTypeStatisticsItemRel = ConvertReportTypeStatisticsItemRelTool.INSTANCE.convertUpdateDto2Entity(reportTypeStatisticsItemRel, updateReportTypeStatisticsItemRelDTO);
        reportTypeStatisticsItemRel.setModifier(AppContext.getContext().getAccountId());
        updateById(reportTypeStatisticsItemRel);
    }

    /**
     * 校验报告类型和统计条目关联关系是否可删除
     *
     * @param id 报告类型和统计条目关联关系主键
     * @return 报告类型和统计条目关联关系do类
     * @author lixing
     * @version V1.0 2021-05-20 19:20:20
     */
    public ReportTypeStatisticsItemRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportTypeStatisticsItemRel时发生异常，主键为空");
        }

        ReportTypeStatisticsItemRel reportTypeStatisticsItemRel = getById(id);

        if (reportTypeStatisticsItemRel == null) {
            throw new IllegalArgumentException("删除ReportTypeStatisticsItemRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportTypeStatisticsItemRel;
    }

    /**
     * 删除报告类型和统计条目关联关系
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-20 19:20:20
     */
    @Override
    public void deleteReportTypeStatisticsItemRel(String id) {
        // 校验是否可删除
        ReportTypeStatisticsItemRel reportTypeStatisticsItemRel = checkDeletable(id);

        reportTypeStatisticsItemRel.setValid(ValidEnum.FALSE.getType());
        updateById(reportTypeStatisticsItemRel);
    }

    /**
     * 查询报告类型和统计条目关联关系
     *
     * @return List<ReportTypeStatisticsItemRel>
     * @author lixing
     * @version V1.0 2021-05-20 19:20:20
     */
    @Override
    public List<ReportTypeStatisticsItemRel> queryReportTypeStatisticsItemRelList(QueryReportTypeStatisticsItemRelDTO queryReportTypeStatisticsItemRelDTO) {
        QueryWrapper<ReportTypeStatisticsItemRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeStatisticsItemRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportTypeStatisticsItemRel.PROP_CREATIONTIME);

        if (queryReportTypeStatisticsItemRelDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportTypeStatisticsItemRelDTO.getReportTypeId())) {
                queryWrapper.like(ReportTypeStatisticsItemRel.PROP_REPORT_TYPE_ID, queryReportTypeStatisticsItemRelDTO.getReportTypeId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportTypeStatisticsItemRelDTO.getStatisticsItemId())) {
                queryWrapper.like(ReportTypeStatisticsItemRel.PROP_STATISTICS_ITEM_ID, queryReportTypeStatisticsItemRelDTO.getStatisticsItemId());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告类型和统计条目关联关系
     *
     * @return IPage<ReportTypeStatisticsItemRel>
     * @author lixing
     * @version V1.0 2021-05-20 19:20:20
     */
    @Override
    public IPage<ReportTypeStatisticsItemRel> pageQueryReportTypeStatisticsItemRel(PageQueryReportTypeStatisticsItemRelDTO pageQueryReportTypeStatisticsItemRelDTO) {
        QueryWrapper<ReportTypeStatisticsItemRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportTypeStatisticsItemRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportTypeStatisticsItemRel> pageParam = new Page<>(pageQueryReportTypeStatisticsItemRelDTO.getPage(), pageQueryReportTypeStatisticsItemRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportTypeStatisticsItemRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportTypeStatisticsItemRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportTypeStatisticsItemRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeStatisticsItemRelDTO.getReportTypeId())) {
            queryWrapper.like(ReportTypeStatisticsItemRel.PROP_REPORT_TYPE_ID, pageQueryReportTypeStatisticsItemRelDTO.getReportTypeId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportTypeStatisticsItemRelDTO.getStatisticsItemId())) {
            queryWrapper.like(ReportTypeStatisticsItemRel.PROP_STATISTICS_ITEM_ID, pageQueryReportTypeStatisticsItemRelDTO.getStatisticsItemId());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
