package com.persagy.apm.energy.report.monthly.config.statisticsitems.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.ConvertStatisticItemsTool;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.dto.*;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.vo.ResponseStatisticItemsItemVO;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.vo.ResponseStatisticItemsListItemVO;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.service.IStatisticItemsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 统计条目控制层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:21:08
 */
@ApiIgnore
@Api(tags = "统计条目", hidden = true)
@Validated
@RestController
@RequestMapping("statisticItemss")
public class StatisticItemsController {
    @Autowired
    private IStatisticItemsService statisticItemsService;

    @ApiOperation(value = "创建统计条目")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createStatisticItems(
            @Valid @RequestBody AddStatisticItemsDTO addStatisticItemsDTO) {
        String id = statisticItemsService.createStatisticItems(addStatisticItemsDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "统计条目详情")
    @PostMapping("/get")
    public CommonResult<ResponseStatisticItemsItemVO> queryStatisticItemsDetail(
            @Valid @RequestBody QueryStatisticItemsDetailDTO queryStatisticItemsDetailDTO) {
        StatisticItems statisticItems = statisticItemsService.queryStatisticItemsDetail(queryStatisticItemsDetailDTO.getId());
        ResponseStatisticItemsItemVO responseStatisticItemsItemVO = ConvertStatisticItemsTool.INSTANCE.convert2ResponseItemDTO(statisticItems);
        return ResultHelper.single(responseStatisticItemsItemVO);
    }


    @ApiOperation(value = "更新统计条目")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateStatisticItems(
            @Valid @RequestBody UpdateStatisticItemsDTO updateStatisticItemsDTO) {
        statisticItemsService.updateStatisticItems(updateStatisticItemsDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateStatisticItemsDTO.getId()));
    }


    @ApiOperation(value = "删除统计条目")
    @PostMapping("/delete")
    public CommonResult deleteStatisticItems(
            @RequestBody @Valid DeleteStatisticItemsDTO deleteStatisticItemsDTO) {
        statisticItemsService.deleteStatisticItems(deleteStatisticItemsDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "统计条目列表")
    @PostMapping("/query")
    public CommonResult<PageList<ResponseStatisticItemsListItemVO>> queryStatisticItems(
            @Valid @RequestBody QueryStatisticItemsDTO queryStatisticItemsDTO) {
        List<StatisticItems> statisticItemsList = statisticItemsService.queryStatisticItemsList(queryStatisticItemsDTO);
        List<ResponseStatisticItemsListItemVO> resultList = ConvertStatisticItemsTool.INSTANCE.convert2List(statisticItemsList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "统计条目列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ResponseStatisticItemsListItemVO>> queryStatisticItems(
    //        @Valid @RequestBody PageQueryStatisticItemsDTO pageQueryStatisticItemsDTO) {
    //    IPage<StatisticItems> pagedList = statisticItemsService.pageQueryStatisticItems(pageQueryStatisticItemsDTO);
    //    IPage<ResponseStatisticItemsListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertStatisticItemsTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，StatisticItems实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

