package com.persagy.apm.energy.report.monthly.config.statisticsitems.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.dao.StatisticItemsMapper;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.service.IStatisticItemsService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.*;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 统计条目(StatisticItems) service层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:21:09
 */
@Service
public class StatisticItemsServiceImpl extends ServiceImpl<StatisticItemsMapper, StatisticItems>
        implements IStatisticItemsService {

    /**
     * 创建统计条目
     *
     * @return 统计条目主键
     * @author lixing
     * @version V1.0 2021-05-20 19:21:09
     */
    @Override
    public String createStatisticItems(AddStatisticItemsDTO addStatisticItemsDTO) {
        StatisticItems statisticItems = ConvertStatisticItemsTool.INSTANCE.convertAddDto2Entity(addStatisticItemsDTO);
        // 设置默认值
        setDefaultValue(statisticItems);
        save(statisticItems);
        return statisticItems.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param statisticItems 统计条目实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(StatisticItems statisticItems) {
        statisticItems.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 统计条目详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-20 19:21:09
     */
    @Override
    public StatisticItems queryStatisticItemsDetail(String id) {
        StatisticItems statisticItems = getById(id);
        if (statisticItems == null) {
            throw new IllegalArgumentException("查看StatisticItems详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return statisticItems;
    }

    /**
     * 更新统计条目
     *
     * @author lixing
     * @version V1.0 2021-05-20 19:21:09
     */
    @Override
    public void updateStatisticItems(UpdateStatisticItemsDTO updateStatisticItemsDTO) {
        StatisticItems statisticItems = getById(updateStatisticItemsDTO.getId());
        statisticItems = ConvertStatisticItemsTool.INSTANCE.convertUpdateDto2Entity(statisticItems, updateStatisticItemsDTO);
        statisticItems.setModifier(AppContext.getContext().getAccountId());
        updateById(statisticItems);
    }

    /**
     * 校验统计条目是否可删除
     *
     * @param id 统计条目主键
     * @return 统计条目do类
     * @author lixing
     * @version V1.0 2021-05-20 19:21:09
     */
    public StatisticItems checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除StatisticItems时发生异常，主键为空");
        }

        StatisticItems statisticItems = getById(id);

        if (statisticItems == null) {
            throw new IllegalArgumentException("删除StatisticItems时发生异常，找不到要删除的数据，id:" + id);
        }

        return statisticItems;
    }

    /**
     * 删除统计条目
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-20 19:21:09
     */
    @Override
    public void deleteStatisticItems(String id) {
        // 校验是否可删除
        StatisticItems statisticItems = checkDeletable(id);

        statisticItems.setValid(ValidEnum.FALSE.getType());
        updateById(statisticItems);
    }

    /**
     * 查询统计条目
     *
     * @return List<StatisticItems>
     * @author lixing
     * @version V1.0 2021-05-20 19:21:09
     */
    @Override
    public List<StatisticItems> queryStatisticItemsList(QueryStatisticItemsDTO queryStatisticItemsDTO) {
        QueryWrapper<StatisticItems> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StatisticItems.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, StatisticItems.PROP_CREATIONTIME);

        if (queryStatisticItemsDTO != null) {

            if (StringUtils.isNotEmpty(queryStatisticItemsDTO.getItemCode())) {
                queryWrapper.eq(StatisticItems.PROP_ITEM_CODE, queryStatisticItemsDTO.getItemCode());
            }

            if (StringUtils.isNotEmpty(queryStatisticItemsDTO.getItemSource())) {
                queryWrapper.eq(StatisticItems.PROP_ITEM_SOURCE, queryStatisticItemsDTO.getItemSource());
            }

            if (StringUtils.isNotEmpty(queryStatisticItemsDTO.getStandardValue())) {
                queryWrapper.eq(StatisticItems.PROP_STANDARD_VALUE, queryStatisticItemsDTO.getStandardValue());
            }

            if (StringUtils.isNotEmpty(queryStatisticItemsDTO.getQualifyFormula())) {
                queryWrapper.eq(StatisticItems.PROP_QUALIFY_FORMULA, queryStatisticItemsDTO.getQualifyFormula());
            }

            if (queryStatisticItemsDTO.getStatisticItemIdList() != null) {
                queryWrapper.in(StatisticItems.PROP_ID, queryStatisticItemsDTO.getStatisticItemIdList());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询统计条目
     *
     * @return IPage<StatisticItems>
     * @author lixing
     * @version V1.0 2021-05-20 19:21:09
     */
    @Override
    public IPage<StatisticItems> pageQueryStatisticItems(PageQueryStatisticItemsDTO pageQueryStatisticItemsDTO) {
        QueryWrapper<StatisticItems> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(StatisticItems.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<StatisticItems> pageParam = new Page<>(pageQueryStatisticItemsDTO.getPage(), pageQueryStatisticItemsDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryStatisticItemsDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, StatisticItems.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryStatisticItemsDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryStatisticItemsDTO.getItemCode())) {
            queryWrapper.like(StatisticItems.PROP_ITEM_CODE, pageQueryStatisticItemsDTO.getItemCode());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryStatisticItemsDTO.getItemSource())) {
            queryWrapper.like(StatisticItems.PROP_ITEM_SOURCE, pageQueryStatisticItemsDTO.getItemSource());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryStatisticItemsDTO.getStandardValue())) {
            queryWrapper.like(StatisticItems.PROP_STANDARD_VALUE, pageQueryStatisticItemsDTO.getStandardValue());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryStatisticItemsDTO.getQualifyFormula())) {
            queryWrapper.like(StatisticItems.PROP_QUALIFY_FORMULA, pageQueryStatisticItemsDTO.getQualifyFormula());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
