package com.persagy.apm.energy.report.monthly.config.type.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.config.type.model.vo.*;
import com.persagy.apm.energy.report.monthly.config.type.model.dto.*;

import java.util.List;

/**
 * 报告类型(ReportType) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-19 10:39:18 2021-05-19 10:39:18
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertReportTypeTool {

    ConvertReportTypeTool INSTANCE = Mappers.getMapper(ConvertReportTypeTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param reportType do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-05-19 10:39:18
     */
    ResponseReportTypeItemVO convert2ResponseItemDTO(ReportType reportType);

    /**
     * do转换为ResponseListItemVO
     *
     * @param reportType do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-05-19 10:39:18
     */
    ResponseReportTypeListItemVO convert2ResponseListItemDTO(ReportType reportType);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param reportTypeList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-05-19 10:39:18
     */
    List<ResponseReportTypeListItemVO> convert2List(List<ReportType> reportTypeList);

    /**
     * addDTO转换为do
     *
     * @param addReportTypeDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-05-19 10:39:18
     */
    ReportType convertAddDto2Entity(AddReportTypeDTO addReportTypeDTO);

    /**
     * updateDTO转换为实体
     *
     * @param reportType          更新前的do对象
     * @param updateReportTypeDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-05-19 10:39:18
     */
    ReportType convertUpdateDto2Entity(@MappingTarget ReportType reportType, UpdateReportTypeDTO updateReportTypeDTO);
}

