package com.persagy.apm.energy.report.monthly.detail.business.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.*;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.*;

import java.util.List;

/**
 * 报告详情(ReportBusinessDetail) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:08:35 2021-05-17 11:08:35
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertReportBusinessDetailTool {

    ConvertReportBusinessDetailTool INSTANCE = Mappers.getMapper(ConvertReportBusinessDetailTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param reportBusinessDetail do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-05-17 11:08:35
     */
    ResponseReportBusinessDetailItemVO convert2ResponseItemDTO(ReportBusinessDetail reportBusinessDetail);

    /**
     * do转换为ResponseListItemVO
     *
     * @param reportBusinessDetail do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-05-17 11:08:35
     */
    ResponseReportBusinessDetailListItemVO convert2ResponseListItemDTO(ReportBusinessDetail reportBusinessDetail);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param reportBusinessDetailList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-05-17 11:08:35
     */
    List<ResponseReportBusinessDetailListItemVO> convert2List(List<ReportBusinessDetail> reportBusinessDetailList);

    /**
     * addDTO转换为do
     *
     * @param addReportBusinessDetailDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-05-17 11:08:35
     */
    ReportBusinessDetail convertAddDto2Entity(AddReportBusinessDetailDTO addReportBusinessDetailDTO);

    /**
     * updateDTO转换为实体
     *
     * @param reportBusinessDetail          更新前的do对象
     * @param updateReportBusinessDetailDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-05-17 11:08:35
     */
    ReportBusinessDetail convertUpdateDto2Entity(@MappingTarget ReportBusinessDetail reportBusinessDetail, UpdateReportBusinessDetailDTO updateReportBusinessDetailDTO);
}

