package com.persagy.apm.energy.report.monthly.detail.business.service.impl;

import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.ReportTypeStatisticsItemRel;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.model.dto.QueryReportTypeStatisticsItemRelDTO;
import com.persagy.apm.energy.report.monthly.config.rel.typestatisticsitem.service.IReportTypeStatisticsItemRelService;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.dto.QueryStatisticItemsDTO;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.service.IStatisticItemsService;
import com.persagy.apm.energy.report.monthly.config.type.model.ReportType;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeService;
import com.persagy.apm.energy.report.monthly.detail.business.model.Platform;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.detail.business.service.IBusinessReportPlatformInfoService;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 商业类报告平台运维信息service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 10:22 下午
 **/
@Service
public class BusinessReportPlatformInfoServiceImpl implements IBusinessReportPlatformInfoService {
    @Autowired
    private IReportTypeStatisticsItemRelService reportTypeStatisticsItemRelService;
    @Autowired
    private IStatisticItemsService statisticItemsService;
    @Autowired
    private CommonService commonService;
    @Autowired
    private IReportOutlineService reportOutlineService;
    @Autowired
    private IReportTypeService reportTypeService;

    @Value("${group.code}")
    private String groupCode;

    @Override
    public List<Platform> getPlatforms(ReportOutline reportOutline) {
        // 查询报告类型关联的统计条目
        QueryReportTypeStatisticsItemRelDTO queryReportTypeStatisticsItemRelDTO = new QueryReportTypeStatisticsItemRelDTO();
        String reportTypeId = reportOutline.getReportTypeId();
        queryReportTypeStatisticsItemRelDTO.setReportTypeId(reportTypeId);
        List<ReportTypeStatisticsItemRel> reportTypeStatisticsItemRels = reportTypeStatisticsItemRelService.
                queryReportTypeStatisticsItemRelList(queryReportTypeStatisticsItemRelDTO);
        if (CollectionUtils.isEmpty(reportTypeStatisticsItemRels)) {
            return Lists.newArrayList();
        }
        List<String> statisticItemIds = reportTypeStatisticsItemRels.stream().
                map(ReportTypeStatisticsItemRel::getStatisticsItemId).
                collect(Collectors.toList());

        // 查询条目对象
        QueryStatisticItemsDTO queryStatisticItemsDTO = new QueryStatisticItemsDTO();
        queryStatisticItemsDTO.setStatisticItemIdList(statisticItemIds);
        List<StatisticItems> statisticItemList = statisticItemsService.queryStatisticItemsList(queryStatisticItemsDTO);
        if (CollectionUtils.isEmpty(statisticItemList)) {
            return Lists.newArrayList();
        }

        // 获取报告关联的项目
        List<String> projectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);

        // 获取报告类型的所属类型
        ReportType reportType = reportTypeService.queryReportTypeDetail(reportTypeId);
        String belongType = reportType.getBelongType();

        // 拼装运维信息查询条件
        QueryPlatformParamDTO queryPlatformParamDTO = new QueryPlatformParamDTO();
        queryPlatformParamDTO.setReportDate(reportOutline.getReportMonth());
        queryPlatformParamDTO.setProjectIdList(projectIds);
        queryPlatformParamDTO.setGroupCode(groupCode);
        queryPlatformParamDTO.setUserId(PoemsContext.getContext().getUserId());
        queryPlatformParamDTO.setBelongType(belongType);

        List<Platform> result = new ArrayList<>();
        for (StatisticItems statisticItems : statisticItemList) {
            result.add(commonService.getPlatform(statisticItems, queryPlatformParamDTO));
        }

        return result;
    }
}
