package com.persagy.apm.energy.report.monthly.detail.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.ConvertReportDetailStatisticsItemRelTool;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.ReportDetailStatisticsItemRel;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.dto.AddReportDetailStatisticsItemRelDTO;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.dto.QueryReportDetailStatisticsItemRelDTO;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.model.vo.ResponseReportDetailStatisticsItemRelListItemVO;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.service.IReportDetailStatisticsItemRelService;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BelongTypeEnum;
import com.persagy.apm.energy.report.monthly.config.type.model.ReportType;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeService;
import com.persagy.apm.energy.report.monthly.detail.business.dao.ReportBusinessDetailMapper;
import com.persagy.apm.energy.report.monthly.detail.business.model.ConvertReportBusinessDetailTool;
import com.persagy.apm.energy.report.monthly.detail.business.model.ReportBusinessDetail;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.AddReportBusinessDetailDTO;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.PageQueryReportBusinessDetailDTO;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.QueryReportBusinessDetailDTO;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.UpdateReportBusinessDetailDTO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.ResponseReportBusinessDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.business.service.IReportBusinessDetailService;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportAreaVO;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 商业报告详情(ReportBusinessDetail) service层
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:08:34
 */
@Service
public class ReportBusinessDetailServiceImpl extends ServiceImpl<ReportBusinessDetailMapper, ReportBusinessDetail>
        implements IReportBusinessDetailService {
    @Autowired
    IReportDetailStatisticsItemRelService reportDetailStatisticsItemRelService;
    @Autowired
    private IReportOutlineService reportOutlineService;
    @Autowired
    private IReportTypeService reportTypeService;
    @Autowired
    private ISaasWebService saasWebService;

    /**
     * 创建报告详情
     *
     * @return 报告详情主键
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public String createReportBusinessDetail(AddReportBusinessDetailDTO addReportBusinessDetailDTO) {
        ReportBusinessDetail reportBusinessDetail = ConvertReportBusinessDetailTool.INSTANCE.convertAddDto2Entity(addReportBusinessDetailDTO);
        // 设置默认值
        setDefaultValue(reportBusinessDetail);
        save(reportBusinessDetail);
        return reportBusinessDetail.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportBusinessDetail 报告详情实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportBusinessDetail reportBusinessDetail) {
        reportBusinessDetail.setCreator(PoemsContext.getContext().getUserId());
        // todo 其他默认的属性

    }

    @Override
    public ReportBusinessDetail getReportBusinessDetail(String id) {
        ReportBusinessDetail reportBusinessDetail = getById(id);
        if (reportBusinessDetail == null) {
            throw new IllegalArgumentException("查看ReportBusinessDetail详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportBusinessDetail;
    }

    /**
     * 更新报告详情
     *
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateReportBusinessDetail(UpdateReportBusinessDetailDTO updateReportBusinessDetailDTO) {
        String reportOutlineId = updateReportBusinessDetailDTO.getReportOutlineId();
        ReportOutline reportOutline = reportOutlineService.queryReportOutlineDetail(reportOutlineId);
        ReportBusinessDetail reportBusinessDetail = getById(reportOutline.getDetailId());
        reportBusinessDetail = ConvertReportBusinessDetailTool.INSTANCE.convertUpdateDto2Entity(reportBusinessDetail, updateReportBusinessDetailDTO);
        reportBusinessDetail.setModifier(PoemsContext.getContext().getUserId());
        updateById(reportBusinessDetail);
        // 更新统计条目信息，先删后加
        QueryReportDetailStatisticsItemRelDTO queryReportDetailStatisticsItemRelDTO = new QueryReportDetailStatisticsItemRelDTO();
        queryReportDetailStatisticsItemRelDTO.setReportDetailId(reportOutline.getDetailId());
        queryReportDetailStatisticsItemRelDTO.setReportTypeId(reportOutline.getReportTypeId());
        List<ReportDetailStatisticsItemRel> reportDetailStatisticsItemRels = reportDetailStatisticsItemRelService.queryReportDetailStatisticsItemRelList(queryReportDetailStatisticsItemRelDTO);
        for (ReportDetailStatisticsItemRel reportDetailStatisticsItemRel : reportDetailStatisticsItemRels) {
            reportDetailStatisticsItemRelService.deleteReportDetailStatisticsItemRel(reportDetailStatisticsItemRel.getId());
        }

        List<AddReportDetailStatisticsItemRelDTO> statisticItemsSupplements = updateReportBusinessDetailDTO.getStatisticItemsSupplements();
        for (AddReportDetailStatisticsItemRelDTO statisticItemsSupplement : statisticItemsSupplements) {
            reportDetailStatisticsItemRelService.createReportDetailStatisticsItemRel(statisticItemsSupplement);
        }
    }

    /**
     * 校验报告详情是否可删除
     *
     * @param id 报告详情主键
     * @return 报告详情do类
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    public ReportBusinessDetail checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportBusinessDetail时发生异常，主键为空");
        }

        ReportBusinessDetail reportBusinessDetail = getById(id);

        if (reportBusinessDetail == null) {
            throw new IllegalArgumentException("删除ReportBusinessDetail时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportBusinessDetail;
    }

    /**
     * 删除报告详情
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public void deleteReportBusinessDetail(String id) {
        if (StringUtils.isNotBlank(id)) {
            removeById(id);
        }
    }

    /**
     * 查询报告详情
     *
     * @return List<ReportBusinessDetail>
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public List<ReportBusinessDetail> queryReportBusinessDetailList(QueryReportBusinessDetailDTO queryReportBusinessDetailDTO) {
        QueryWrapper<ReportBusinessDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportBusinessDetail.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportBusinessDetail.PROP_CREATIONTIME);

        if (queryReportBusinessDetailDTO != null) {

            if (queryReportBusinessDetailDTO.getOpenPower() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_POWER, queryReportBusinessDetailDTO.getOpenPower());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getOpenPowerExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_OPEN_POWER_EXPLAIN, queryReportBusinessDetailDTO.getOpenPowerExplain());
            }

            if (queryReportBusinessDetailDTO.getComparePower() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_POWER, queryReportBusinessDetailDTO.getComparePower());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getComparePowerExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_POWER_EXPLAIN, queryReportBusinessDetailDTO.getComparePowerExplain());
            }

            if (queryReportBusinessDetailDTO.getOpenCost() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_COST, queryReportBusinessDetailDTO.getOpenCost());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getOpenCostExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_OPEN_COST_EXPLAIN, queryReportBusinessDetailDTO.getOpenCostExplain());
            }

            if (queryReportBusinessDetailDTO.getCompareCost() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_COST, queryReportBusinessDetailDTO.getCompareCost());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getCompareCostExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_COST_EXPLAIN, queryReportBusinessDetailDTO.getCompareCostExplain());
            }

            if (queryReportBusinessDetailDTO.getJob() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_JOB, queryReportBusinessDetailDTO.getJob());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getRemark())) {
                queryWrapper.like(ReportBusinessDetail.PROP_REMARK, queryReportBusinessDetailDTO.getRemark());
            }

            if (queryReportBusinessDetailDTO.getPlatform() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_PLATFORM, queryReportBusinessDetailDTO.getPlatform());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportBusinessDetailDTO.getItemExplain())) {
                queryWrapper.like(ReportBusinessDetail.PROP_ITEM_EXPLAIN, queryReportBusinessDetailDTO.getItemExplain());
            }

            if (queryReportBusinessDetailDTO.getProjectIds() != null) {
                queryWrapper.eq(ReportBusinessDetail.PROP_PROJECT_IDS, queryReportBusinessDetailDTO.getProjectIds());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告详情
     *
     * @return IPage<ReportBusinessDetail>
     * @author lixing
     * @version V1.0 2021-05-17 11:08:34
     */
    @Override
    public IPage<ReportBusinessDetail> pageQueryReportBusinessDetail(PageQueryReportBusinessDetailDTO pageQueryReportBusinessDetailDTO) {
        QueryWrapper<ReportBusinessDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportBusinessDetail.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportBusinessDetail> pageParam = new Page<>(pageQueryReportBusinessDetailDTO.getPage(), pageQueryReportBusinessDetailDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportBusinessDetailDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportBusinessDetail.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportBusinessDetailDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (pageQueryReportBusinessDetailDTO.getOpenPower() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_POWER, pageQueryReportBusinessDetailDTO.getOpenPower());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getOpenPowerExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_OPEN_POWER_EXPLAIN, pageQueryReportBusinessDetailDTO.getOpenPowerExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getComparePower() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_POWER, pageQueryReportBusinessDetailDTO.getComparePower());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getComparePowerExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_POWER_EXPLAIN, pageQueryReportBusinessDetailDTO.getComparePowerExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getOpenCost() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_OPEN_COST, pageQueryReportBusinessDetailDTO.getOpenCost());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getOpenCostExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_OPEN_COST_EXPLAIN, pageQueryReportBusinessDetailDTO.getOpenCostExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getCompareCost() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_COMPARE_COST, pageQueryReportBusinessDetailDTO.getCompareCost());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getCompareCostExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_COMPARE_COST_EXPLAIN, pageQueryReportBusinessDetailDTO.getCompareCostExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getJob() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_JOB, pageQueryReportBusinessDetailDTO.getJob());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getRemark())) {
            queryWrapper.like(ReportBusinessDetail.PROP_REMARK, pageQueryReportBusinessDetailDTO.getRemark());
        }

        if (pageQueryReportBusinessDetailDTO.getPlatform() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_PLATFORM, pageQueryReportBusinessDetailDTO.getPlatform());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportBusinessDetailDTO.getItemExplain())) {
            queryWrapper.like(ReportBusinessDetail.PROP_ITEM_EXPLAIN, pageQueryReportBusinessDetailDTO.getItemExplain());
        }

        if (pageQueryReportBusinessDetailDTO.getProjectIds() != null) {
            queryWrapper.eq(ReportBusinessDetail.PROP_PROJECT_IDS, pageQueryReportBusinessDetailDTO.getProjectIds());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public ResponseReportBusinessDetailItemVO getResponseBusinessDetail(String reportOutlineId) {
        ReportOutline reportOutline = reportOutlineService.queryReportOutlineDetail(reportOutlineId);
        String reportTypeId = reportOutline.getReportTypeId();
        ReportType reportType = reportTypeService.queryReportTypeDetail(reportTypeId);
        String buildingType = reportType.getBuildingType();
        ReportBusinessDetail reportBusinessDetail = getReportBusinessDetail(reportOutline.getDetailId());
        ResponseReportBusinessDetailItemVO responseReportBusinessDetailItemVO = ConvertReportBusinessDetailTool.INSTANCE.
                convert2ResponseItemDTO(reportBusinessDetail);

        if (BelongTypeEnum.PROJECT.getType().equals(reportType.getBelongType())) {
            ReportProjectVO reportProjectInfo = saasWebService.getReportProjectInfo(reportOutline.getBelong());
            responseReportBusinessDetailItemVO.setProjectInfo(reportProjectInfo);
        } else {
            ReportAreaVO openReportAreaInfo = saasWebService.getReportAreaInfo(reportOutline.getBelong(), buildingType);
            responseReportBusinessDetailItemVO.setOpenAreaInfo(openReportAreaInfo);

            ReportAreaVO compareReportAreaInfo = saasWebService.getCompareReportAreaInfo(
                    reportOutline.getBelong(), buildingType, reportOutline.getReportMonth());
            responseReportBusinessDetailItemVO.setCompareAreaInfo(compareReportAreaInfo);
        }


        // 查询统计条目补充信息
        QueryReportDetailStatisticsItemRelDTO queryReportDetailStatisticsItemRelDTO = new QueryReportDetailStatisticsItemRelDTO();
        queryReportDetailStatisticsItemRelDTO.setReportDetailId(reportOutline.getDetailId());
        queryReportDetailStatisticsItemRelDTO.setReportTypeId(reportTypeId);
        List<ReportDetailStatisticsItemRel> reportDetailStatisticsItemRels = reportDetailStatisticsItemRelService.queryReportDetailStatisticsItemRelList(queryReportDetailStatisticsItemRelDTO);
        List<ResponseReportDetailStatisticsItemRelListItemVO> responseReportDetailStatisticsItemRelListItemVOList = ConvertReportDetailStatisticsItemRelTool.INSTANCE.convert2List(reportDetailStatisticsItemRels);
        responseReportBusinessDetailItemVO.setStatisticItemsSupplements(responseReportDetailStatisticsItemRelListItemVOList);
        return responseReportBusinessDetailItemVO;
    }

}
