package com.persagy.apm.energy.report.monthly.detail.common.service;

import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.CostItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.CostVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.GroupInfo;

import java.util.Date;
import java.util.List;

/**
 * 报告费用信息service接口
 *
 * @author lixing
 * @version V1.0 2021/5/30 10:26 下午
 **/
public interface IReportCostInfoService {
    /**
     * 根据项目id获取费用信息
     *
     * @param groupIdList    费用信息点分组列表
     * @param functionIdList 费用信息点列表
     * @param reportMonth    报告月份
     * @param projectIds     项目id列表
     * @return 费用信息
     * @author lixing
     * @version V1.0 2021/5/30 11:41 下午
     */
    CostVO getCostInfoByProjectIds(List<String> groupIdList, List<String> functionIdList, Date reportMonth, List<String> projectIds);

    /**
     * 获取费用条目信息
     *
     * @param functionIdList 费用条目列表
     * @param reportMonth    报告月份
     * @param projectIds     项目id列表
     * @return 费用条目列表
     * @author lixing
     * @version V1.0 2021/5/24 7:15 下午
     */
    List<CostItemVO> getCostItemInfos(List<String> functionIdList, Date reportMonth, List<String> projectIds);

    /**
     * 获取一组项目一个功能点的费用信息
     *
     * @param reportMonth 报告月份
     * @param projectIds  一组项目id
     * @param function    功能点（条目）
     * @return 一组项目一个功能点的费用信息
     * @author lixing
     * @version V1.0 2021/5/30 5:55 下午
     */
    List<CostItemVO> getCostItemsByProjects(Date reportMonth, List<String> projectIds, Function function);

    /**
     * 获取统计费用需要使用的项目面积
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 统计费用需要使用的项目面积
     * @author lixing
     * @version V1.0 2021/6/9 7:51 下午
     */
    Double getProjectBuildingArea(List<String> projectIds);

    /**
     * 获取一个条目的费用信息
     *
     * @param reportMonth      报告月份
     * @param projectId        项目id
     * @param costBuildingArea 统计费用使用的建筑面积
     * @param function         功能点
     * @return 费用对象
     * @author lixing
     * @version V1.0 2021/5/30 5:54 下午
     */
    CostItemVO getCostItemInfo(
            Date reportMonth, String projectId,
            Double costBuildingArea, Function function);

    /**
     * 求一组费用条目的统计值
     *
     * @param costItems 一组费用条目
     * @return 一个新的费用条目对象，将统计值属性赋值给它
     * @author lixing
     * @version V1.0 2021/5/30 10:46 上午
     */
    CostItemVO getCostSum(List<CostItemVO> costItems);

    /**
     * 获取费用分组信息
     *
     * @param groupIdList 分组id列表
     * @param reportMonth 报告月份
     * @param projectIds  报告涉及的项目id列表
     * @return 分组对象列表
     * @author lixing
     * @version V1.0 2021/5/24 7:12 下午
     */
    List<GroupInfo<CostItemVO>> getCostGroupInfos(List<String> groupIdList, Date reportMonth, List<String> projectIds);
}
