package com.persagy.apm.energy.report.monthly.detail.common.service;

import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.GroupInfo;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerVO;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/**
 * 报告用电量信息服务接口
 *
 * @author lixing
 * @version V1.0 2021/5/30 10:25 下午
 **/
public interface IReportPowerInfoService {
    /**
     * 获取项目用电量
     *
     * @param reportOutline  报告outline对象
     * @param groupIdList    用电量信息点分组列表
     * @param functionIdList 用电量信息点列表
     * @return 项目用电量
     * @author lixing
     * @version V1.0 2021/5/24 4:14 下午
     */
    PowerVO getPowerInfo(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList);

    /**
     * 获取可同比项目用电量
     *
     * @param reportOutline  报告outline对象
     * @param groupIdList    用电量信息点分组列表
     * @param functionIdList 用电量信息点列表
     * @return 项目用电量
     * @author lixing
     * @version V1.0 2021/5/24 4:14 下午
     */
    PowerVO getComparePowerInfo(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList);

    /**
     * 根据项目id获取用电信息
     *
     * @param groupIdList    费用信息点分组列表
     * @param functionIdList 费用信息点列表
     * @param reportMonth    报告月份
     * @param projectIds     项目id列表
     * @return 用电信息
     * @author lixing
     * @version V1.0 2021/5/30 11:41 下午
     */
    PowerVO getPowerInfoByProjectIds(
            List<String> groupIdList, List<String> functionIdList,
            Date reportMonth, List<String> projectIds);

    /**
     * 获取用电量条目信息
     *
     * @param functionIdList 用电量条目列表
     * @param reportMonth    报告月份
     * @param projectIds     项目id列表
     * @return 用电量条目列表
     * @author lixing
     * @version V1.0 2021/5/24 7:15 下午
     */
    List<PowerItemVO> getPowerItemInfos(List<String> functionIdList, Date reportMonth, List<String> projectIds);

    /**
     * 获取一组项目一个功能点的用电量信息
     *
     * @param reportMonth 报告月份
     * @param projectIds  一组项目id
     * @param function    功能点（条目）
     * @return 一组项目一个功能点的用电量信息
     * @author lixing
     * @version V1.0 2021/5/30 5:55 下午
     */
    List<PowerItemVO> getPowerItemsByProjects(Date reportMonth, List<String> projectIds, Function function);

    /**
     * 获取一个条目的用电量信息
     *
     * @param reportMonth       报告月份
     * @param reportProjectInfo 报告项目信息
     * @param dateValueMap      日期 -> 用电量
     * @return 用电量对象
     * @author lixing
     * @version V1.0 2021/5/30 5:54 下午
     */
    PowerItemVO getPowerItemInfo(Date reportMonth, Double reportProjectInfo, TreeMap<Date, Double> dateValueMap);

    /**
     * 获取条目能耗值map
     *
     * @param reportMonth    报告月份
     * @param itemId         条目id
     * @param modelCode      模型编码
     * @param projectLocalId 项目本地编码
     * @return 分项每月能耗信息{分项id:{时间(Date)：数据(Double)}
     * @author lixing
     * @version V1.0 2021/5/30 11:39 上午
     */
    Map<String, TreeMap<Date, Double>> getItemEnergyDataMap(
            Date reportMonth, String itemId, String modelCode, String projectLocalId);

    /**
     * 求一组用电量条目的统计值
     *
     * @param powerItems 一组用电量条目
     * @return 一个新的用电量条目对象，将统计值属性赋值给它
     * @author lixing
     * @version V1.0 2021/5/30 10:46 上午
     */
    PowerItemVO getPowerSum(List<PowerItemVO> powerItems);

    /**
     * 获取用电量分组信息
     *
     * @param groupIdList 分组id列表
     * @param reportMonth 报告月份
     * @param projectIds  报告涉及的项目id列表
     * @return 分组对象列表
     * @author lixing
     * @version V1.0 2021/5/24 7:12 下午
     */
    List<GroupInfo<PowerItemVO>> getPowerGroupInfos(List<String> groupIdList, Date reportMonth, List<String> projectIds);

    /**
     * 获取统计费用需要使用的项目面积
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 统计费用需要使用的项目面积
     * @author lixing
     * @version V1.0 2021/6/9 7:51 下午
     */
    Double getProjectBuildingArea(List<String> projectIds);
}
