package com.persagy.apm.energy.report.monthly.detail.common.service.impl;

import com.persagy.apm.energy.report.centermiddleware.constant.enums.TimeTypeEnum;
import com.persagy.apm.energy.report.centermiddleware.model.dto.QueryEnergyDataDTO;
import com.persagy.apm.energy.report.centermiddleware.service.ICenterMiddlewareWebService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.monthly.config.function.model.Function;
import com.persagy.apm.energy.report.monthly.config.function.service.IFunctionService;
import com.persagy.apm.energy.report.monthly.config.functiongroup.model.FunctionGroup;
import com.persagy.apm.energy.report.monthly.config.functiongroup.service.IFunctionGroupService;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.GroupInfo;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerVO;
import com.persagy.apm.energy.report.monthly.detail.common.service.IReportPowerInfoService;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 商业类报告用电量service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 7:30 下午
 **/
@Service
public abstract class ReportPowerInfoServiceImpl implements IReportPowerInfoService {
    @Autowired
    private IReportOutlineService reportOutlineService;
    @Autowired
    private IFunctionGroupService functionGroupService;
    @Autowired
    private IFunctionService functionService;
    @Autowired
    private ICenterMiddlewareWebService centerMiddlewareWebService;
    @Autowired
    private ISaasWebService saasWebService;

    @Override
    public PowerVO getPowerInfo(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList) {
        Date reportMonth = reportOutline.getReportMonth();

        List<String> projectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);

        return getPowerInfoByProjectIds(groupIdList, functionIdList, reportMonth, projectIds);
    }

    @Override
    public PowerVO getComparePowerInfo(ReportOutline reportOutline, List<String> groupIdList, List<String> functionIdList) {
        Date reportMonth = reportOutline.getReportMonth();

        List<String> projectIds = reportOutlineService.queryRelatedProjectIds(reportOutline);

        List<String> comparableProjects = saasWebService.getComparableProjects(projectIds, reportMonth);

        return getPowerInfoByProjectIds(groupIdList, functionIdList, reportMonth, comparableProjects);
    }

    @Override
    public PowerVO getPowerInfoByProjectIds(
            List<String> groupIdList, List<String> functionIdList,
            Date reportMonth, List<String> projectIds) {
        PowerVO powerVO = new PowerVO();
        List<GroupInfo<PowerItemVO>> powerGroupInfos = getPowerGroupInfos(groupIdList, reportMonth, projectIds);
        powerVO.setGroups(powerGroupInfos);

        List<PowerItemVO> powerItemInfos = getPowerItemInfos(functionIdList, reportMonth, projectIds);
        powerVO.setItems(powerItemInfos);

        List<PowerItemVO> paragraphSums = new ArrayList<>();

        if (!CollectionUtils.isEmpty(powerGroupInfos)) {
            paragraphSums.addAll(
                    powerGroupInfos.stream().map(GroupInfo::getSummary).collect(Collectors.toList()));
        }
        if (!CollectionUtils.isEmpty(powerItemInfos)) {
            paragraphSums.addAll(powerItemInfos);
        }
        powerVO.setSummary(getPowerSum(paragraphSums));
        return powerVO;
    }

    @Override
    public List<PowerItemVO> getPowerItemInfos(List<String> functionIdList, Date reportMonth, List<String> projectIds) {
        List<PowerItemVO> powerItemVOList = Lists.newArrayList();
        if (CollectionUtils.isEmpty(functionIdList) || CollectionUtils.isEmpty(projectIds)) {
            return powerItemVOList;
        }

        for (String functionId : functionIdList) {
            Function function = functionService.queryFunctionDetail(functionId);
            String itemName = functionService.getItemName(function, projectIds.get(0));

            List<PowerItemVO> projectPowerItems = getPowerItemsByProjects(reportMonth, projectIds, function);
            PowerItemVO powerSumByProjects = getPowerSum(projectPowerItems);
            powerSumByProjects.setName(itemName);
            powerItemVOList.add(powerSumByProjects);
        }
        return powerItemVOList;
    }

    @Override
    public List<PowerItemVO> getPowerItemsByProjects(Date reportMonth, List<String> projectIds, Function function) {
        List<PowerItemVO> projectPowerItems = Lists.newArrayList();
        if (CollectionUtils.isEmpty(projectIds)) {
            return projectPowerItems;
        }

        // 统计项目的面积
        Double projectBuildingArea = getProjectBuildingArea(projectIds);

        for (String projectId : projectIds) {
            // 获取项目信息
            ReportProjectVO reportProjectInfo = saasWebService.getReportProjectInfo(projectId);
            if (reportProjectInfo == null) {
                continue;
            }
            Map<String, TreeMap<Date, Double>> itemEnergyDataMap = getItemEnergyDataMap(
                    reportMonth, function.getItemId(), function.getModelCode(), reportProjectInfo.getProjectLocalID());
            TreeMap<Date, Double> dateValueMap = itemEnergyDataMap.get(function.getItemId());
            if (dateValueMap != null) {
                PowerItemVO powerItemInfo = getPowerItemInfo(reportMonth, projectBuildingArea, dateValueMap);
                projectPowerItems.add(powerItemInfo);
            }
        }
        return projectPowerItems;
    }

    @Override
    public PowerItemVO getPowerItemInfo(Date reportMonth, Double buildingArea, TreeMap<Date, Double> dateValueMap) {
        PowerItemVO powerItemVO = new PowerItemVO();
        Double currentMonth = dateValueMap.get(reportMonth);
        if (buildingAreaIsValid(buildingArea) && currentMonth != null) {
            powerItemVO.setCurrentMonthPerSqm(DataUtils.doubleDivide(currentMonth, buildingArea));
        }
        powerItemVO.setCurrentMonth(currentMonth);
        Double lastMonth = dateValueMap.get(DateUtils.getFirstDayOfLastMonth(reportMonth));
        powerItemVO.setLastMonth(lastMonth);
        Double lastYearSameTime = dateValueMap.get(DateUtils.getSameMonthFirstDayOfLastYear(reportMonth));
        powerItemVO.setLastYearSameTime(lastYearSameTime);
        // 去年同期每平方米耗电量
        Double lastYearSameTimePerSqm = null;
        if (buildingAreaIsValid(buildingArea) && lastYearSameTime != null) {
            lastYearSameTimePerSqm = DataUtils.doubleDivide(lastYearSameTime, buildingArea);
            powerItemVO.setLastYearSameTimePerSqm(lastYearSameTimePerSqm);
        }

        if (lastYearSameTime != null && currentMonth != null && buildingAreaIsValid(buildingArea)) {
            Double sameTimeGrowth = DataUtils.doubleSubtract(currentMonth, lastYearSameTime);
            powerItemVO.setSameTimeGrowth(sameTimeGrowth);
            Double sameTimeRange = DataUtils.doubleDivide(sameTimeGrowth, lastYearSameTime);
            powerItemVO.setSameTimeRange(sameTimeRange);
            Double sameTimeGrowthPerSqm = DataUtils.doubleDivide(sameTimeGrowth, buildingArea);
            powerItemVO.setSameTimeGrowthPerSqm(sameTimeGrowthPerSqm);
            if (lastYearSameTimePerSqm != null) {
                Double sameTimeRangePerSqm = DataUtils.doubleDivide(sameTimeGrowthPerSqm, lastYearSameTimePerSqm);
                powerItemVO.setSameTimeRangePerSqm(sameTimeRangePerSqm);
            }
        }

        // 统计去年同期的用电量
        Double lastYearCount = null;
        List<Date> lastYearMonths = DateUtils.getFirstDayOfEveryMonth(
                DateUtils.getFirstDayOfLastYear(reportMonth),
                DateUtils.getSameMonthFirstDayOfLastYear(reportMonth));
        List<PowerItemVO> lastYearItems = new ArrayList<>();
        for (Date lastYearMonth : lastYearMonths) {
            PowerItemVO tmp = new PowerItemVO();
            tmp.setLastYearCount(dateValueMap.get(lastYearMonth));
            lastYearItems.add(tmp);
        }

        PowerItemVO lastYearSumItem = getPowerSum(lastYearItems);
        if (lastYearSumItem != null) {
            lastYearCount = lastYearSumItem.getLastYearCount();
            powerItemVO.setLastYearCount(lastYearCount);
            if (buildingAreaIsValid(buildingArea) && lastYearCount != null) {
                powerItemVO.setLastYearPerCount(DataUtils.doubleDivide(lastYearCount, buildingArea));
            }
        }

        // 统计今年的用电量
        Double yearCount = null;
        List<Date> yearMonths = DateUtils.getFirstDayOfEveryMonth(
                DateUtils.getFirstDayOfYear(reportMonth),
                reportMonth);
        List<PowerItemVO> yearItems = new ArrayList<>();
        for (Date yearMonth : yearMonths) {
            PowerItemVO tmp = new PowerItemVO();
            tmp.setYearCount(dateValueMap.get(yearMonth));
            yearItems.add(tmp);
        }

        PowerItemVO yearSumItem = getPowerSum(yearItems);
        if (yearSumItem != null) {
            yearCount = yearSumItem.getYearCount();
            powerItemVO.setYearCount(yearCount);
            if (buildingAreaIsValid(buildingArea) && yearCount != null) {
                powerItemVO.setYearPerCount(DataUtils.doubleDivide(yearCount, buildingArea));
            }
        }

        if (currentMonth != null && lastMonth != null) {
            Double linkCount = DataUtils.doubleSubtract(currentMonth, lastMonth);
            Double linkRange = DataUtils.doubleDivide(linkCount, lastMonth);
            powerItemVO.setLinkCount(linkCount);
            powerItemVO.setLinkRange(linkRange);
        }

        if (lastYearCount != null && yearCount != null) {
            double sameCount = DataUtils.doubleSubtract(yearCount, lastYearCount);
            powerItemVO.setSameCount(sameCount);
            powerItemVO.setSameRange(DataUtils.doubleDivide(sameCount, lastYearCount));
        }
        return powerItemVO;
    }

    /**
     * 判断建筑面积是否合法（不为空，且不为0）
     *
     * @param buildingArea 建筑面积
     * @return 建筑面积是否合法
     * @author lixing
     * @version V1.0 2021/6/10 3:06 下午
     */
    private boolean buildingAreaIsValid(Double buildingArea) {
        Double zero = 0d;
        return buildingArea != null && !buildingArea.equals(zero);
    }

    @Override
    public Map<String, TreeMap<Date, Double>> getItemEnergyDataMap(
            Date reportMonth, String itemId, String modelCode, String projectLocalId) {
        QueryEnergyDataDTO queryEnergyDataDTO = new QueryEnergyDataDTO();
        queryEnergyDataDTO.setProjectId(projectLocalId);
        queryEnergyDataDTO.setBuildingId(projectLocalId);
        queryEnergyDataDTO.setModelCode(modelCode);
        // 去年一月
        queryEnergyDataDTO.setStartDate(DateUtils.date2Str(
                DateUtils.getFirstDayOfLastYear(reportMonth), DateUtils.SDF_SECOND));
        // 结束时间往后延一个月，中间件时间查询为左闭右开
        queryEnergyDataDTO.setEndDate(DateUtils.date2Str(DateUtils.getMonthOff(reportMonth, 1), DateUtils.SDF_SECOND));
        queryEnergyDataDTO.setItemIdList(Lists.newArrayList(itemId));
        queryEnergyDataDTO.setTimeType(TimeTypeEnum.MONTH.getType());
        return centerMiddlewareWebService.getItemEnergyDataMap(queryEnergyDataDTO);
    }

    @Override
    public PowerItemVO getPowerSum(List<PowerItemVO> powerItems) {
        return DataUtils.sumDoubleParams(powerItems, PowerItemVO.class);
    }

    @Override
    public List<GroupInfo<PowerItemVO>> getPowerGroupInfos(List<String> groupIdList, Date reportMonth, List<String> projectIds) {
        List<GroupInfo<PowerItemVO>> groupInfoList = Lists.newArrayList();
        if (CollectionUtils.isEmpty(groupIdList)) {
            return groupInfoList;
        }
        for (String groupId : groupIdList) {
            GroupInfo<PowerItemVO> groupInfo = new GroupInfo<>();
            FunctionGroup functionGroup = functionGroupService.queryFunctionGroupDetail(groupId);
            groupInfo.setGroupName(functionGroup.getName());
            // 获取组下的分组（本期不做）
            // 获取组下的分项
            String functionIdStr = functionGroup.getFunctionIds();
            List<String> functionIds = Lists.newArrayList(functionIdStr.split(","));
            List<PowerItemVO> powerItemInfos = getPowerItemInfos(functionIds, reportMonth, projectIds);
            groupInfo.setItems(powerItemInfos);
            groupInfo.setSummary(getPowerSum(powerItemInfos));
            groupInfoList.add(groupInfo);
        }
        return groupInfoList;
    }

    /**
     * 获取统计费用需要使用的项目面积
     *
     * @param projectIds 需要统计的项目的id列表
     * @return 统计费用需要使用的项目面积
     * @author lixing
     * @version V1.0 2021/6/9 7:51 下午
     */
    @Override
    public abstract Double getProjectBuildingArea(List<String> projectIds);
}
