package com.persagy.apm.energy.report.monthly.detail.hotel.area.model;

import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectPowerConsumptionOutline;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectPublicPowerConsumption;
import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.vo.*;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.dto.*;

import java.util.List;

/**
 * 商业报告详情(ReportHotelAreaDetail) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-06-07 09:42:08 2021-06-07 09:42:08
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertReportHotelAreaDetailTool {

    ConvertReportHotelAreaDetailTool INSTANCE = Mappers.getMapper(ConvertReportHotelAreaDetailTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param reportHotelAreaDetail do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-06-07 09:42:08
     */
    ResponseReportHotelAreaDetailItemVO convert2ResponseItemDTO(ReportHotelAreaDetail reportHotelAreaDetail);

    /**
     * do转换为ResponseListItemVO
     *
     * @param reportHotelAreaDetail do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-06-07 09:42:08
     */
    ResponseReportHotelAreaDetailListItemVO convert2ResponseListItemDTO(ReportHotelAreaDetail reportHotelAreaDetail);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param reportHotelAreaDetailList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-06-07 09:42:08
     */
    List<ResponseReportHotelAreaDetailListItemVO> convert2List(List<ReportHotelAreaDetail> reportHotelAreaDetailList);

    /**
     * addDTO转换为do
     *
     * @param addReportHotelAreaDetailDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-06-07 09:42:08
     */
    ReportHotelAreaDetail convertAddDto2Entity(AddReportHotelAreaDetailDTO addReportHotelAreaDetailDTO);

    /**
     * updateDTO转换为实体
     *
     * @param reportHotelAreaDetail          更新前的do对象
     * @param updateReportHotelAreaDetailDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-06-07 09:42:08
     */
    ReportHotelAreaDetail convertUpdateDto2Entity(
            @MappingTarget ReportHotelAreaDetail reportHotelAreaDetail, UpdateReportHotelAreaDetailDTO updateReportHotelAreaDetailDTO);

    /**
     * HotelProjectPowerConsumptionOutline 和 HotelProjectPublicPowerConsumption 转换为 HotelAreaPowerConsumptionOutline
     *
     * @param consumptionOutline HotelProjectPowerConsumptionOutline
     * @param publicConsumption HotelProjectPublicPowerConsumption
     * @return HotelAreaPowerConsumptionOutline
     * @author lixing
     * @version V1.0 2021/6/10 8:47 下午
     */
    @Mapping(source = "consumptionOutline.projectName", target = "projectName")
    @Mapping(source = "consumptionOutline.consumptionCurrentMonth", target = "consumptionCurrentMonth")
    @Mapping(source = "consumptionOutline.consumptionPerSqm", target = "consumptionPerSqm")
    @Mapping(source = "consumptionOutline.consumptionSameTimeRange", target = "consumptionSameTimeRange")
    @Mapping(source = "consumptionOutline.consumptionChainRange", target = "consumptionChainRange")
    @Mapping(source = "publicConsumption.consumptionCurrentMonth", target = "publicConsumptionCurrentMonth")
    HotelAreaPowerConsumptionOutline convert2HotelAreaPowerConsumptionOutline(
            HotelProjectPowerConsumptionOutline consumptionOutline, HotelProjectPublicPowerConsumption publicConsumption);
}

