package com.persagy.apm.energy.report.monthly.detail.hotel.area.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energy.report.monthly.detail.business.model.ConvertReportBusinessDetailTool;
import com.persagy.apm.energy.report.monthly.detail.business.model.ReportBusinessDetail;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.dao.ReportHotelAreaDetailMapper;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.vo.ResponseReportHotelAreaDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.service.IReportHotelAreaDetailService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.*;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 商业报告详情(ReportHotelAreaDetail) service层
 *
 * @author lixing
 * @version V1.0 2021-06-07 09:42:06
 */
@Service
public class ReportHotelAreaDetailServiceImpl extends ServiceImpl<ReportHotelAreaDetailMapper, ReportHotelAreaDetail>
        implements IReportHotelAreaDetailService {
    @Autowired
    private IReportOutlineService reportOutlineService;

    /**
     * 创建商业报告详情
     *
     * @return 商业报告详情主键
     * @author lixing
     * @version V1.0 2021-06-07 09:42:06
     */
    @Override
    public String createReportHotelAreaDetail(AddReportHotelAreaDetailDTO addReportHotelAreaDetailDTO) {
        ReportHotelAreaDetail reportHotelAreaDetail = ConvertReportHotelAreaDetailTool.INSTANCE.convertAddDto2Entity(addReportHotelAreaDetailDTO);
        // 设置默认值
        setDefaultValue(reportHotelAreaDetail);
        save(reportHotelAreaDetail);
        return reportHotelAreaDetail.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportHotelAreaDetail 商业报告详情实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportHotelAreaDetail reportHotelAreaDetail) {
        reportHotelAreaDetail.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 商业报告详情详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-06-07 09:42:06
     */
    @Override
    public ReportHotelAreaDetail getReportHotelAreaDetail(String id) {
        ReportHotelAreaDetail reportHotelAreaDetail = getById(id);
        if (reportHotelAreaDetail == null) {
            throw new IllegalArgumentException("查看ReportHotelAreaDetail详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportHotelAreaDetail;
    }

    /**
     * 更新商业报告详情
     *
     * @author lixing
     * @version V1.0 2021-06-07 09:42:06
     */
    @Override
    public void updateReportHotelAreaDetail(UpdateReportHotelAreaDetailDTO updateReportHotelAreaDetailDTO) {
        String reportOutlineId = updateReportHotelAreaDetailDTO.getReportOutlineId();
        ReportOutline reportOutline = reportOutlineService.queryReportOutlineDetail(reportOutlineId);
        ReportHotelAreaDetail areaDetail = getReportHotelAreaDetail(reportOutline.getDetailId());
        areaDetail = ConvertReportHotelAreaDetailTool.INSTANCE.convertUpdateDto2Entity(areaDetail, updateReportHotelAreaDetailDTO);
        areaDetail.setModifier(PoemsContext.getContext().getUserId());
        updateById(areaDetail);
    }

    /**
     * 校验商业报告详情是否可删除
     *
     * @param id 商业报告详情主键
     * @return 商业报告详情do类
     * @author lixing
     * @version V1.0 2021-06-07 09:42:06
     */
    public ReportHotelAreaDetail checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportHotelAreaDetail时发生异常，主键为空");
        }

        ReportHotelAreaDetail reportHotelAreaDetail = getById(id);

        if (reportHotelAreaDetail == null) {
            throw new IllegalArgumentException("删除ReportHotelAreaDetail时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportHotelAreaDetail;
    }

    /**
     * 删除商业报告详情
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-06-07 09:42:06
     */
    @Override
    public void deleteReportHotelAreaDetail(String id) {
        // 校验是否可删除
        ReportHotelAreaDetail reportHotelAreaDetail = checkDeletable(id);

        reportHotelAreaDetail.setValid(ValidEnum.FALSE.getType());
        updateById(reportHotelAreaDetail);
    }

    /**
     * 查询商业报告详情
     *
     * @return List<ReportHotelAreaDetail>
     * @author lixing
     * @version V1.0 2021-06-07 09:42:06
     */
    @Override
    public List<ReportHotelAreaDetail> queryReportHotelAreaDetailList(QueryReportHotelAreaDetailDTO queryReportHotelAreaDetailDTO) {
        QueryWrapper<ReportHotelAreaDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportHotelAreaDetail.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportHotelAreaDetail.PROP_CREATIONTIME);

        if (queryReportHotelAreaDetailDTO != null) {

            if (queryReportHotelAreaDetailDTO.getPlatformRuntimeInfo() != null) {
                queryWrapper.eq(ReportHotelAreaDetail.PROP_PLATFORM_RUNTIME_INFO, queryReportHotelAreaDetailDTO.getPlatformRuntimeInfo());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportHotelAreaDetailDTO.getPlatformRuntimeDescription())) {
                queryWrapper.like(ReportHotelAreaDetail.PROP_PLATFORM_RUNTIME_DESCRIPTION, queryReportHotelAreaDetailDTO.getPlatformRuntimeDescription());
            }

            if (queryReportHotelAreaDetailDTO.getCostInfo() != null) {
                queryWrapper.eq(ReportHotelAreaDetail.PROP_COST_INFO, queryReportHotelAreaDetailDTO.getCostInfo());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportHotelAreaDetailDTO.getCostDescription())) {
                queryWrapper.like(ReportHotelAreaDetail.PROP_COST_DESCRIPTION, queryReportHotelAreaDetailDTO.getCostDescription());
            }

            if (queryReportHotelAreaDetailDTO.getPowerUsageOutline() != null) {
                queryWrapper.eq(ReportHotelAreaDetail.PROP_POWER_USAGE_OUTLINE, queryReportHotelAreaDetailDTO.getPowerUsageOutline());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportHotelAreaDetailDTO.getPowerUsageDescription())) {
                queryWrapper.like(ReportHotelAreaDetail.PROP_POWER_USAGE_DESCRIPTION, queryReportHotelAreaDetailDTO.getPowerUsageDescription());
            }

            if (queryReportHotelAreaDetailDTO.getEnergyWorkOrderInfo() != null) {
                queryWrapper.eq(ReportHotelAreaDetail.PROP_ENERGY_WORK_ORDER_INFO, queryReportHotelAreaDetailDTO.getEnergyWorkOrderInfo());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryReportHotelAreaDetailDTO.getEnergyWorkOrderDescription())) {
                queryWrapper.like(ReportHotelAreaDetail.PROP_ENERGY_WORK_ORDER_DESCRIPTION, queryReportHotelAreaDetailDTO.getEnergyWorkOrderDescription());
            }

            if (queryReportHotelAreaDetailDTO.getAlarmDealInfo() != null) {
                queryWrapper.eq(ReportHotelAreaDetail.PROP_ALARM_DEAL_INFO, queryReportHotelAreaDetailDTO.getAlarmDealInfo());
            }

            if (queryReportHotelAreaDetailDTO.getRemainProblemsDealInfo() != null) {
                queryWrapper.eq(ReportHotelAreaDetail.PROP_REMAIN_PROBLEMS_DEAL_INFO, queryReportHotelAreaDetailDTO.getRemainProblemsDealInfo());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询商业报告详情
     *
     * @return IPage<ReportHotelAreaDetail>
     * @author lixing
     * @version V1.0 2021-06-07 09:42:06
     */
    @Override
    public IPage<ReportHotelAreaDetail> pageQueryReportHotelAreaDetail(PageQueryReportHotelAreaDetailDTO pageQueryReportHotelAreaDetailDTO) {
        QueryWrapper<ReportHotelAreaDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportHotelAreaDetail.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportHotelAreaDetail> pageParam = new Page<>(pageQueryReportHotelAreaDetailDTO.getPage(), pageQueryReportHotelAreaDetailDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportHotelAreaDetailDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportHotelAreaDetail.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryReportHotelAreaDetailDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (pageQueryReportHotelAreaDetailDTO.getPlatformRuntimeInfo() != null) {
            queryWrapper.eq(ReportHotelAreaDetail.PROP_PLATFORM_RUNTIME_INFO, pageQueryReportHotelAreaDetailDTO.getPlatformRuntimeInfo());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportHotelAreaDetailDTO.getPlatformRuntimeDescription())) {
            queryWrapper.like(ReportHotelAreaDetail.PROP_PLATFORM_RUNTIME_DESCRIPTION, pageQueryReportHotelAreaDetailDTO.getPlatformRuntimeDescription());
        }

        if (pageQueryReportHotelAreaDetailDTO.getCostInfo() != null) {
            queryWrapper.eq(ReportHotelAreaDetail.PROP_COST_INFO, pageQueryReportHotelAreaDetailDTO.getCostInfo());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportHotelAreaDetailDTO.getCostDescription())) {
            queryWrapper.like(ReportHotelAreaDetail.PROP_COST_DESCRIPTION, pageQueryReportHotelAreaDetailDTO.getCostDescription());
        }

        if (pageQueryReportHotelAreaDetailDTO.getPowerUsageOutline() != null) {
            queryWrapper.eq(ReportHotelAreaDetail.PROP_POWER_USAGE_OUTLINE, pageQueryReportHotelAreaDetailDTO.getPowerUsageOutline());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportHotelAreaDetailDTO.getPowerUsageDescription())) {
            queryWrapper.like(ReportHotelAreaDetail.PROP_POWER_USAGE_DESCRIPTION, pageQueryReportHotelAreaDetailDTO.getPowerUsageDescription());
        }

        if (pageQueryReportHotelAreaDetailDTO.getEnergyWorkOrderInfo() != null) {
            queryWrapper.eq(ReportHotelAreaDetail.PROP_ENERGY_WORK_ORDER_INFO, pageQueryReportHotelAreaDetailDTO.getEnergyWorkOrderInfo());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryReportHotelAreaDetailDTO.getEnergyWorkOrderDescription())) {
            queryWrapper.like(ReportHotelAreaDetail.PROP_ENERGY_WORK_ORDER_DESCRIPTION, pageQueryReportHotelAreaDetailDTO.getEnergyWorkOrderDescription());
        }

        if (pageQueryReportHotelAreaDetailDTO.getAlarmDealInfo() != null) {
            queryWrapper.eq(ReportHotelAreaDetail.PROP_ALARM_DEAL_INFO, pageQueryReportHotelAreaDetailDTO.getAlarmDealInfo());
        }

        if (pageQueryReportHotelAreaDetailDTO.getRemainProblemsDealInfo() != null) {
            queryWrapper.eq(ReportHotelAreaDetail.PROP_REMAIN_PROBLEMS_DEAL_INFO, pageQueryReportHotelAreaDetailDTO.getRemainProblemsDealInfo());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public ResponseReportHotelAreaDetailItemVO getReportHotelAreaDetailByReportOutline(String reportOutlineId) {
        ReportOutline reportOutline = reportOutlineService.queryReportOutlineDetail(reportOutlineId);
        String detailId = reportOutline.getDetailId();
        ReportHotelAreaDetail reportHotelAreaDetail = getReportHotelAreaDetail(detailId);
        return ConvertReportHotelAreaDetailTool.INSTANCE.
                convert2ResponseItemDTO(reportHotelAreaDetail);
    }
}
