package com.persagy.apm.energy.report.monthly.detail.hotel.project.model;

import com.persagy.apm.energy.report.monthly.detail.common.model.vo.HotelPowerConsumptionItem;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.HotelPowerPublicConsumptionItem;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerItemVO;
import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.*;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.dto.*;

import java.util.List;

/**
 * 商业报告详情(ReportHotelProjectDetail) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-06-07 09:42:59 2021-06-07 09:42:59
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertReportHotelProjectDetailTool {

    ConvertReportHotelProjectDetailTool INSTANCE = Mappers.getMapper(ConvertReportHotelProjectDetailTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param reportHotelProjectDetail do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-06-07 09:42:59
     */
    ResponseReportHotelProjectDetailItemVO convert2ResponseItemDTO(ReportHotelProjectDetail reportHotelProjectDetail);

    /**
     * do转换为ResponseListItemVO
     *
     * @param reportHotelProjectDetail do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-06-07 09:42:59
     */
    ResponseReportHotelProjectDetailListItemVO convert2ResponseListItemDTO(ReportHotelProjectDetail reportHotelProjectDetail);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param reportHotelProjectDetailList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-06-07 09:42:59
     */
    List<ResponseReportHotelProjectDetailListItemVO> convert2List(List<ReportHotelProjectDetail> reportHotelProjectDetailList);

    /**
     * addDTO转换为do
     *
     * @param addReportHotelProjectDetailDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-06-07 09:42:59
     */
    @Mapping(target = "publicAreaPowerUsage", source = "publicPowerUsage")
    ReportHotelProjectDetail convertAddDto2Entity(AddReportHotelProjectDetailDTO addReportHotelProjectDetailDTO);

    /**
     * updateDTO转换为实体
     *
     * @param reportHotelProjectDetail          更新前的do对象
     * @param updateReportHotelProjectDetailDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-06-07 09:42:59
     */
    ReportHotelProjectDetail convertUpdateDto2Entity(@MappingTarget ReportHotelProjectDetail reportHotelProjectDetail, UpdateReportHotelProjectDetailDTO updateReportHotelProjectDetailDTO);

    /**
     * PowerItemVO转换为 HotelPowerConsumptionItem
     *
     * @param item PowerItemVO
     * @return HotelPowerConsumptionItem
     * @author lixing
     * @version V1.0 2021/6/10 3:28 下午
     */
    HotelPowerConsumptionItem convert2HotelPowerConsumptionItem(PowerItemVO item);

    /**
     * PowerItemVO转换为 HotelPowerPublicConsumptionItem
     *
     * @param powerItemVO PowerItemVO
     * @return HotelPowerPublicConsumptionItem
     * @author lixing
     * @version V1.0 2021/6/10 3:28 下午
     */
    HotelPowerPublicConsumptionItem convert2HotelPowerPublicConsumptionItem(PowerItemVO powerItemVO);
}

