package com.persagy.apm.energy.report.monthly.detail.hotel.project.service.impl;

import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.HotelPowerConsumptionItem;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerVO;
import com.persagy.apm.energy.report.monthly.detail.common.service.impl.ReportPowerInfoServiceImpl;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.ConvertReportHotelProjectDetailTool;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectPowerConsumptionOutline;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IHotelProjectReportPowerInfoOutlineService;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 商业类报告用电量service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 7:30 下午
 **/
@Service
public class HotelProjectReportPowerInfoOutlineServiceImpl extends ReportPowerInfoServiceImpl
        implements IHotelProjectReportPowerInfoOutlineService {
    @Autowired
    private ISaasWebService saasWebService;

    @Override
    public Double getProjectBuildingArea(List<String> projectIds) {
        return saasWebService.getTotalBuildingArea(projectIds);
    }

    @Override
    public HotelProjectPowerConsumptionOutline getConsumptionOutline(
            String projectId, Date reportMonth, List<String> functionIdList) {
        PowerVO powerInfo = getPowerInfoByProjectIds(null, functionIdList, reportMonth, Lists.newArrayList(projectId));
        HotelProjectPowerConsumptionOutline hotelProjectPowerConsumptionOutline = new HotelProjectPowerConsumptionOutline();
        // 获取项目信息
        SimpleProjectVO simpleProjectInfo = saasWebService.getSimpleProjectInfo(projectId);
        if (simpleProjectInfo == null) {
            return null;
        }
        hotelProjectPowerConsumptionOutline.setProjectName(simpleProjectInfo.getProjectName());
        PowerItemVO summary = powerInfo.getSummary();
        if (summary != null) {
            // 本月耗电量
            Double currentMonth = summary.getCurrentMonth();
            hotelProjectPowerConsumptionOutline.setConsumptionCurrentMonth(currentMonth);
            // 同等碳排放量
            Double slewRate = 2500d;
            if (currentMonth != null) {
                hotelProjectPowerConsumptionOutline.setTce(DataUtils.doubleDivide(currentMonth, slewRate));
            }
            // 单位建筑面积能耗
            hotelProjectPowerConsumptionOutline.setConsumptionPerSqm(
                    summary.getCurrentMonthPerSqm());
            // 总建筑面积
            hotelProjectPowerConsumptionOutline.setOverallFloorage(
                    getProjectBuildingArea(Lists.newArrayList(projectId)));
            // 去年同期耗电量
            hotelProjectPowerConsumptionOutline.setConsumptionLastYear(summary.getLastYearSameTime());
            // 环比涨幅
            hotelProjectPowerConsumptionOutline.setConsumptionChainRange(summary.getLinkRange());
            // 月同比涨幅
            hotelProjectPowerConsumptionOutline.setConsumptionSameTimeRange(summary.getSameTimeRange());
        }
        // 条目信息
        List<PowerItemVO> items = powerInfo.getItems();
        if (!CollectionUtils.isEmpty(items)) {
            List<HotelPowerConsumptionItem> consumptions = items.stream().map(
                    ConvertReportHotelProjectDetailTool.INSTANCE::convert2HotelPowerConsumptionItem
            ).collect(Collectors.toList());
            hotelProjectPowerConsumptionOutline.setDetailInfo(consumptions);
        }
        return hotelProjectPowerConsumptionOutline;
    }
}
