package com.persagy.apm.energy.report.monthly.detail.hotel.project.service.impl;

import com.google.common.collect.Maps;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.HotelPowerPublicConsumptionItem;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerItemVO;
import com.persagy.apm.energy.report.monthly.detail.common.model.vo.PowerVO;
import com.persagy.apm.energy.report.monthly.detail.common.service.impl.ReportPowerInfoServiceImpl;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.ConvertReportHotelProjectDetailTool;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectPublicPowerConsumption;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IHotelProjectReportPublicPowerInfoService;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.concurrent.ConcurrentMap;
import java.util.stream.Collectors;

/**
 * 商业类报告用电量service类
 *
 * @author lixing
 * @version V1.0 2021/5/30 7:30 下午
 **/
@Service
public class HotelProjectReportPublicPowerInfoServiceImpl extends ReportPowerInfoServiceImpl
        implements IHotelProjectReportPublicPowerInfoService {
    @Autowired
    private ISaasWebService saasWebService;

    @Override
    public Double getProjectBuildingArea(List<String> projectIds) {
        return saasWebService.getTotalPublicArea(projectIds);
    }

    @Override
    public HotelProjectPublicPowerConsumption getPublicConsumption(
            String projectId, Date reportMonth, List<String> functionIdList) {
        // 获取用电量信息
        PowerVO powerInfo = getPowerInfoByProjectIds(
                null, functionIdList, reportMonth, Lists.newArrayList(projectId));
        HotelProjectPublicPowerConsumption hotelProjectPublicPowerConsumption = new HotelProjectPublicPowerConsumption();

        PowerItemVO summary = powerInfo.getSummary();
        if (summary != null) {
            // 本月总耗电量
            hotelProjectPublicPowerConsumption.setConsumptionCurrentMonth(summary.getCurrentMonth());
            // 本月单位建筑面积能耗
            hotelProjectPublicPowerConsumption.setConsumptionPerSqmCurrentMonth(summary.getCurrentMonthPerSqm());
            // 费用环比涨幅
            hotelProjectPublicPowerConsumption.setConsumptionMonthRange(summary.getLinkRange());
            // 本年单位建筑面积能耗
            hotelProjectPublicPowerConsumption.setConsumptionPerSqmCurrentYear(summary.getYearPerCount());
            // 能耗同比涨幅
            hotelProjectPublicPowerConsumption.setConsumptionYearRange(summary.getSameRange());
            // 国标引导值
            hotelProjectPublicPowerConsumption.setGuideValue(getGuideValue(projectId));
        }

        // 设置条目信息
        List<PowerItemVO> items = powerInfo.getItems();
        if (!CollectionUtils.isEmpty(items)) {
            List<HotelPowerPublicConsumptionItem> publicConsumptionItems = items.stream().map(
                    ConvertReportHotelProjectDetailTool.INSTANCE::convert2HotelPowerPublicConsumptionItem
            ).collect(Collectors.toList());
            hotelProjectPublicPowerConsumption.setItemsInfo(publicConsumptionItems);
        }

        return hotelProjectPublicPowerConsumption;
    }

    /**
     * 获取项目公区能耗国标引导值
     *
     * @param projectId 项目id
     * @return 引导值
     * @author lixing
     * @version V1.0 2021/6/10 5:11 下午
     */
    private String getGuideValue(String projectId) {
        ConcurrentMap<String, String> guideValueMap = Maps.newConcurrentMap();
        // 严寒地区和寒冷地区：80 kWh/㎡；夏热冬冷地区：135 kWh/㎡；夏热冬暖地区：110 kWh/㎡；温和地区：60 kWh/㎡
        guideValueMap.put("10", "80 kWh/㎡");
        guideValueMap.put("20", "80 kWh/㎡");
        guideValueMap.put("30", "135 kWh/㎡");
        guideValueMap.put("40", "110 kWh/㎡");
        guideValueMap.put("50", "60 kWh/㎡");

        // 获取项目所在的气候区
        SimpleProjectVO simpleProjectInfo = saasWebService.getSimpleProjectInfo(projectId);
        if (simpleProjectInfo != null) {
            String climateZone = simpleProjectInfo.getClimateZone();
            return guideValueMap.get(climateZone);
        }
        return null;
    }
}
