package com.persagy.apm.energy.report.monthly.functionvalue.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.monthly.functionvalue.model.ConvertFunctionValueTool;
import com.persagy.apm.energy.report.monthly.functionvalue.model.FunctionValue;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.*;
import com.persagy.apm.energy.report.monthly.functionvalue.model.vo.ResponseFunctionValueItemVO;
import com.persagy.apm.energy.report.monthly.functionvalue.model.vo.ResponseFunctionValueListItemVO;
import com.persagy.apm.energy.report.monthly.functionvalue.service.IFunctionValueService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 能耗报告信息点取值控制层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:07:13
 */
@ApiIgnore
@Api(tags = "能耗报告信息点取值", hidden = true)
@Validated
@RestController
@RequestMapping("functionValues")
public class FunctionValueController {
    @Autowired
    private IFunctionValueService functionValueService;

    @ApiOperation(value = "创建能耗报告信息点取值")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createFunctionValue(
            @Valid @RequestBody AddFunctionValueDTO addFunctionValueDTO) {
        String id = functionValueService.createFunctionValue(addFunctionValueDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "能耗报告信息点取值详情")
    @PostMapping("/get")
    public CommonResult<ResponseFunctionValueItemVO> queryFunctionValueDetail(
            @Valid @RequestBody QueryFunctionValueDetailDTO queryFunctionValueDetailDTO) {
        FunctionValue functionValue = functionValueService.queryFunctionValueDetail(queryFunctionValueDetailDTO.getId());
        ResponseFunctionValueItemVO responseFunctionValueItemVO = ConvertFunctionValueTool.INSTANCE.convert2ResponseItemDTO(functionValue);
        return ResultHelper.single(responseFunctionValueItemVO);
    }


    @ApiOperation(value = "更新能耗报告信息点取值")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateFunctionValue(
            @Valid @RequestBody UpdateFunctionValueDTO updateFunctionValueDTO) {
        functionValueService.updateFunctionValue(updateFunctionValueDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateFunctionValueDTO.getId()));
    }


    @ApiOperation(value = "删除能耗报告信息点取值")
    @PostMapping("/delete")
    public CommonResult deleteFunctionValue(
            @RequestBody @Valid DeleteFunctionValueDTO deleteFunctionValueDTO) {
        functionValueService.deleteFunctionValue(deleteFunctionValueDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "能耗报告信息点取值列表")
    @PostMapping("/query")
    public CommonResult<PageList<ResponseFunctionValueListItemVO>> queryFunctionValue(
            @Valid @RequestBody QueryFunctionValueDTO queryFunctionValueDTO) {
        List<FunctionValue> functionValueList = functionValueService.queryFunctionValueList(queryFunctionValueDTO);
        List<ResponseFunctionValueListItemVO> resultList = ConvertFunctionValueTool.INSTANCE.convert2List(functionValueList);
        return ResultHelper.multi(resultList);
    }

    //@ApiOperation(value = "能耗报告信息点取值列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ResponseFunctionValueListItemVO>> queryFunctionValue(
    //        @Valid @RequestBody PageQueryFunctionValueDTO pageQueryFunctionValueDTO) {
    //    IPage<FunctionValue> pagedList = functionValueService.pageQueryFunctionValue(pageQueryFunctionValueDTO);
    //    IPage<ResponseFunctionValueListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertFunctionValueTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，FunctionValue实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

