package com.persagy.apm.energy.report.monthly.functionvalue.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.functionvalue.model.vo.*;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.*;

import java.util.List;

/**
 * 能耗报告信息点取值(FunctionValue) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:07:15 2021-05-20 19:07:15
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertFunctionValueTool {

    ConvertFunctionValueTool INSTANCE = Mappers.getMapper(ConvertFunctionValueTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param functionValue do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-05-20 19:07:15
     */
    ResponseFunctionValueItemVO convert2ResponseItemDTO(FunctionValue functionValue);

    /**
     * do转换为ResponseListItemVO
     *
     * @param functionValue do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-05-20 19:07:15
     */
    ResponseFunctionValueListItemVO convert2ResponseListItemDTO(FunctionValue functionValue);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param functionValueList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-05-20 19:07:15
     */
    List<ResponseFunctionValueListItemVO> convert2List(List<FunctionValue> functionValueList);

    /**
     * addDTO转换为do
     *
     * @param addFunctionValueDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-05-20 19:07:15
     */
    FunctionValue convertAddDto2Entity(AddFunctionValueDTO addFunctionValueDTO);

    /**
     * updateDTO转换为实体
     *
     * @param functionValue          更新前的do对象
     * @param updateFunctionValueDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-05-20 19:07:15
     */
    FunctionValue convertUpdateDto2Entity(@MappingTarget FunctionValue functionValue, UpdateFunctionValueDTO updateFunctionValueDTO);
}

