package com.persagy.apm.energy.report.monthly.functionvalue.service;

import com.persagy.apm.energy.report.monthly.functionvalue.model.*;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.*;

import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 能耗报告信息点取值(FunctionValue) service接口
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:07:13
 */
public interface IFunctionValueService {

    /**
     * 创建能耗报告信息点取值
     *
     * @param addFunctionValueDTO addDTO
     * @return 能耗报告信息点取值主键
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    String createFunctionValue(AddFunctionValueDTO addFunctionValueDTO);

    /**
     * 批量保存能耗报告信息点
     *
     * @param saveList
     */
    void batchCreateFunctionValue(List<FunctionValue> saveList);

    /**
     * 能耗报告信息点取值详情
     *
     * @param id 主键
     * @return 能耗报告信息点取值do对象
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    FunctionValue queryFunctionValueDetail(String id);

    /**
     * 更新能耗报告信息点取值
     *
     * @param updateFunctionValueDTO updateDTO
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    void updateFunctionValue(UpdateFunctionValueDTO updateFunctionValueDTO);

    /**
     * 删除能耗报告信息点取值
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    void deleteFunctionValue(String id);

    /**
     * 批量删除能耗报告信息点取值
     *
     * @param batchDeleteFunctionValueDTO
     */
    void batchDeleteFunctionValue(BatchDeleteFunctionValueDTO batchDeleteFunctionValueDTO);

    /**
     * 查询能耗报告信息点取值
     *
     * @param queryFunctionValueDTO queryDTO
     * @return List<FunctionValue>
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    List<FunctionValue> queryFunctionValueList(QueryFunctionValueDTO queryFunctionValueDTO);

    /**
     * 获取信息点取值
     *
     * @param functionId 功能点id
     * @param dataTime 时间
     * @param projectId 项目id
     * @return 信息点的值
     * @author lixing
     * @version V1.0 2021/5/30 9:17 下午
     */
    Double getFunctionValue(String functionId, Date dataTime, String projectId);

    /**
     * 分页查询能耗报告信息点取值
     *
     * @param pageQueryFunctionValueDTO pageQueryDTO
     * @return IPage<FunctionValue>
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    IPage<FunctionValue> pageQueryFunctionValue(PageQueryFunctionValueDTO pageQueryFunctionValueDTO);
}
