package com.persagy.apm.energy.report.monthly.functionvalue.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.energy.report.monthly.functionvalue.dao.FunctionValueMapper;
import com.persagy.apm.energy.report.monthly.functionvalue.model.ConvertFunctionValueTool;
import com.persagy.apm.energy.report.monthly.functionvalue.model.FunctionValue;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.*;
import com.persagy.apm.energy.report.monthly.functionvalue.service.IFunctionValueService;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 能耗报告信息点取值(FunctionValue) service层
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:07:13
 */
@Service
public class FunctionValueServiceImpl extends ServiceImpl<FunctionValueMapper, FunctionValue>
        implements IFunctionValueService {

    /**
     * 创建能耗报告信息点取值
     *
     * @return 能耗报告信息点取值主键
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    @Override
    public String createFunctionValue(AddFunctionValueDTO addFunctionValueDTO) {
        FunctionValue functionValue = ConvertFunctionValueTool.INSTANCE.convertAddDto2Entity(addFunctionValueDTO);
        // 设置默认值
        setDefaultValue(functionValue);
        save(functionValue);
        return functionValue.getId();
    }

    @Override
    public void batchCreateFunctionValue(List<FunctionValue> saveList) {
        if (!CollectionUtils.isEmpty(saveList)) {
            saveBatch(saveList);
        }
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param functionValue 能耗报告信息点取值实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(FunctionValue functionValue) {
        functionValue.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 能耗报告信息点取值详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    @Override
    public FunctionValue queryFunctionValueDetail(String id) {
        FunctionValue functionValue = getById(id);
        if (functionValue == null) {
            throw new IllegalArgumentException("查看FunctionValue详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return functionValue;
    }

    /**
     * 更新能耗报告信息点取值
     *
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    @Override
    public void updateFunctionValue(UpdateFunctionValueDTO updateFunctionValueDTO) {
        FunctionValue functionValue = getById(updateFunctionValueDTO.getId());
        functionValue = ConvertFunctionValueTool.INSTANCE.convertUpdateDto2Entity(functionValue, updateFunctionValueDTO);
        functionValue.setModifier(AppContext.getContext().getAccountId());
        updateById(functionValue);
    }

    /**
     * 校验能耗报告信息点取值是否可删除
     *
     * @param id 能耗报告信息点取值主键
     * @return 能耗报告信息点取值do类
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    public FunctionValue checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除FunctionValue时发生异常，主键为空");
        }

        FunctionValue functionValue = getById(id);

        if (functionValue == null) {
            throw new IllegalArgumentException("删除FunctionValue时发生异常，找不到要删除的数据，id:" + id);
        }

        return functionValue;
    }

    /**
     * 删除能耗报告信息点取值
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    @Override
    public void deleteFunctionValue(String id) {
        // 校验是否可删除
        FunctionValue functionValue = checkDeletable(id);

        functionValue.setValid(ValidEnum.FALSE.getType());
        updateById(functionValue);
    }

    @Override
    public void batchDeleteFunctionValue(BatchDeleteFunctionValueDTO batchDeleteFunctionValueDTO) {
        if (batchDeleteFunctionValueDTO != null) {
            QueryWrapper<FunctionValue> deleteWrapper = new QueryWrapper<>();
            deleteWrapper.eq(FunctionValue.PROP_PROJECT_ID, batchDeleteFunctionValueDTO.getProjectId());
            deleteWrapper.eq(FunctionValue.PROP_FUNCTION_ID, batchDeleteFunctionValueDTO.getFunctionId());
            deleteWrapper.in(FunctionValue.PROP_DATA_TIME, batchDeleteFunctionValueDTO.getDataTimeList());
            remove(deleteWrapper);
        }
    }

    /**
     * 查询能耗报告信息点取值
     *
     * @return List<FunctionValue>
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    @Override
    public List<FunctionValue> queryFunctionValueList(QueryFunctionValueDTO queryFunctionValueDTO) {
        QueryWrapper<FunctionValue> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(FunctionValue.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, FunctionValue.PROP_CREATIONTIME);

        if (queryFunctionValueDTO != null) {

            if (StringUtils.isNotEmpty(queryFunctionValueDTO.getFunctionId())) {
                queryWrapper.eq(FunctionValue.PROP_FUNCTION_ID, queryFunctionValueDTO.getFunctionId());
            }

            if (StringUtils.isNotEmpty(queryFunctionValueDTO.getProjectId())) {
                queryWrapper.eq(FunctionValue.PROP_PROJECT_ID, queryFunctionValueDTO.getProjectId());
            }

            if (queryFunctionValueDTO.getDataTime() != null) {
                queryWrapper.eq(FunctionValue.PROP_DATA_TIME, queryFunctionValueDTO.getDataTime());
            }

            if (StringUtils.isNotEmpty(queryFunctionValueDTO.getValue())) {
                queryWrapper.eq(FunctionValue.PROP_VALUE, queryFunctionValueDTO.getValue());
            }

            if(!CollectionUtils.isEmpty(queryFunctionValueDTO.getFunctionIdList())){
                queryWrapper.in(FunctionValue.PROP_FUNCTION_ID, queryFunctionValueDTO.getFunctionIdList());
            }

            if(queryFunctionValueDTO.getStartTime() != null){
                queryWrapper.ge(FunctionValue.PROP_DATA_TIME, queryFunctionValueDTO.getStartTime());
            }

            if(queryFunctionValueDTO.getEndTime() != null){
                queryWrapper.lt(FunctionValue.PROP_DATA_TIME, queryFunctionValueDTO.getEndTime());
            }

        }

        return list(queryWrapper);
    }

    @Override
    public Double getFunctionValue(String functionId, Date dataTime, String projectId) {
        QueryFunctionValueDTO queryFunctionValueDTO = new QueryFunctionValueDTO();
        queryFunctionValueDTO.setFunctionId(functionId);
        queryFunctionValueDTO.setDataTime(dataTime);
        queryFunctionValueDTO.setProjectId(projectId);
        List<FunctionValue> functionValues = queryFunctionValueList(queryFunctionValueDTO);
        if (!CollectionUtils.isEmpty(functionValues)) {
            String value = functionValues.get(0).getValue();
            if (StringUtils.isNotBlank(value)) {
                return Double.valueOf(value);
            }
        }
        return null;
    }

    /**
     * 分页查询能耗报告信息点取值
     *
     * @return IPage<FunctionValue>
     * @author lixing
     * @version V1.0 2021-05-20 19:07:13
     */
    @Override
    public IPage<FunctionValue> pageQueryFunctionValue(PageQueryFunctionValueDTO pageQueryFunctionValueDTO) {
        QueryWrapper<FunctionValue> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(FunctionValue.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<FunctionValue> pageParam = new Page<>(pageQueryFunctionValueDTO.getPage(), pageQueryFunctionValueDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryFunctionValueDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, FunctionValue.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryFunctionValueDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionValueDTO.getFunctionId())) {
            queryWrapper.like(FunctionValue.PROP_FUNCTION_ID, pageQueryFunctionValueDTO.getFunctionId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionValueDTO.getProjectId())) {
            queryWrapper.like(FunctionValue.PROP_PROJECT_ID, pageQueryFunctionValueDTO.getProjectId());
        }

        if (pageQueryFunctionValueDTO.getDataTime() != null) {
            queryWrapper.eq(FunctionValue.PROP_DATA_TIME, pageQueryFunctionValueDTO.getDataTime());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryFunctionValueDTO.getValue())) {
            queryWrapper.like(FunctionValue.PROP_VALUE, pageQueryFunctionValueDTO.getValue());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
