package com.persagy.apm.energy.report.monthly.outline.constants.enums;

import com.persagy.apm.common.model.annotation.SwaggerDisplayEnum;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang.StringUtils;

import java.util.Arrays;
import java.util.List;

/**
 * 报告状态
 *
 * @author lixing
 * @version V1.0 2021/5/17 10:51 上午
 */
@SwaggerDisplayEnum
public enum ReportStateEnum {
    /**
     * 报告状态
     */
    BUILDING("0", "生成中"),
    GENERATED("1", "已生成"),
    BUILD_FAILED("2", "生成失败");

    @Setter
    @Getter
    private String type;
    @Setter
    @Getter
    private String desc;

    ReportStateEnum(String type, String desc) {
        this.type = type;
        this.desc = desc;
    }

    /**
     * 根据type获取到枚举对象
     *
     * @param type 报告状态编码
     * @return 报告状态枚举对象
     * @author lixing
     * @version V1.0 2021/5/31 4:06 下午
     */
    public static ReportStateEnum getByType(String type) {
        if (StringUtils.isBlank(type)) {
            return null;
        }
        ReportStateEnum[] reportStateEnums = ReportStateEnum.values();
        for (ReportStateEnum reportStateEnum : reportStateEnums) {
            if (type.equals(reportStateEnum.getType())) {
                return reportStateEnum;
            }
        }
        return null;
    }
}
