package com.persagy.apm.energy.report.monthly.outline.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energy.report.monthly.config.type.model.ConvertReportTypeTool;
import com.persagy.apm.energy.report.monthly.config.type.model.ReportType;
import com.persagy.apm.energy.report.monthly.config.type.model.vo.ResponseReportTypeItemVO;
import com.persagy.apm.energy.report.monthly.config.type.model.vo.ResponseReportTypeListItemVO;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeService;
import com.persagy.apm.energy.report.monthly.detail.business.model.dto.GetReportBusinessDetailDTO;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.ResponseReportBusinessDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.vo.ResponseReportHotelAreaDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.ResponseReportHotelProjectDetailItemVO;
import com.persagy.apm.energy.report.monthly.outline.constants.enums.ReportStateEnum;
import com.persagy.apm.energy.report.monthly.outline.model.ConvertReportOutlineTool;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.model.dto.*;
import com.persagy.apm.energy.report.monthly.outline.model.vo.ResponseReportOutlineListItemVO;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.persagy.apm.energy.report.saasweb.model.vo.PartitionVO;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * 报告概要控制层
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:09:37
 */
@Api(tags = "报告概要")
@Validated
@RestController
@RequestMapping("reportOutlines")
public class ReportOutlineController {
    @Autowired
    private IReportOutlineService reportOutlineService;
    @Autowired
    private IReportTypeService reportTypeService;
    @Autowired
    private ISaasWebService saasWebService;

    @ApiOperation(value = "生成报告")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createReportOutline(
            @Valid @RequestBody AddReportOutlineDTO addReportOutlineDTO) {
        String id = reportOutlineService.createReportOutline(addReportOutlineDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "重新生成报告")
    @PostMapping("/recreate")
    public CommonResult<ApmResponseUpsertVO> recreateReportOutline(
            @Valid @RequestBody RecreateReportOutlineDTO recreateReportOutlineDTO) {
        String id = reportOutlineService.recreateReportOutline(recreateReportOutlineDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "批量删除报告概要")
    @PostMapping("/batchDelete")
    public CommonResult deleteReportOutline(
            @RequestBody @Valid DeleteReportOutlineDTO deleteReportOutlineDTO) {
        reportOutlineService.batchDeleteReportOutline(deleteReportOutlineDTO.getIds());
        return ResultHelper.success();
    }


    @ApiOperation(value = "报告概要列表, 分页查询")
    @PostMapping("/query")
    public CommonResult<PageList<ResponseReportOutlineListItemVO>> queryReportOutline(
            @Valid @RequestBody PageQueryReportOutlineDTO pageQueryReportOutlineDTO) {
        IPage<ReportOutline> pagedList = reportOutlineService.pageQueryReportOutline(pageQueryReportOutlineDTO);
        IPage<ResponseReportOutlineListItemVO> pagedResultList = pagedList.convert(item -> {
            try {

                ResponseReportOutlineListItemVO responseReportOutlineListItemVO = ConvertReportOutlineTool.INSTANCE.convert2ResponseListItemDTO(item);
                ReportType reportType = reportTypeService.queryReportTypeDetail(item.getReportTypeId());
                ResponseReportTypeItemVO responseReportTypeItemVO = ConvertReportTypeTool.INSTANCE.convert2ResponseItemDTO(reportType);
                responseReportOutlineListItemVO.setReportType(responseReportTypeItemVO);

                responseReportOutlineListItemVO.setBelongName(
                        reportOutlineService.getBelongName(reportType.getBelongType(), item.getBelong()));
                ReportStateEnum reportStateEnum = ReportStateEnum.getByType(item.getStatus());
                if (reportStateEnum != null) {
                    responseReportOutlineListItemVO.setStatusName(reportStateEnum.getDesc());
                }
                return responseReportOutlineListItemVO;
            } catch (Exception e) {
                throw new IllegalArgumentException("分页查询，ReportOutline实体类转换为列表对象发生异常", e);
            }
        });
        return ResultHelper.multi(pagedResultList);
    }

    @ApiOperation(value = "查询用户可选项目范围")
    @PostMapping("/queryAvailableProjects")
    public CommonResult<PageList<SimpleProjectVO>> queryAvailableProjects(
            @Valid @RequestBody QueryAvailableBelongsDTO queryAvailableBelongsDTO) {
        List<SimpleProjectVO> availableProjects = saasWebService.getAvailableProjects(queryAvailableBelongsDTO.getBuildingType());
        IPage<SimpleProjectVO> resultList = new Page<>();
        resultList.setRecords(availableProjects);
        resultList.setTotal(availableProjects.size());
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "查询用户可选分区范围")
    @PostMapping("/queryAvailableAreas")
    public CommonResult<PageList<PartitionVO>> queryAvailableAreas(
            @Valid @RequestBody QueryAvailableBelongsDTO queryAvailableBelongsDTO) {
        List<PartitionVO> availableAreas = saasWebService.getAvailableAreas(queryAvailableBelongsDTO.getBuildingType());
        IPage<PartitionVO> resultList = new Page<>();
        resultList.setRecords(availableAreas);
        resultList.setTotal(availableAreas.size());
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "当前用户可用的报告类型")
    @PostMapping("/availableReportTypes")
    public CommonResult<PageList<ResponseReportTypeListItemVO>> queryAvailableReportTypes(@RequestBody NullBodyDTO nullBodyDTO) {
        IPage<ResponseReportTypeListItemVO> resultList = reportTypeService.queryAvailableReportTypes();
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "查看商业报告详情")
    @PostMapping("/business/get")
    public CommonResult<ResponseReportBusinessDetailItemVO> getReportBusinessDetail(
            @Valid @RequestBody GetReportBusinessDetailDTO getReportBusinessDetailDTO) {
        ResponseReportBusinessDetailItemVO reportDetailInfo = reportOutlineService.getReportBusinessDetailInfo(
                getReportBusinessDetailDTO.getReportOutlineId());
        return ResultHelper.single(reportDetailInfo);
    }

    @ApiOperation(value = "查看酒店项目报告详情")
    @PostMapping("/hotel/project/get")
    public CommonResult<ResponseReportHotelProjectDetailItemVO> getReportHotelProjectDetail(
            @Valid @RequestBody GetReportBusinessDetailDTO getReportBusinessDetailDTO) {
        ResponseReportHotelProjectDetailItemVO reportDetailInfo = reportOutlineService.getReportHotelProjectDetail(
                getReportBusinessDetailDTO.getReportOutlineId());
        return ResultHelper.single(reportDetailInfo);
    }

    @ApiOperation(value = "查看酒店区域报告详情")
    @PostMapping("/hotel/area/get")
    public CommonResult<ResponseReportHotelAreaDetailItemVO> getReportHotelAreaDetail(
            @Valid @RequestBody GetReportBusinessDetailDTO getReportBusinessDetailDTO) {
        ResponseReportHotelAreaDetailItemVO reportDetailInfo = reportOutlineService.getReportHotelAreaDetail(
                getReportBusinessDetailDTO.getReportOutlineId());
        return ResultHelper.single(reportDetailInfo);
    }
}

