package com.persagy.apm.energy.report.monthly.outline.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.energy.report.monthly.outline.model.vo.*;
import com.persagy.apm.energy.report.monthly.outline.model.dto.*;

import java.util.List;

/**
 * 报告概要(ReportOutline) dto、vo、do转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:09:38 2021-05-17 11:09:38
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertReportOutlineTool {

    ConvertReportOutlineTool INSTANCE = Mappers.getMapper(ConvertReportOutlineTool.class);

    /**
     * do转换为ResponseItemVO
     *
     * @param reportOutline do对象
     * @return ResponseItemVO
     * @author lixing
     * @version V1.0 2021-05-17 11:09:38
     */
    ResponseReportOutlineItemVO convert2ResponseItemDTO(ReportOutline reportOutline);

    /**
     * do转换为ResponseListItemVO
     *
     * @param reportOutline do对象
     * @return ResponseListItemVO
     * @author lixing
     * @version V1.0 2021-05-17 11:09:38
     */
    ResponseReportOutlineListItemVO convert2ResponseListItemDTO(ReportOutline reportOutline);

    /**
     * do列表转换为ResponseListItemVO列表
     *
     * @param reportOutlineList do列表
     * @return ResponseListItemVO列表
     * @author lixing
     * @version V1.0 2021-05-17 11:09:38
     */
    List<ResponseReportOutlineListItemVO> convert2List(List<ReportOutline> reportOutlineList);

    /**
     * addDTO转换为do
     *
     * @param addReportOutlineDTO addDTO
     * @return 要创建的do对象
     * @author lixing
     * @version V1.0 2021-05-17 11:09:38
     */
    ReportOutline convertAddDto2Entity(AddReportOutlineDTO addReportOutlineDTO);

    /**
     * updateDTO转换为实体
     *
     * @param reportOutline          更新前的do对象
     * @param updateReportOutlineDTO updateDTO
     * @return 更新后的do对象
     * @author lixing
     * @version V1.0 2021-05-17 11:09:38
     */
    ReportOutline convertUpdateDto2Entity(@MappingTarget ReportOutline reportOutline, UpdateReportOutlineDTO updateReportOutlineDTO);
}

