package com.persagy.apm.energy.report.monthly.outline.service;

import com.persagy.apm.energy.report.monthly.detail.business.model.vo.ResponseReportBusinessDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.vo.ResponseReportHotelAreaDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.ResponseReportHotelProjectDetailItemVO;
import com.persagy.apm.energy.report.monthly.outline.model.*;
import com.persagy.apm.energy.report.monthly.outline.model.dto.*;

import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 报告概要(ReportOutline) service接口
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:09:37
 */
public interface IReportOutlineService {

    /**
     * 创建报告概要
     *
     * @param addReportOutlineDTO addDTO
     * @return 报告概要主键
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    String createReportOutline(AddReportOutlineDTO addReportOutlineDTO);

    /**
     * 获取报告概要的所属类型
     *
     * @param reportOutline 报告概要
     * @return 报告概要的所属类型
     * @author lixing
     * @version V1.0 2021/6/8 5:26 下午
     */
    String getBelongType(ReportOutline reportOutline);

    /**
     * 获取报告概要的业态
     *
     * @param reportOutline 报告概要
     * @return 报告概要的业态
     * @author lixing
     * @version V1.0 2021/6/8 5:26 下午
     */
    String getBuildingType(ReportOutline reportOutline);

    /**
     * 报告概要详情
     *
     * @param id 主键
     * @return 报告概要do对象
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    ReportOutline queryReportOutlineDetail(String id);

    /**
     * 获取商业报告详情信息
     *
     * @param reportOutlineId 主键
     * @return 商业报告详情信息
     * @author lixing
     * @version V1.0 2021/5/31 6:24 下午
     */
    ResponseReportBusinessDetailItemVO getReportBusinessDetailInfo(String reportOutlineId);

    /**
     * 更新报告概要
     *
     * @param updateReportOutlineDTO updateDTO
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    void updateReportOutline(UpdateReportOutlineDTO updateReportOutlineDTO);

    /**
     * 更新报告概要
     *
     * @param reportOutline 报告概要对象
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    void updateReportOutline(ReportOutline reportOutline);

    /**
     * 查询和报告类型相关的项目id列表
     *
     * @param reportOutline 报告概要对象
     * @return 项目id列表
     * @author lixing
     * @version V1.0 2021/5/30 8:48 下午
     */
    List<String> queryRelatedProjectIds(ReportOutline reportOutline);

    /**
     * 批量删除报告概要
     *
     * @param ids 主键列表
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    void batchDeleteReportOutline(List<String> ids);

    /**
     * 查询报告概要
     *
     * @param queryReportOutlineDTO queryDTO
     * @return List<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    List<ReportOutline> queryReportOutlineList(QueryReportOutlineDTO queryReportOutlineDTO);

    /**
     * 分页查询报告概要
     *
     * @param pageQueryReportOutlineDTO pageQueryDTO
     * @return IPage<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    IPage<ReportOutline> pageQueryReportOutline(PageQueryReportOutlineDTO pageQueryReportOutlineDTO);

    /**
     * 查询和报告类型相关的项目列表
     *
     * @param reportOutline 报告概要对象
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/30 8:48 下午
     */
    List<SimpleProjectVO> queryRelatedProjects(ReportOutline reportOutline);

    /**
     * 重新生成报告
     *
     * @param recreateReportOutlineDTO 重新生成报告入参
     * @return 报告id
     * @author lixing
     * @version V1.0 2021/5/31 9:47 上午
     */
    String recreateReportOutline(RecreateReportOutlineDTO recreateReportOutlineDTO);

    /**
     * 获取报告所属名称
     *
     * @param reportBelongType 报告所属类型
     * @param belongId 报告所属id
     * @return 报告所属中文名
     * @author lixing
     * @version V1.0 2021/6/3 4:19 下午
     */
    String getBelongName(String reportBelongType, String belongId);

    /**
     * 获取酒店项目报告详情信息
     *
     * @param reportOutlineId 主键
     * @return 酒店项目报告详情信息
     * @author lixing
     * @version V1.0 2021/5/31 6:24 下午
     */
    ResponseReportHotelProjectDetailItemVO getReportHotelProjectDetail(String reportOutlineId);

    /**
     * 获取酒店区域报告详情信息
     *
     * @param reportOutlineId 主键
     * @return 酒店区域报告详情信息
     * @author lixing
     * @version V1.0 2021/5/31 6:24 下午
     */
    ResponseReportHotelAreaDetailItemVO getReportHotelAreaDetail(String reportOutlineId);
}
