package com.persagy.apm.energy.report.monthly.outline.service.builder.hotel;

import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.*;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.service.builder.common.IReportBuilder;

/**
 * 生成酒店项目报告建造者接口
 *
 * @author lixing
 * @version V1.0 2021/5/30 10:26 下午
 **/
public interface IHotelProjectReportBuilder extends IReportBuilder {

    /**
     * 获取项目报警处理信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目报警处理信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectAlarmDealInfo getAlarmDealInfo(ReportOutline reportOutline);

    /**
     * 获取项目平台运行信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目平台运行信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectMaintenanceInfo getMaintenanceInfo(ReportOutline reportOutline);

    /**
     * 获取项目节能工单信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目节能工单信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectEnergyWorkerOrderInfo getEnergyWorkOrderInfo(ReportOutline reportOutline);

    /**
     * 获取项目环境指标信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目环境指标信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectEnvKpi getEnvKpi(ReportOutline reportOutline);

    /**
     * 获取项目冷站效率信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目冷站效率信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectChillerPlantEfficiency getChillerPlanEfficiency(ReportOutline reportOutline);

    /**
     * 获取项目公区用电信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目公区用电信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectPublicPowerConsumption getPublicPowerConsumption(ReportOutline reportOutline);

    /**
     * 获取项目用电量信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目用电量信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectPowerConsumptionOutline getPowerConsumptionOutline(ReportOutline reportOutline);

    /**
     * 获取项目费用信息
     *
     * @param reportOutline 报告outline对象
     * @return 项目费用信息
     * @author lixing
     * @version V1.0 2021/6/9 10:04 上午
     */
    HotelProjectCostInfo getCostInfo(ReportOutline reportOutline);
}
