package com.persagy.apm.energy.report.monthly.outline.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.monthly.config.rel.detailstatisticsitem.service.IReportDetailStatisticsItemRelService;
import com.persagy.apm.energy.report.monthly.config.type.constant.enums.BelongTypeEnum;
import com.persagy.apm.energy.report.monthly.config.type.model.ReportType;
import com.persagy.apm.energy.report.monthly.config.type.service.IReportTypeService;
import com.persagy.apm.energy.report.monthly.detail.business.model.vo.ResponseReportBusinessDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.business.service.IReportBusinessDetailService;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.model.vo.ResponseReportHotelAreaDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.area.service.IReportHotelAreaDetailService;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.ResponseReportHotelProjectDetailItemVO;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.service.IReportHotelProjectDetailService;
import com.persagy.apm.energy.report.monthly.outline.constants.enums.ReportStateEnum;
import com.persagy.apm.energy.report.monthly.outline.dao.ReportOutlineMapper;
import com.persagy.apm.energy.report.monthly.outline.model.ConvertReportOutlineTool;
import com.persagy.apm.energy.report.monthly.outline.model.ReportOutline;
import com.persagy.apm.energy.report.monthly.outline.model.dto.*;
import com.persagy.apm.energy.report.monthly.outline.service.IReportOutlineService;
import com.persagy.apm.energy.report.saasweb.model.ConvertSaasWebTool;
import com.persagy.apm.energy.report.saasweb.model.vo.PoemsProjectVO;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportAreaVO;
import com.persagy.apm.energy.report.saasweb.model.vo.ReportProjectVO;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import com.persagy.apm.energy.report.saasweb.service.ISaasWebService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 报告概要(ReportOutline) service层
 *
 * @author lixing
 * @version V1.0 2021-05-17 11:09:37
 */
@Service
public class ReportOutlineServiceImpl extends ServiceImpl<ReportOutlineMapper, ReportOutline>
        implements IReportOutlineService {
    @Autowired
    IReportTypeService reportTypeService;
    @Autowired
    ISaasWebService saasWebService;
    @Autowired
    GenerateReportThreadPool generateReportThreadPool;
    @Autowired
    IReportBusinessDetailService reportBusinessDetailService;
    @Autowired
    IReportDetailStatisticsItemRelService reportDetailStatisticsItemRelService;
    @Autowired
    IReportHotelAreaDetailService hotelAreaDetailService;
    @Autowired
    IReportHotelProjectDetailService hotelProjectDetailService;


    /**
     * 创建报告概要
     *
     * @return 报告概要主键
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public String createReportOutline(AddReportOutlineDTO addReportOutlineDTO) {
        ReportOutline reportOutline = ConvertReportOutlineTool.INSTANCE.convertAddDto2Entity(addReportOutlineDTO);
        return generateReport(reportOutline);
    }

    /**
     * 生成报告
     *
     * @param reportOutline 报告概要对象
     * @return 报告概要id
     * @author lixing
     * @version V1.0 2021/5/31 11:09 上午
     */
    private String generateReport(ReportOutline reportOutline) {
        // 设置默认值
        setDefaultValue(reportOutline);
        saveOrUpdate(reportOutline);
        // 后台生成报告
        generateReportThreadPool.generateReport(reportOutline);
        return reportOutline.getId();
    }


    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param reportOutline 报告概要实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ReportOutline reportOutline) {
        reportOutline.setCreator(PoemsContext.getContext().getUserId());
        reportOutline.setStatus(ReportStateEnum.BUILDING.getType());

        /* 拼接报告的名称 */
        // 根据报告类型判断报告所属是项目还是管理分区, 获取报告所属名称
        ReportType reportType = getReportType(reportOutline);
        String belongName = "";
        if (BelongTypeEnum.PROJECT.getType().equals(reportType.getBelongType())) {
            ReportProjectVO reportProjectInfo = saasWebService.getReportProjectInfo(reportOutline.getBelong());
            belongName = reportProjectInfo.getProjectName();
        }
        if (BelongTypeEnum.AREA.getType().equals(reportType.getBelongType())) {
            ReportAreaVO reportAreaInfo = saasWebService.getReportAreaInfo(reportOutline.getBelong(), reportType.getBuildingType());
            belongName = reportAreaInfo.getAreaName();
        }

        LocalDate reportMonthLocalDate = DateUtils.convert2LocalDate(reportOutline.getReportMonth());
        reportOutline.setName(reportMonthLocalDate.getYear() + "年" +
                reportMonthLocalDate.getMonthValue() + "月" +
                belongName + reportType.getName());
    }

    /**
     * 获取报告概要的报告类型
     *
     * @param reportOutline 报告概要
     * @return 报告概要的报告类型
     * @author lixing
     * @version V1.0 2021/6/8 5:26 下午
     */
    private ReportType getReportType(ReportOutline reportOutline) {
        String reportTypeId = reportOutline.getReportTypeId();
        return reportTypeService.queryReportTypeDetail(reportTypeId);
    }

    @Override
    public String getBelongType(ReportOutline reportOutline) {
        ReportType reportType = getReportType(reportOutline);
        if (reportType != null) {
            return reportType.getBelongType();
        }
        return null;
    }

    @Override
    public String getBuildingType(ReportOutline reportOutline) {
        ReportType reportType = getReportType(reportOutline);
        if (reportType != null) {
            return reportType.getBuildingType();
        }
        return null;
    }



    /**
     * 报告概要详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public ReportOutline queryReportOutlineDetail(String id) {
        ReportOutline reportOutline = getById(id);
        if (reportOutline == null) {
            throw new IllegalArgumentException("查看ReportOutline详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return reportOutline;
    }

    @Override
    public ResponseReportBusinessDetailItemVO getReportBusinessDetailInfo(String reportOutlineId) {
        return reportBusinessDetailService.getResponseBusinessDetail(reportOutlineId);
    }

    @Override
    public ResponseReportHotelProjectDetailItemVO getReportHotelProjectDetail(String reportOutlineId) {
        return hotelProjectDetailService.getReportHotelProjectDetailByReportOutline(reportOutlineId);
    }

    @Override
    public ResponseReportHotelAreaDetailItemVO getReportHotelAreaDetail(String reportOutlineId) {
        return hotelAreaDetailService.getReportHotelAreaDetailByReportOutline(reportOutlineId);
    }

    @Override
    public void updateReportOutline(UpdateReportOutlineDTO updateReportOutlineDTO) {
        ReportOutline reportOutline = getById(updateReportOutlineDTO.getId());
        reportOutline = ConvertReportOutlineTool.INSTANCE.convertUpdateDto2Entity(reportOutline, updateReportOutlineDTO);
        reportOutline.setModifier(PoemsContext.getContext().getUserId());
        updateById(reportOutline);
    }

    @Override
    public void updateReportOutline(ReportOutline reportOutline) {
        reportOutline.setModifier(PoemsContext.getContext().getUserId());
        updateById(reportOutline);
    }

    /**
     * 校验报告概要是否可删除
     *
     * @param id 报告概要主键
     * @return 报告概要do类
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    public ReportOutline checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ReportOutline时发生异常，主键为空");
        }

        ReportOutline reportOutline = getById(id);

        if (reportOutline == null) {
            throw new IllegalArgumentException("删除ReportOutline时发生异常，找不到要删除的数据，id:" + id);
        }

        return reportOutline;
    }

    /**
     * 校验报告概要是否存在
     *
     * @param id 报告概要主键
     * @return 报告概要对象
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    public ReportOutline checkExists(String id) {
        if (id == null) {
            throw new IllegalArgumentException("主键不能为空");
        }

        ReportOutline reportOutline = getById(id);

        if (reportOutline == null) {
            throw new IllegalArgumentException("找不到数据，id:" + id);
        }

        return reportOutline;
    }

    @Override
    @Transactional
    public void batchDeleteReportOutline(List<String> ids) {
        for (String id : ids) {
            // 校验是否可删除
            ReportOutline reportOutline = checkDeletable(id);
            reportOutline.setValid(ValidEnum.FALSE.getType());
            updateById(reportOutline);
        }
    }

    /**
     * 查询报告概要
     *
     * @return List<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public List<ReportOutline> queryReportOutlineList(QueryReportOutlineDTO queryReportOutlineDTO) {
        QueryWrapper<ReportOutline> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportOutline.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ReportOutline.PROP_CREATIONTIME);

        if (queryReportOutlineDTO != null) {

            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getName())) {
                queryWrapper.like(ReportOutline.PROP_NAME, queryReportOutlineDTO.getName());
            }

            if (queryReportOutlineDTO.getDate() != null) {
                queryWrapper.eq(ReportOutline.PROP_REPORT_MONTH, queryReportOutlineDTO.getDate());
            }

            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getBelong())) {
                queryWrapper.eq(ReportOutline.PROP_BELONG, queryReportOutlineDTO.getBelong());
            }

            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getType())) {
                queryWrapper.eq(ReportOutline.PROP_REPORT_TYPE_ID, queryReportOutlineDTO.getType());
            }

            if (StringUtils.isNotEmpty(queryReportOutlineDTO.getDetailId())) {
                queryWrapper.eq(ReportOutline.PROP_DETAIL_ID, queryReportOutlineDTO.getDetailId());
            }

            if (queryReportOutlineDTO.getStatus() != null) {
                queryWrapper.eq(ReportOutline.PROP_STATUS, queryReportOutlineDTO.getStatus());
            }

            if (queryReportOutlineDTO.getRemark() != null) {
                queryWrapper.eq(ReportOutline.PROP_REMARK, queryReportOutlineDTO.getRemark());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报告概要
     *
     * @return IPage<ReportOutline>
     * @author lixing
     * @version V1.0 2021-05-17 11:09:37
     */
    @Override
    public IPage<ReportOutline> pageQueryReportOutline(PageQueryReportOutlineDTO pageQueryReportOutlineDTO) {
        QueryWrapper<ReportOutline> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ReportOutline.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ReportOutline> pageParam = new Page<>(pageQueryReportOutlineDTO.getPage(), pageQueryReportOutlineDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryReportOutlineDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ReportOutline.PROP_MODIFIEDTIME);
        } else {
            List<Sort> orders = pageQueryReportOutlineDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (StringUtils.isNotEmpty(pageQueryReportOutlineDTO.getName())) {
            queryWrapper.like(ReportOutline.PROP_NAME, pageQueryReportOutlineDTO.getName());
        }

        if (pageQueryReportOutlineDTO.getStartDate() != null) {
            queryWrapper.ge(ReportOutline.PROP_REPORT_MONTH, pageQueryReportOutlineDTO.getStartDate());
        }

        if (pageQueryReportOutlineDTO.getEndDate() != null) {
            queryWrapper.le(ReportOutline.PROP_REPORT_MONTH, pageQueryReportOutlineDTO.getEndDate());
        }

        if (StringUtils.isNotBlank(pageQueryReportOutlineDTO.getStatus())) {
            queryWrapper.eq(ReportOutline.PROP_STATUS, pageQueryReportOutlineDTO.getStatus());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public List<String> queryRelatedProjectIds(ReportOutline reportOutline) {
        String belongType = getBelongType(reportOutline);
        String buildingType = getBuildingType(reportOutline);

        List<String> projectIds = new ArrayList<>();
        if (BelongTypeEnum.PROJECT.getType().equals(belongType)) {
            projectIds.add(reportOutline.getBelong());
        } else {
            String areaId = reportOutline.getBelong();
            // 获取区域下的项目
            List<PoemsProjectVO> projectsByArea = saasWebService.getProjectsByAreaAndBuildingType(areaId, buildingType);
            if (!CollectionUtils.isEmpty(projectsByArea)) {
                projectIds.addAll(projectsByArea.stream().map(
                        PoemsProjectVO::getProjectId).collect(Collectors.toList()));
            }
        }
        return projectIds;
    }

    @Override
    public List<SimpleProjectVO> queryRelatedProjects(ReportOutline reportOutline) {
        String belongType = getBelongType(reportOutline);
        String buildingType = getBuildingType(reportOutline);

        List<SimpleProjectVO> projects = new ArrayList<>();
        if (BelongTypeEnum.PROJECT.getType().equals(belongType)) {
            projects.add(saasWebService.getSimpleProjectInfo(reportOutline.getBelong()));
        } else {
            String areaId = reportOutline.getBelong();
            // 获取区域下的项目
            List<PoemsProjectVO> projectsByArea = saasWebService.getProjectsByAreaAndBuildingType(areaId, buildingType);
            if (!CollectionUtils.isEmpty(projectsByArea)) {
                projects.addAll(projectsByArea.stream().map(
                        ConvertSaasWebTool.INSTANCE::convert2SimpleProjectVO
                ).collect(Collectors.toList()));
            }
        }
        return projects;
    }

    @Override
    public String recreateReportOutline(RecreateReportOutlineDTO recreateReportOutlineDTO) {
        String id = recreateReportOutlineDTO.getId();
        ReportOutline reportOutline = checkExists(id);
        return generateReport(reportOutline);
    }

    @Override
    public String getBelongName(String reportBelongType, String belongId) {
        String belongName = "";
        if (BelongTypeEnum.PROJECT.getType().equals(reportBelongType)) {
            SimpleProjectVO simpleProjectInfo = saasWebService.getSimpleProjectInfo(belongId);
            if (simpleProjectInfo != null) {
                belongName = simpleProjectInfo.getProjectName();
            }
        }
        if (BelongTypeEnum.AREA.getType().equals(reportBelongType)) {
            ReportAreaVO reportAreaBaseInfo = saasWebService.getReportAreaBaseInfo(belongId);
            if (reportAreaBaseInfo != null) {
                belongName = reportAreaBaseInfo.getAreaName();
            }
        }
        return belongName;
    }
}
