package com.persagy.apm.energy.report.saasweb.client;

import com.persagy.apm.common.model.dto.PoemsFeignBaseDTO;
import com.persagy.apm.common.response.PoemsFeignResponse;
import com.persagy.apm.energy.report.common.FeignClientNameConstants;
import com.persagy.apm.energy.report.common.FeignConfig;
import com.persagy.apm.energy.report.saasweb.client.fallback.SaasWebClientFallbackFactory;
import com.persagy.apm.energy.report.saasweb.model.dto.QueryByProjectIdDTO;
import com.persagy.apm.energy.report.saasweb.model.dto.QueryPartitionDTO;
import com.persagy.apm.energy.report.saasweb.model.dto.QueryProjectDTO;
import com.persagy.apm.energy.report.saasweb.model.vo.*;
import feign.Headers;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 运维平台feign客户端
 *
 * @author lixing
 * @version V1.0 2021/3/22 2:13 下午
 **/
@FeignClient(name = FeignClientNameConstants.SAAS_WEB, fallbackFactory = SaasWebClientFallbackFactory.class,
        configuration = FeignConfig.class)
public interface SaasWebClient {

    /**
     * 根据项目id，获取项目信息
     *
     * @param queryByProjectIdDTO 查询条件
     * @return 项目下的建筑信息
     * @author lixing
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/getProjectByIdService")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<SimpleProjectVO> getProjectById(@RequestParam("jsonString") String queryByProjectIdDTO);

    /**
     * 根据项目id，加载项目下的建筑信息
     *
     * @param queryByProjectIdDTO 查询条件
     * @return 项目下的建筑信息
     * @author lixing
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/listBuildingsByProjectIdService")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<BuildingInfoVO> listBuildingsByProjectId(@RequestParam("jsonString") String queryByProjectIdDTO);

    /**
     * 查询项目列表
     *
     * @param queryProjectDTO 查询条件
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/listProjectsService")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<PoemsProjectVO> queryProjects(@RequestParam("jsonString") String queryProjectDTO);

    /**
     * 根据id查询管理分区
     *
     * @param poemsFeignBaseDTO 查询条件
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/19 2:27 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/queryAllManagementPartitionsService")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<PoemsPartitionVO> queryAllPartitions(@RequestParam("jsonString") String poemsFeignBaseDTO);

    /**
     * 获取用户信息（基本信息和关联的项目）
     *
     * @param poemsFeignBaseDTO 登录信息
     * @return 用户信息
     * @author lixing
     * @version V1.0 2021/5/19 3:36 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/saasGetCurrentUserService")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<UserProjectsInfoVO> getCurrentUserProjectsInfo(@RequestParam("jsonString") String poemsFeignBaseDTO);

    /**
     * 获取用户信息（基本信息和关联的分区项目树）
     *
     * @param poemsFeignBaseDTO 登录信息
     * @return 用户信息
     * @author lixing
     * @version V1.0 2021/5/19 3:36 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/saasGetCurrentUserProjectsTreeService")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<UserPartitionProjectsInfoVO> getCurrentUserPartitionProjectsInfo(@RequestParam("jsonString") String poemsFeignBaseDTO);

    /**
     * 获取所有项目功能类型
     *
     * @param poemsFeignBaseDTO 登录信息
     * @return 所有项目功能类型树
     * @author lixing
     * @version V1.0 2021/5/19 3:36 下午
     */
    @PostMapping("/Spring/MVC/entrance/unifier/listFunctionTypesService")
    @Headers({"Content-Type: application/x-www-form-urlencoded"})
    PoemsFeignResponse<FunctionTypeVO> getAllProjectFunctionTypes(@RequestParam("jsonString") String poemsFeignBaseDTO);
}
