package com.persagy.apm.energy.report.saasweb.model;

import com.persagy.apm.energy.report.monthly.functionvalue.model.FunctionValue;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.AddFunctionValueDTO;
import com.persagy.apm.energy.report.monthly.functionvalue.model.dto.UpdateFunctionValueDTO;
import com.persagy.apm.energy.report.monthly.functionvalue.model.vo.ResponseFunctionValueItemVO;
import com.persagy.apm.energy.report.monthly.functionvalue.model.vo.ResponseFunctionValueListItemVO;
import com.persagy.apm.energy.report.saasweb.model.vo.FilterPartitionVO;
import com.persagy.apm.energy.report.saasweb.model.vo.PartitionVO;
import com.persagy.apm.energy.report.saasweb.model.vo.PoemsProjectVO;
import com.persagy.apm.energy.report.saasweb.model.vo.SimpleProjectVO;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValueMappingStrategy;
import org.mapstruct.factory.Mappers;

import java.util.List;

/**
 * 运维平台转换工具类
 *
 * @author lixing
 * @version V1.0 2021-05-20 19:07:15 2021-05-20 19:07:15
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertSaasWebTool {

    ConvertSaasWebTool INSTANCE = Mappers.getMapper(ConvertSaasWebTool.class);

    /**
     * PoemsProjectVO转换为SimpleProjectVO
     *
     * @param project PoemsProjectVO
     * @return SimpleProjectVO
     * @author lixing
     * @version V1.0 2021/5/21 11:57 上午
     */
    SimpleProjectVO convert2SimpleProjectVO(PoemsProjectVO project);

    /**
     * PoemsProjectVO列表转换为SimpleProjectVO列表
     *
     * @param projects PoemsProjectVO列表
     * @return SimpleProjectVO列表
     * @author lixing
     * @version V1.0 2021/5/21 11:57 上午
     */
    List<SimpleProjectVO> convert2SimpleProjectVOList(List<PoemsProjectVO> projects);

    /**
     * FilterPartitionVO 转换为 PartitionVO
     *
     * @param filterPartitionVO FilterPartitionVO
     * @return PartitionVO
     * @author lixing
     * @version V1.0 2021/5/21 6:13 下午
     */
    PartitionVO convert2PartitionVO(FilterPartitionVO filterPartitionVO);

    /**
     * FilterPartitionVO列表 转换为 PartitionVO列表
     *
     * @param filterPartitionVOList FilterPartitionVO列表
     * @return PartitionVO列表
     * @author lixing
     * @version V1.0 2021/5/21 6:13 下午
     */
    List<PartitionVO> convert2PartitionVOList(List<FilterPartitionVO> filterPartitionVOList);
}

