package com.persagy.apm.energy.report.saasweb.service;

import com.persagy.apm.energy.report.saasweb.model.vo.*;
import io.swagger.annotations.Api;

import java.util.Date;
import java.util.List;

/**
 * @author lixing
 * @version V1.0 2021/5/19 3:59 下午
 **/
@Api(value = "运维平台service接口")
public interface ISaasWebService {

    /**
     * 获取项目基础信息
     *
     * @param projectId 项目id
     * @return 项目信息
     * @author lixing
     * @version V1.0 2021/5/19 4:04 下午
     */
    SimpleProjectVO getSimpleProjectInfo(String projectId);

    /**
     * 获取项目所在气候区
     *
     * @param projectId 项目id
     * @return 气候区编码
     * @author lixing
     * @version V1.0 2021/6/10 6:10 下午
     */
    String getProjectClimateZone(String projectId);

    /**
     * 获取项目信息
     *
     * @param projectId 项目id
     * @return 项目信息
     * @author lixing
     * @version V1.0 2021/5/19 4:04 下午
     */
    ReportProjectVO getReportProjectInfo(String projectId);

    /**
     * 获取一组项目的总的商业面积
     *
     * @param projectIds 一组项目id
     * @return 这组项目总的商业面积
     * @author lixing
     * @version V1.0 2021/5/30 11:28 下午
     */
    Double getTotalCommercialArea(List<String> projectIds);

    /**
     * 获取一组项目的总的公区面积
     *
     * @param projectIds 一组项目id
     * @return 这组项目总的建筑面积
     * @author lixing
     * @version V1.0 2021/5/30 11:28 下午
     */
    Double getTotalPublicArea(List<String> projectIds);

    /**
     * 获取一组项目的总的建筑面积
     *
     * @param projectIds 一组项目id
     * @return 这组项目总的建筑面积
     * @author lixing
     * @version V1.0 2021/5/30 11:28 下午
     */
    Double getTotalBuildingArea(List<String> projectIds);

    /**
     * 获取区域可同比项目信息
     *
     * @param areaCode     区域编码
     * @param buildingType 建筑类型
     * @param reportMonth  报告月份
     * @return 区域可同比项目信息
     * @author lixing
     * @version V1.0 2021/5/30 11:28 下午
     */
    ReportAreaVO getCompareReportAreaInfo(String areaCode, String buildingType, Date reportMonth);

    /**
     * 获取区域信息
     *
     * @param areaCode     区域编码
     * @param buildingType 建筑类型
     * @return 区域信息
     * @author lixing
     * @version V1.0 2021/5/19 4:04 下午
     */
    ReportAreaVO getReportAreaInfo(String areaCode, String buildingType);

    /**
     * 获取区域的基本信息
     *
     * @param areaCode 区域编码
     * @return 区域的基本信息
     * @author lixing
     * @version V1.0 2021/5/31 6:04 下午
     */
    ReportAreaVO getReportAreaBaseInfo(String areaCode);

    /**
     * 获取分区下所有的项目
     *
     * @param areaCode 分区编码
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/24 5:37 下午
     */
    List<PoemsProjectVO> getProjectsByArea(String areaCode);

    /**
     * 获取集团下所有的项目
     *
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/24 5:37 下午
     */
    List<PoemsProjectVO> getAllProjects();

    /**
     * 根据气候带和建筑类型获取项目
     *
     * @param climateZoneCode  气候区编码
     * @param buildingTypeCode 业态编码
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/24 5:37 下午
     */
    List<PoemsProjectVO> getProjectsByClimateAndBuildingType(String climateZoneCode, String buildingTypeCode);

    /**
     * 根据区域和业态过滤项目
     *
     * @param areaCode     区域编码
     * @param buildingType 业态
     * @return 项目列表
     * @author lixing
     * @version V1.0 2021/5/24 6:01 下午
     */
    List<PoemsProjectVO> getProjectsByAreaAndBuildingType(String areaCode, String buildingType);

    /**
     * 根据业态获取用户可选的项目
     *
     * @param buildingType 项目业态
     * @return 用户可选的项目列表
     * @author lixing
     * @version V1.0 2021/5/21 10:53 上午
     */
    List<SimpleProjectVO> getAvailableProjects(String buildingType);

    /**
     * 级联获取所有关联的业态
     *
     * @param buildingType 查询条件
     * @return 所有关联的业态（包括自己）
     * @author lixing
     * @version V1.0 2021/5/24 6:04 下午
     */
    List<String> getRelatedBuildingTypes(String buildingType);

    /**
     * 根据业态获取用户可选的分区树
     *
     * @param buildingType 项目业态
     * @return 用户可选的分区树
     * @author lixing
     * @version V1.0 2021/5/21 10:53 上午
     */
    List<PartitionVO> getAvailableAreas(String buildingType);

    /**
     * 获取可同比项目（开业时间在指定月份的上一年1月1日之前）
     *
     * @param projectIds 要筛选的项目集
     * @param month      月份
     * @return 可同比项目id列表
     * @author lixing
     * @version V1.0 2021/5/31 10:49 上午
     */
    List<String> getComparableProjects(List<String> projectIds, Date month);
}
