package com.persagy.apm.energy.report.workorder.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.workorder.model.WorkOrder;
import com.persagy.apm.energy.report.workorder.repository.WorkOrderRepository;
import com.persagy.apm.energy.report.workorder.service.IWorkOrderWebService;
import org.apache.commons.lang.StringUtils;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.elasticsearch.core.query.NativeSearchQuery;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;


@Service
public class IWorkOrderWebServiceImpl implements IWorkOrderWebService {

    @Autowired
    private WorkOrderRepository workOrderRepository;

    @Override
    public List<WorkOrder> getWorkOrderNumByParam(String projectId, Date reportDate, String timeType, String orderType, String orderState) {
        try {
            String startTime = DateUtils.date2Str(reportDate, DateUtils.SDFSECOND);
            String endTime = DateUtils.date2Str(DateUtils.getMonthOff(reportDate, 1), DateUtils.SDFSECOND);
            BoolQueryBuilder queryOrderNumBuilder = createQueryParam(projectId, timeType, startTime, endTime, orderType, orderState);
            NativeSearchQueryBuilder nativeSearchQueryBuilder = new NativeSearchQueryBuilder();
            nativeSearchQueryBuilder.withQuery(queryOrderNumBuilder);
            NativeSearchQuery query = nativeSearchQueryBuilder.build();
            Page<WorkOrder> resultList = workOrderRepository.search(query);
            if (null != resultList && resultList.getSize() != 0) {
                return  resultList.getContent();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    private BoolQueryBuilder createQueryParam(String pjId, String timeType, String startTime, String endTime, String orderType, String orderState) {
        BoolQueryBuilder queryBuilder = QueryBuilders.boolQuery();
        queryBuilder.must(QueryBuilders.matchQuery("project_id", pjId));
        //时间查询
        if (StringUtils.isNotBlank(timeType)) {
            if (StringUtils.isNotBlank(startTime)) {
                if ("create_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("create_time").gte(startTime));
                } else if ("ask_end_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("ask_end_time").gte(startTime));
                } else if ("real_finish_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("real_finish_time").gte(startTime));
                }
            }
            if (StringUtils.isNotBlank(endTime)) {
                if ("create_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("create_time").lt(endTime));
                } else if ("ask_end_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("ask_end_time").lt(endTime));
                } else if ("real_finish_time".equals(timeType)) {
                    queryBuilder.must(QueryBuilders.rangeQuery("real_finish_time").lt(endTime));
                }
            }
        }
        if (StringUtils.isNotEmpty(orderType)) {
            List<String> orderTypeList = JSONObject.parseArray(orderType, String.class);
            queryBuilder.must(QueryBuilders.termsQuery("order_type.keyword", orderTypeList));
        }

        if (StringUtils.isNotEmpty(orderState)) {
            queryBuilder.must(QueryBuilders.matchQuery("order_state", orderState));
        }

        return queryBuilder;
    }


}
