package com.persagy.apm.energy.report.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energy.report.common.dto.QueryPlatformParamDTO;
import com.persagy.apm.energy.report.common.service.CommonService;
import com.persagy.apm.energy.report.common.service.PlatformService;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.common.utils.HttpUtils;
import com.persagy.apm.energy.report.common.vo.PlatformAreaInfoVO;
import com.persagy.apm.energy.report.monthly.config.statisticsitems.model.StatisticItems;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service("zongZhiRateDataAnalysisWebService")
@Slf4j
public class ZongZhiDataAnalysisPlatformServiceImpl implements PlatformService {

    @Value("${iot.data.analysis.url}")
    private String iotDataAnalysisUrl;

    @Resource
    private CommonService commonService;

    public final static String SUCCESS = "success";

    @Override
    public List<Double> getPjPlatformRate(QueryPlatformParamDTO queryPlatformParamDTO) {
        if (null == queryPlatformParamDTO || org.springframework.util.CollectionUtils.isEmpty(queryPlatformParamDTO.getProjectIdList()) || null == queryPlatformParamDTO.getReportDate()) {
            return new ArrayList<>();
        }
        String pjId = queryPlatformParamDTO.getProjectIdList().get(0);
        TreeMap<Date, Double> date_rateMap = new TreeMap<>();
        Date reportDate = queryPlatformParamDTO.getReportDate();
        Date startDate = DateUtils.getFirstDayOfYear(reportDate);
        while (startDate.before(DateUtils.getMonthOff(reportDate, 1))) {
            date_rateMap.put(startDate, getZongZhiRate(queryPlatformParamDTO.getUserId(), queryPlatformParamDTO.getGroupCode(), pjId, startDate));
            startDate = DateUtils.getMonthOff(startDate, 1);
        }
        List<Double> monthlySummaries = date_rateMap.values().stream().collect(Collectors.toList());
        return monthlySummaries;
    }

    private Double getZongZhiRate(String userId, String groupCode, String projectId, Date reportDate) {
        Double rate = null;
        try {
            JSONObject paramObject = new JSONObject();
            paramObject.put("userId", userId);
            paramObject.put("groupCode", groupCode);
            List<String> pjIdList = new ArrayList<>();
            pjIdList.add(projectId);
            paramObject.put("projectIdList", pjIdList);
            Date startTime = DateUtils.getStartTimeOfDay(reportDate);
            Date endTime = DateUtils.getMonthOff(startTime, 1);
            paramObject.put("alarmStartTime", startTime);
            paramObject.put("alarmEndTime", endTime);
            String url = iotDataAnalysisUrl + "/alarm/show/view/queryList";
            String response = HttpUtils.postJson(url, paramObject.toString());
            JSONObject responseObject = JSONObject.parseObject(response);
            if (SUCCESS.equals(responseObject.getString("respMsg"))) {
                List<Map> content = (List<Map>) responseObject.get("content");
                if (CollectionUtils.isNotEmpty(content)) {
                    Map data = content.get(0);
                    rate = DataUtils.parseDouble(data.get("zongZhiRate"));
                }
            }
        } catch (Exception e) {
            log.error("获取数据质量服务总支不平衡数据异常", e);
        }
        return rate;
    }

    @Override
    public PlatformAreaInfoVO getAreaPlatformRate(StatisticItems statisticItems, QueryPlatformParamDTO queryPlatformParamDTO) {
        if (null == queryPlatformParamDTO || org.springframework.util.CollectionUtils.isEmpty(queryPlatformParamDTO.getProjectIdList()) || null == queryPlatformParamDTO.getReportDate()) {
            return new PlatformAreaInfoVO();
        }
        // 合格标准
        String standardValueStr = statisticItems.getStandardValue();
        Double standardValue = DataUtils.parseDouble(standardValueStr);
        // 校验规则
        String qualifyFormula = statisticItems.getQualifyFormula();
        // 达标数量
        Integer satisfiedCount = 0;
        // 未达标数量
        Integer unsatisfiedCount = 0;
        // 未达标项目id列表
        List<String> unsatisfiedPjIdList = new ArrayList<>();
        for (String pjId : queryPlatformParamDTO.getProjectIdList()) {
            Double zongZhiRate = getZongZhiRate(queryPlatformParamDTO.getUserId(), queryPlatformParamDTO.getGroupCode(), pjId, queryPlatformParamDTO.getReportDate());
            if (null != zongZhiRate && null != standardValue) {
                if (DataUtils.getQualifyResult(standardValue, qualifyFormula, zongZhiRate) == 0.0) {
                    unsatisfiedCount++;
                    unsatisfiedPjIdList.add(pjId);
                } else {
                    satisfiedCount++;
                }
            }
        }
        // 根据未达标项目id列表获取不达标项目名称
        List<String> unsatisfiedPjNameList = commonService.getPjNameByPjId(unsatisfiedPjIdList);
        PlatformAreaInfoVO platformAreaInfoVO = new PlatformAreaInfoVO();
        platformAreaInfoVO.setSatisfiedCount(satisfiedCount);
        platformAreaInfoVO.setUnsatisfiedCount(unsatisfiedCount);
        platformAreaInfoVO.setUnsatisfiedProjects(unsatisfiedPjNameList);
        return platformAreaInfoVO;
    }
}
