package com.persagy.apm.energy.report.emsweather.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.common.utils.HttpUtils;
import com.persagy.apm.energy.report.emsweather.service.EMSWeatherWebService;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class EMSWeatherWebServiceImpl implements EMSWeatherWebService {

    @Value("${ems.weather.url}")
    private String emsWeatherUrl;

    @Override
    public Double getPjMonthAvgTemp(String projectId, Date reportDate) {
        Double result = null;
        try {
            Date startDate = DateUtils.getStartTimeOfDay(reportDate);
            String startTime = DateUtils.date2Str(startDate, DateUtils.SDF_SECOND);
            String endTime = DateUtils.date2Str(DateUtils.getMonthOff(startDate, 1), DateUtils.SDF_SECOND);
            JSONObject paramObject = new JSONObject();
            paramObject.put("projectId", projectId);
            paramObject.put("startTime", startTime);
            paramObject.put("endTime", endTime);
            paramObject.put("dataType", "All");
            String url = emsWeatherUrl + "/Spring/MVC/entrance/unifierJson/DayStaticData";
            String response = HttpUtils.postJson(url, paramObject.toString());
            JSONObject responseObject = JSONObject.parseObject(response);
            if (DataConstants.SUCCESS.equals(responseObject.getString("result"))) {
                List<Map> content = (List<Map>) responseObject.get("content");
                if (CollectionUtils.isNotEmpty(content)) {
                    Integer count = 0;
                    Double allAvgTemp = 0.0;
                    for (Map map : content) {
                        Double avgTemp = DataUtils.parseDouble(map.get("avgTemp"));
                        if (null == avgTemp) {
                            continue;
                        }
                        allAvgTemp = allAvgTemp + avgTemp;
                        count++;
                    }
                    if (count != 0) {
                        result = allAvgTemp / count;
                        return result;
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }
}
