package com.persagy.apm.energy.report.iotdataanalysis.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energy.report.common.DataConstants;
import com.persagy.apm.energy.report.common.utils.DataUtils;
import com.persagy.apm.energy.report.common.utils.DateUtils;
import com.persagy.apm.energy.report.common.utils.HttpUtils;
import com.persagy.apm.energy.report.iotdataanalysis.model.AlarmInfoVo;
import com.persagy.apm.energy.report.iotdataanalysis.model.CountAndDayRateDataVo;
import com.persagy.apm.energy.report.iotdataanalysis.service.IDataAnalysisWebService;
import com.persagy.apm.energy.report.monthly.detail.hotel.project.model.vo.HotelProjectMaintenanceBaseInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

@Service
@Slf4j
public class IDataAnalysisWebServiceImpl implements IDataAnalysisWebService {

    @Value("${iot.data.analysis.url}")
    private String iotDataAnalysisUrl;

    @Value("${iot.data.analysis.alarmtype}")
    private String alarmTypeStr;

    @Override
    public String getTargetCode(String userId, String groupCode, String functionItemId) {
        try {
            JSONObject paramObject = new JSONObject();
            paramObject.put("userId", userId);
            paramObject.put("groupCode", groupCode);
            String url = iotDataAnalysisUrl + "/setting/alarm/show/queryAll";
            String response = HttpUtils.postJson(url, paramObject.toString());
            JSONObject responseObject = JSONObject.parseObject(response);
            if (DataConstants.SUCCESS.equals(responseObject.getString("respMsg"))) {
                List<Map> content = (List<Map>) responseObject.get("content");
                if (CollectionUtils.isNotEmpty(content)) {
                    for (Map map : content) {
                        String targetCode = (String) map.get("targetCode");
                        if (targetCode.contains(functionItemId)) {
                            return targetCode;
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("获取数据质量基本配置信息异常:", e);
        }
        return null;
    }

    @Override
    public CountAndDayRateDataVo getIotDataAnalysisData(String userId, String groupCode, String projectId, Date reportDate, String targetCode) {
        CountAndDayRateDataVo vo = new CountAndDayRateDataVo();
        try {
            JSONObject paramObject = new JSONObject();
            paramObject.put("userId", userId);
            paramObject.put("groupCode", groupCode);
            paramObject.put("projectId", projectId);
            Date startTime = DateUtils.getStartTimeOfDay(reportDate);
            Date endTime = DateUtils.getMonthOff(startTime, 1);
            paramObject.put("alarmStartTime", startTime);
            paramObject.put("alarmEndTime", endTime);
            paramObject.put("targetCode", targetCode);
            String url = iotDataAnalysisUrl + "/alarm/show/detail/queryList";
            String response = HttpUtils.postJson(url, paramObject.toString());
            JSONObject responseObject = JSONObject.parseObject(response);
            if (DataConstants.SUCCESS.equals(responseObject.getString("respMsg"))) {
                Map data = (Map) responseObject.get("content");
                if (null != data) {
                    vo.setCountRate(DataUtils.parseDouble(data.get("rate")));
                    List<Map> dataList = (List<Map>) data.get("list");
                    TreeMap<Date, Double> dataMap = new TreeMap<>();
                    if (CollectionUtils.isNotEmpty(dataList)) {
                        for (Map map : dataList) {
                            Double rate = DataUtils.parseDouble(map.get("rate"));
                            long dataLong = (long) map.get("dataTime");
                            Date date = new Date(dataLong);
                            dataMap.put(date, rate);
                        }
                    }
                    vo.setDataMap(dataMap);
                }
            }
        } catch (Exception e) {
            log.error("获取数据质量看板信息异常:", e);
        }
        return vo;
    }

    @Override
    public List<AlarmInfoVo> getAlarmInfoVo(String groupCode, List<String> projectIdList, Date reportDate) {
        List<AlarmInfoVo> resultList = new ArrayList<>();
        try {
            JSONObject paramObject = new JSONObject();
            paramObject.put("groupCode", groupCode);
            paramObject.put("projectIds", projectIdList);
            Date startTime = DateUtils.getStartTimeOfDay(reportDate);
            Date endTime = DateUtils.getMonthOff(startTime, 1);
            paramObject.put("alarmStartTime", startTime.getTime());
            paramObject.put("alarmEndTime", endTime.getTime());
            List<String> alarmTypeList = JSONObject.parseArray(alarmTypeStr, String.class);
            paramObject.put("alarmType", alarmTypeList);
            paramObject.put("alarmStartTimeDesc", true);
            String url = iotDataAnalysisUrl + "/alarm/alarmList";
            String response = HttpUtils.postJson(url, paramObject.toString());
            JSONObject responseObject = JSONObject.parseObject(response);
            if (DataConstants.SUCCESS.equals(responseObject.getString("respMsg"))) {
                List<Map> content = (List<Map>) responseObject.get("content");
                if (CollectionUtils.isNotEmpty(content)) {
                    Map<String, Integer> pjId_alarmCountMap = new HashMap<>();
                    Map<String, Double> pjId_alarmDurationMap = new HashMap<>();
                    for (Map map : content) {
                        String pjId = (String) map.get("projectId");
                        Long alarmBeginTime = (Long) map.get("alarmStartTime");
                        if (StringUtils.isEmpty(pjId) || null == alarmBeginTime) {
                            continue;
                        }
                        Long alarmEndTime = (Long) map.get("alarmEndTime");
                        if (null == alarmEndTime) {
                            alarmEndTime = endTime.getTime();
                        }
                        Double duration = DateUtils.getDateDuration(alarmBeginTime, alarmEndTime);
                        Integer alarmCount = pjId_alarmCountMap.get(pjId);
                        if (null == alarmCount) {
                            alarmCount = 0;
                        }
                        alarmCount++;
                        pjId_alarmCountMap.put(pjId, alarmCount);
                        Double alarmDuration = pjId_alarmDurationMap.get(pjId);
                        if (null == alarmDuration) {
                            alarmDuration = 0.0;
                        }
                        alarmDuration = alarmDuration + duration;
                        pjId_alarmDurationMap.put(pjId, alarmDuration);
                    }
                    if (pjId_alarmCountMap.size() != 0) {
                        for (Map.Entry<String, Integer> entry : pjId_alarmCountMap.entrySet()) {
                            AlarmInfoVo alarmInfoVo = new AlarmInfoVo();
                            alarmInfoVo.setProjectId(entry.getKey());
                            alarmInfoVo.setAlarmCount(entry.getValue());
                            alarmInfoVo.setTotalDuration(pjId_alarmDurationMap.get(entry.getKey()));
                            resultList.add(alarmInfoVo);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("获取数据质量报警数据异常:", e);
        }
        return resultList;
    }

    @Override
    public HotelProjectMaintenanceBaseInfo getHotelProjectMaintenanceBaseInfo(String userId, String groupCode, String projectId, Date reportDate) {
        HotelProjectMaintenanceBaseInfo hotelProjectMaintenanceBaseInfo = new HotelProjectMaintenanceBaseInfo();
        try {
            JSONObject paramObject = new JSONObject();
            paramObject.put("userId", userId);
            paramObject.put("groupCode", groupCode);
            List<String> pjIdList = new ArrayList<>();
            pjIdList.add(projectId);
            paramObject.put("projectIdList", pjIdList);
            Date startTime = DateUtils.getStartTimeOfDay(reportDate);
            Date endTime = DateUtils.getMonthOff(startTime, 1);
            paramObject.put("alarmStartTime", startTime);
            paramObject.put("alarmEndTime", endTime);
            String url = iotDataAnalysisUrl + "/alarm/show/view/queryList";
            String response = HttpUtils.postJson(url, paramObject.toString());
            JSONObject responseObject = JSONObject.parseObject(response);
            if (DataConstants.SUCCESS.equals(responseObject.getString("respMsg"))) {
                List<Map> content = (List<Map>) responseObject.get("content");
                if (CollectionUtils.isNotEmpty(content)) {
                    Map data = content.get(0);
                    if (null != data) {
                        hotelProjectMaintenanceBaseInfo.setImbalanceRate(DataUtils.parseDouble(data.get("zongZhiRate")));
                        hotelProjectMaintenanceBaseInfo.setChangeLossRate(DataUtils.parseDouble(data.get("bianSunRate")));
                    }
                }
            }
        } catch (Exception e) {
            log.error("获取数据质量服务变损数据异常", e);
        }
        return hotelProjectMaintenanceBaseInfo;
    }


}
